=== modified file '.testr.conf'
--- .testr.conf	2010-02-28 10:08:29 +0000
+++ .testr.conf	2010-12-20 11:57:14 +0000
@@ -1,3 +1,4 @@
 [DEFAULT]
-test_command=./bzr selftest --subunit $IDOPTION
+test_command=./bzr selftest --subunit $IDOPTION $LISTOPT
 test_id_option=--load-list $IDFILE
+test_list_option=--list

=== modified file 'bzr'
--- bzr	2010-11-26 17:59:08 +0000
+++ bzr	2011-01-14 05:34:20 +0000
@@ -23,7 +23,7 @@
 import warnings
 
 # update this on each release
-_script_version = (2, 3, 0)
+_script_version = (2, 4, 0)
 
 try:
     version_info = sys.version_info

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2010-12-02 16:24:54 +0000
+++ bzrlib/__init__.py	2011-01-15 19:43:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,10 +52,10 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (2, 3, 0, 'dev', 5)
+version_info = (2, 4, 0, 'dev', 1)
 
 # API compatibility version
-api_minimum_version = (2, 3, 0)
+api_minimum_version = (2, 4, 0)
 
 
 def _format_version_tuple(version_info):

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2010-12-14 04:20:17 +0000
+++ bzrlib/branch.py	2011-01-14 20:52:48 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1380,13 +1380,8 @@
         """Return the most suitable metadir for a checkout of this branch.
         Weaves are used if this branch's repository uses weaves.
         """
-        if isinstance(self.bzrdir, bzrdir.BzrDirPreSplitOut):
-            from bzrlib.repofmt import weaverepo
-            format = bzrdir.BzrDirMetaFormat1()
-            format.repository_format = weaverepo.RepositoryFormat7()
-        else:
-            format = self.repository.bzrdir.checkout_metadir()
-            format.set_branch_format(self._format)
+        format = self.repository.bzrdir.checkout_metadir()
+        format.set_branch_format(self._format)
         return format
 
     def create_clone_on_transport(self, to_transport, revision_id=None,
@@ -2057,7 +2052,7 @@
             raise NotImplementedError
         if found_repository is None:
             found_repository = a_bzrdir.open_repository()
-        return BzrBranch(_format=self,
+        return BzrBranchPreSplitOut(_format=self,
                          _control_files=a_bzrdir._control_files,
                          a_bzrdir=a_bzrdir,
                          name=name,
@@ -2669,7 +2664,7 @@
         result.target_branch = target
         result.old_revno, result.old_revid = target.last_revision_info()
         self.update_references(target)
-        if result.old_revid != self.last_revision():
+        if result.old_revid != stop_revision:
             # We assume that during 'push' this repository is closer than
             # the target.
             graph = self.repository.get_graph(target.repository)
@@ -2698,6 +2693,19 @@
                 mode=self.bzrdir._get_file_mode())
 
 
+class BzrBranchPreSplitOut(BzrBranch):
+
+    def _get_checkout_format(self):
+        """Return the most suitable metadir for a checkout of this branch.
+        Weaves are used if this branch's repository uses weaves.
+        """
+        from bzrlib.repofmt.weaverepo import RepositoryFormat7
+        from bzrlib.bzrdir import BzrDirMetaFormat1
+        format = BzrDirMetaFormat1()
+        format.repository_format = RepositoryFormat7()
+        return format
+
+
 class BzrBranch5(BzrBranch):
     """A format 5 branch. This supports new features over plain branches.
 
@@ -3453,7 +3461,8 @@
         if local and not bound_location:
             raise errors.LocalRequiresBoundBranch()
         master_branch = None
-        if not local and bound_location and self.source.user_url != bound_location:
+        source_is_master = (self.source.user_url == bound_location)
+        if not local and bound_location and not source_is_master:
             # not pulling from master, so we need to update master.
             master_branch = self.target.get_master_branch(possible_transports)
             master_branch.lock_write()
@@ -3465,7 +3474,8 @@
             return self._pull(overwrite,
                 stop_revision, _hook_master=master_branch,
                 run_hooks=run_hooks,
-                _override_hook_target=_override_hook_target)
+                _override_hook_target=_override_hook_target,
+                merge_tags_to_master=not source_is_master)
         finally:
             if master_branch:
                 master_branch.unlock()
@@ -3538,7 +3548,8 @@
 
     def _pull(self, overwrite=False, stop_revision=None,
              possible_transports=None, _hook_master=None, run_hooks=True,
-             _override_hook_target=None, local=False):
+             _override_hook_target=None, local=False,
+             merge_tags_to_master=True):
         """See Branch.pull.
 
         This function is the core worker, used by GenericInterBranch.pull to
@@ -3579,7 +3590,7 @@
             # so a tags implementation that versions tags can only 
             # pull in the most recent changes. -- JRV20090506
             result.tag_conflicts = self.source.tags.merge_to(self.target.tags,
-                overwrite)
+                overwrite, ignore_master=not merge_tags_to_master)
             result.new_revno, result.new_revid = self.target.last_revision_info()
             if _hook_master:
                 result.master_branch = _hook_master

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-01-14 05:14:04 +0000
+++ bzrlib/builtins.py	2011-01-19 18:40:15 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,8 +21,6 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import cStringIO
-import itertools
-import re
 import sys
 import time
 
@@ -330,7 +328,8 @@
         if revision_id is None and revision is None:
             raise errors.BzrCommandError('You must supply either'
                                          ' --revision or a revision_id')
-        b = WorkingTree.open_containing(directory)[0].branch
+
+        b = bzrdir.BzrDir.open_containing_tree_or_branch(directory)[1]
 
         revisions = b.repository.revisions
         if revisions is None:
@@ -2053,7 +2052,9 @@
     @display_command
     def run(self, null=False, directory=u'.'):
         tree = WorkingTree.open_containing(directory)[0]
+        self.add_cleanup(tree.lock_read().unlock)
         td = tree.changes_from(tree.basis_tree())
+        self.cleanup_now()
         for path, id, kind, text_modified, meta_modified in td.modified:
             if null:
                 self.outf.write(path + '\0')
@@ -3329,27 +3330,62 @@
 
 
 class cmd_upgrade(Command):
-    __doc__ = """Upgrade branch storage to current format.
-
-    The check command or bzr developers may sometimes advise you to run
-    this command. When the default format has changed you may also be warned
-    during other operations to upgrade.
+    __doc__ = """Upgrade a repository, branch or working tree to a newer format.
+
+    When the default format has changed after a major new release of
+    Bazaar, you may be informed during certain operations that you
+    should upgrade. Upgrading to a newer format may improve performance
+    or make new features available. It may however limit interoperability
+    with older repositories or with older versions of Bazaar.
+
+    If you wish to upgrade to a particular format rather than the
+    current default, that can be specified using the --format option.
+    As a consequence, you can use the upgrade command this way to
+    "downgrade" to an earlier format, though some conversions are
+    a one way process (e.g. changing from the 1.x default to the
+    2.x default) so downgrading is not always possible.
+
+    A backup.bzr.~#~ directory is created at the start of the conversion
+    process (where # is a number). By default, this is left there on
+    completion. If the conversion fails, delete the new .bzr directory
+    and rename this one back in its place. Use the --clean option to ask
+    for the backup.bzr directory to be removed on successful conversion.
+    Alternatively, you can delete it by hand if everything looks good
+    afterwards.
+
+    If the location given is a shared repository, dependent branches
+    are also converted provided the repository converts successfully.
+    If the conversion of a branch fails, remaining branches are still
+    tried.
+
+    For more information on upgrades, see the Bazaar Upgrade Guide,
+    http://doc.bazaar.canonical.com/latest/en/upgrade-guide/.
     """
 
-    _see_also = ['check']
+    _see_also = ['check', 'reconcile', 'formats']
     takes_args = ['url?']
     takes_options = [
-                    RegistryOption('format',
-                        help='Upgrade to a specific format.  See "bzr help'
-                             ' formats" for details.',
-                        lazy_registry=('bzrlib.bzrdir', 'format_registry'),
-                        converter=lambda name: bzrdir.format_registry.make_bzrdir(name),
-                        value_switches=True, title='Branch format'),
-                    ]
+        RegistryOption('format',
+            help='Upgrade to a specific format.  See "bzr help'
+                 ' formats" for details.',
+            lazy_registry=('bzrlib.bzrdir', 'format_registry'),
+            converter=lambda name: bzrdir.format_registry.make_bzrdir(name),
+            value_switches=True, title='Branch format'),
+        Option('clean',
+            help='Remove the backup.bzr directory if successful.'),
+        Option('dry-run',
+            help="Show what would be done, but don't actually do anything."),
+    ]
 
-    def run(self, url='.', format=None):
+    def run(self, url='.', format=None, clean=False, dry_run=False):
         from bzrlib.upgrade import upgrade
-        upgrade(url, format)
+        exceptions = upgrade(url, format, clean_up=clean, dry_run=dry_run)
+        if exceptions:
+            if len(exceptions) == 1:
+                # Compatibility with historical behavior
+                raise exceptions[0]
+            else:
+                return 3
 
 
 class cmd_whoami(Command):
@@ -3391,6 +3427,10 @@
                 self.outf.write(c.username() + '\n')
             return
 
+        if email:
+            raise errors.BzrCommandError("--email can only be used to display existing "
+                                         "identity")
+
         # display a warning if an email address isn't included in the given name.
         try:
             _mod_config.extract_email_address(name)
@@ -5448,24 +5488,17 @@
     takes_options = [
         custom_help('directory',
             help='Branch whose tags should be displayed.'),
-        RegistryOption.from_kwargs('sort',
+        RegistryOption('sort',
             'Sort tags by different criteria.', title='Sorting',
-            natural='Sort numeric substrings as numbers:'
-                    ' suitable for version numbers. (default)',
-            alpha='Sort tags lexicographically.',
-            time='Sort tags chronologically.',
+            lazy_registry=('bzrlib.tag', 'tag_sort_methods')
             ),
         'show-ids',
         'revision',
     ]
 
     @display_command
-    def run(self,
-            directory='.',
-            sort='natural',
-            show_ids=False,
-            revision=None,
-            ):
+    def run(self, directory='.', sort=None, show_ids=False, revision=None):
+        from bzrlib.tag import tag_sort_methods
         branch, relpath = Branch.open_containing(directory)
 
         tags = branch.tags.get_tag_dict().items()
@@ -5480,25 +5513,9 @@
             # only show revisions between revid1 and revid2 (inclusive)
             tags = [(tag, revid) for tag, revid in tags if
                 graph.is_between(revid, revid1, revid2)]
-        if sort == 'natural':
-            def natural_sort_key(tag):
-                return [f(s) for f,s in 
-                        zip(itertools.cycle((unicode.lower,int)),
-                                            re.split('([0-9]+)', tag[0]))]
-            tags.sort(key=natural_sort_key)
-        elif sort == 'alpha':
-            tags.sort()
-        elif sort == 'time':
-            timestamps = {}
-            for tag, revid in tags:
-                try:
-                    revobj = branch.repository.get_revision(revid)
-                except errors.NoSuchRevision:
-                    timestamp = sys.maxint # place them at the end
-                else:
-                    timestamp = revobj.timestamp
-                timestamps[revid] = timestamp
-            tags.sort(key=lambda x: timestamps[x[1]])
+        if sort is None:
+            sort = tag_sort_methods.get()
+        sort(branch, tags)
         if not show_ids:
             # [ (tag, revid), ... ] -> [ (tag, dotted_revno), ... ]
             for index, (tag, revid) in enumerate(tags):

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2010-12-14 23:14:44 +0000
+++ bzrlib/bzrdir.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -927,7 +927,6 @@
         return format.initialize_on_transport(t)
 
 
-
 class BzrDirHooks(hooks.Hooks):
     """Hooks for BzrDir operations."""
 

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2010-04-30 11:35:43 +0000
+++ bzrlib/check.py	2011-01-13 00:14:31 +0000
@@ -46,13 +46,15 @@
   indicating that the revision was found/not found.
 """
 
-from bzrlib import errors
+from bzrlib import (
+    errors,
+    ui,
+    )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.revision import NULL_REVISION
 from bzrlib.symbol_versioning import deprecated_function, deprecated_in
 from bzrlib.trace import note
-import bzrlib.ui
 from bzrlib.workingtree import WorkingTree
 
 class Check(object):
@@ -88,7 +90,7 @@
         if callback_refs is None:
             callback_refs = {}
         self.repository.lock_read()
-        self.progress = bzrlib.ui.ui_factory.nested_progress_bar()
+        self.progress = ui.ui_factory.nested_progress_bar()
         try:
             self.progress.update('check', 0, 4)
             if self.check_repo:
@@ -287,7 +289,7 @@
         """Check all the weaves we can get our hands on.
         """
         weave_ids = []
-        storebar = bzrlib.ui.ui_factory.nested_progress_bar()
+        storebar = ui.ui_factory.nested_progress_bar()
         try:
             self._check_weaves(storebar)
         finally:

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2010-11-10 02:01:33 +0000
+++ bzrlib/commit.py	2011-01-13 02:43:21 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -55,6 +55,7 @@
     revision,
     trace,
     tree,
+    ui,
     )
 from bzrlib.branch import Branch
 from bzrlib.cleanup import OperationWithCleanups
@@ -72,7 +73,6 @@
 from bzrlib.inventory import Inventory, InventoryEntry, make_entry
 from bzrlib import symbol_versioning
 from bzrlib.urlutils import unescape_for_display
-import bzrlib.ui
 
 
 class NullCommitReporter(object):
@@ -346,7 +346,7 @@
             not self.branch.repository._format.supports_tree_reference and
             (self.branch.repository._format.fast_deltas or
              len(self.parents) < 2))
-        self.pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        self.pb = ui.ui_factory.nested_progress_bar()
         operation.add_cleanup(self.pb.finished)
         self.basis_revid = self.work_tree.last_revision()
         self.basis_tree = self.work_tree.basis_tree()

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2010-11-29 01:23:53 +0000
+++ bzrlib/config.py	2011-01-12 18:12:58 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #            and others
 #
@@ -1131,10 +1131,16 @@
     """
     base = os.environ.get('BZR_HOME', None)
     if sys.platform == 'win32':
+        # environ variables on Windows are in user encoding/mbcs. So decode
+        # before using one
+        if base is not None:
+            base = base.decode('mbcs')
         if base is None:
             base = win32utils.get_appdata_location_unicode()
         if base is None:
             base = os.environ.get('HOME', None)
+            if base is not None:
+                base = base.decode('mbcs')
         if base is None:
             raise errors.BzrError('You must have one of BZR_HOME, APPDATA,'
                                   ' or HOME set')

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2010-11-10 16:24:31 +0000
+++ bzrlib/conflicts.py	2011-01-15 17:29:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -514,7 +514,7 @@
         # Adjust the path for the retained file id
         tid = tt.trans_id_file_id(file_id)
         parent_tid = tt.get_tree_parent(tid)
-        tt.adjust_path(path, parent_tid, tid)
+        tt.adjust_path(osutils.basename(path), parent_tid, tid)
         tt.apply()
 
     def _revision_tree(self, tree, revid):
@@ -600,7 +600,7 @@
         # 'item.suffix_to_remove' has been deleted, this is a no-op)
         this_tid = tt.trans_id_file_id(self.file_id)
         parent_tid = tt.get_tree_parent(this_tid)
-        tt.adjust_path(self.path, parent_tid, this_tid)
+        tt.adjust_path(osutils.basename(self.path), parent_tid, this_tid)
         tt.apply()
 
     def action_take_this(self, tree):

=== modified file 'bzrlib/controldir.py'
--- bzrlib/controldir.py	2010-12-14 00:41:05 +0000
+++ bzrlib/controldir.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/crash.py'
--- bzrlib/crash.py	2010-09-28 18:51:47 +0000
+++ bzrlib/crash.py	2011-01-18 00:25:04 +0000
@@ -143,10 +143,11 @@
     exc_type, exc_object, exc_tb = exc_info
 
     pr = Report()
-    # add_proc_info gives you the memory map of the process, which is not so
-    # useful for Bazaar but does tell you what binary libraries are loaded.
-    # More importantly it sets the ExecutablePath, InterpreterPath, etc.
+    # add_proc_info sets the ExecutablePath, InterpreterPath, etc.
     pr.add_proc_info()
+    # It also adds ProcMaps which for us is rarely useful and mostly noise, so
+    # let's remove it.
+    del pr['ProcMaps']
     pr.add_user_info()
 
     # Package and SourcePackage are needed so that apport will report about even

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2010-12-16 20:41:47 +0000
+++ bzrlib/errors.py	2011-01-13 21:51:32 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -2023,7 +2023,7 @@
 
 class BzrBadParameterMissing(BzrBadParameter):
 
-    _fmt = "Parameter $(param)s is required but not present."
+    _fmt = "Parameter %(param)s is required but not present."
 
 
 class BzrBadParameterUnicode(BzrBadParameter):

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2010-12-14 09:21:14 +0000
+++ bzrlib/fetch.py	2011-01-14 17:18:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
+    graph,
     tsort,
     versionedfile,
     )
@@ -93,7 +94,8 @@
         try:
             pb.update("Finding revisions", 0, 2)
             search = self._revids_to_fetch()
-            if search is None:
+            mutter('fetching: %s', search)
+            if search.is_empty():
                 return
             pb.update("Fetching revisions", 1, 2)
             self._fetch_everything_for_search(search)
@@ -148,17 +150,33 @@
         """Determines the exact revisions needed from self.from_repository to
         install self._last_revision in self.to_repository.
 
-        If no revisions need to be fetched, then this just returns None.
+        :returns: A SearchResult of some sort.  (Possibly a
+        PendingAncestryResult, EmptySearchResult, etc.)
         """
-        if self._fetch_spec is not None:
+        mutter("self._fetch_spec, self._last_revision: %r, %r",
+                self._fetch_spec, self._last_revision)
+        get_search_result = getattr(self._fetch_spec, 'get_search_result', None)
+        if get_search_result is not None:
+            mutter(
+                'resolving fetch_spec into search result: %s', self._fetch_spec)
+            # This is EverythingNotInOther or a similar kind of fetch_spec.
+            # Turn it into a search result.
+            return get_search_result()
+        elif self._fetch_spec is not None:
+            # The fetch spec is already a concrete search result.
             return self._fetch_spec
-        mutter('fetch up to rev {%s}', self._last_revision)
-        if self._last_revision is NULL_REVISION:
+        elif self._last_revision == NULL_REVISION:
+            # fetch_spec is None + last_revision is null => empty fetch.
             # explicit limit of no revisions needed
-            return None
-        return self.to_repository.search_missing_revision_ids(
-            self.from_repository, self._last_revision,
-            find_ghosts=self.find_ghosts)
+            return graph.EmptySearchResult()
+        elif self._last_revision is not None:
+            return graph.NotInOtherForRevs(self.to_repository,
+                self.from_repository, [self._last_revision],
+                find_ghosts=self.find_ghosts).get_search_result()
+        else: # self._last_revision is None:
+            return graph.EverythingNotInOther(self.to_repository,
+                self.from_repository,
+                find_ghosts=self.find_ghosts).get_search_result()
 
 
 class Inter1and2Helper(object):

=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2010-12-11 22:06:52 +0000
+++ bzrlib/graph.py	2011-01-14 17:18:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1536,7 +1536,57 @@
             return revs, ghosts
 
 
-class SearchResult(object):
+class AbstractSearchResult(object):
+
+    def get_recipe(self):
+        """Return a recipe that can be used to replay this search.
+
+        The recipe allows reconstruction of the same results at a later date.
+
+        :return: A tuple of (search_kind_str, *details).  The details vary by
+            kind of search result.
+        """
+        raise NotImplementedError(self.get_recipe)
+
+    def get_network_struct(self):
+        """Return a tuple that can be transmitted via the HPSS protocol."""
+        raise NotImplementedError(self.get_network_struct)
+
+    def get_keys(self):
+        """Return the keys found in this search.
+
+        :return: A set of keys.
+        """
+        raise NotImplementedError(self.get_keys)
+
+    def is_empty(self):
+        """Return false if the search lists 1 or more revisions."""
+        raise NotImplementedError(self.is_empty)
+
+    def refine(self, seen, referenced):
+        """Create a new search by refining this search.
+
+        :param seen: Revisions that have been satisfied.
+        :param referenced: Revision references observed while satisfying some
+            of this search.
+        :return: A search result.
+        """
+        raise NotImplementedError(self.refine)
+
+
+class AbstractSearch(object):
+
+    def get_search_result(self):
+        """Construct a network-ready search result from this search description.
+
+        This may take some time to search repositories, etc.
+
+        :return: A search result.
+        """
+        raise NotImplementedError(self.get_search_result)
+
+
+class SearchResult(AbstractSearchResult):
     """The result of a breadth first search.
 
     A SearchResult provides the ability to reconstruct the search or access a
@@ -1557,6 +1607,19 @@
         self._recipe = ('search', start_keys, exclude_keys, key_count)
         self._keys = frozenset(keys)
 
+    def __repr__(self):
+        kind, start_keys, exclude_keys, key_count = self._recipe
+        if len(start_keys) > 5:
+            start_keys_repr = repr(list(start_keys)[:5])[:-1] + ', ...]'
+        else:
+            start_keys_repr = repr(start_keys)
+        if len(exclude_keys) > 5:
+            exclude_keys_repr = repr(list(exclude_keys)[:5])[:-1] + ', ...]'
+        else:
+            exclude_keys_repr = repr(exclude_keys)
+        return '<%s %s:(%s, %s, %d)>' % (self.__class__.__name__,
+            kind, start_keys_repr, exclude_keys_repr, key_count)
+
     def get_recipe(self):
         """Return a recipe that can be used to replay this search.
 
@@ -1580,6 +1643,12 @@
         """
         return self._recipe
 
+    def get_network_struct(self):
+        start_keys = ' '.join(self._recipe[1])
+        stop_keys = ' '.join(self._recipe[2])
+        count = str(self._recipe[3])
+        return (self._recipe[0], '\n'.join((start_keys, stop_keys, count)))
+
     def get_keys(self):
         """Return the keys found in this search.
 
@@ -1617,7 +1686,7 @@
         return SearchResult(pending_refs, exclude, count, keys)
 
 
-class PendingAncestryResult(object):
+class PendingAncestryResult(AbstractSearchResult):
     """A search result that will reconstruct the ancestry for some graph heads.
 
     Unlike SearchResult, this doesn't hold the complete search result in
@@ -1634,6 +1703,14 @@
         self.heads = frozenset(heads)
         self.repo = repo
 
+    def __repr__(self):
+        if len(self.heads) > 5:
+            heads_repr = repr(list(self.heads)[:5] + ', ...]')
+        else:
+            heads_repr = repr(self.heads)
+        return '<%s heads:%s repo:%r>' % (
+            self.__class__.__name__, heads_repr, self.repo)
+
     def get_recipe(self):
         """Return a recipe that can be used to replay this search.
 
@@ -1647,6 +1724,11 @@
         """
         return ('proxy-search', self.heads, set(), -1)
 
+    def get_network_struct(self):
+        parts = ['ancestry-of']
+        parts.extend(self.heads)
+        return parts
+
     def get_keys(self):
         """See SearchResult.get_keys.
 
@@ -1679,6 +1761,104 @@
         return PendingAncestryResult(referenced - seen, self.repo)
 
 
+class EmptySearchResult(AbstractSearchResult):
+    """An empty search result."""
+
+    def is_empty(self):
+        return True
+    
+
+class EverythingResult(AbstractSearchResult):
+    """A search result that simply requests everything in the repository."""
+
+    def __init__(self, repo):
+        self._repo = repo
+
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__, self._repo)
+
+    def get_recipe(self):
+        raise NotImplementedError(self.get_recipe)
+
+    def get_network_struct(self):
+        return ('everything',)
+
+    def get_keys(self):
+        if 'evil' in debug.debug_flags:
+            from bzrlib import remote
+            if isinstance(self._repo, remote.RemoteRepository):
+                # warn developers (not users) not to do this
+                trace.mutter_callsite(
+                    2, "EverythingResult(RemoteRepository).get_keys() is slow.")
+        return self._repo.all_revision_ids()
+
+    def is_empty(self):
+        # It's ok for this to wrongly return False: the worst that can happen
+        # is that RemoteStreamSource will initiate a get_stream on an empty
+        # repository.  And almost all repositories are non-empty.
+        return False
+
+    def refine(self, seen, referenced):
+        heads = set(self._repo.all_revision_ids())
+        heads.difference_update(seen)
+        heads.update(referenced)
+        return PendingAncestryResult(heads, self._repo)
+
+
+class EverythingNotInOther(AbstractSearch):
+    """Find all revisions in that are in one repo but not the other."""
+
+    def __init__(self, to_repo, from_repo, find_ghosts=False):
+        self.to_repo = to_repo
+        self.from_repo = from_repo
+        self.find_ghosts = find_ghosts
+
+    def get_search_result(self):
+        return self.to_repo.search_missing_revision_ids(
+            self.from_repo, find_ghosts=self.find_ghosts)
+
+
+class NotInOtherForRevs(AbstractSearch):
+    """Find all revisions missing in one repo for a some specific heads."""
+
+    def __init__(self, to_repo, from_repo, required_ids, if_present_ids=None,
+            find_ghosts=False):
+        """Constructor.
+
+        :param required_ids: revision IDs of heads that must be found, or else
+            the search will fail with NoSuchRevision.  All revisions in their
+            ancestry not already in the other repository will be included in
+            the search result.
+        :param if_present_ids: revision IDs of heads that may be absent in the
+            source repository.  If present, then their ancestry not already
+            found in other will be included in the search result.
+        """
+        self.to_repo = to_repo
+        self.from_repo = from_repo
+        self.find_ghosts = find_ghosts
+        self.required_ids = required_ids
+        self.if_present_ids = if_present_ids
+
+    def __repr__(self):
+        if len(self.required_ids) > 5:
+            reqd_revs_repr = repr(list(self.required_ids)[:5])[:-1] + ', ...]'
+        else:
+            reqd_revs_repr = repr(self.required_ids)
+        if self.if_present_ids and len(self.if_present_ids) > 5:
+            ifp_revs_repr = repr(list(self.if_present_ids)[:5])[:-1] + ', ...]'
+        else:
+            ifp_revs_repr = repr(self.if_present_ids)
+
+        return "<%s from:%r to:%r find_ghosts:%r req'd:%r if-present:%r>" % (
+            self.__class__.__name__, self.from_repo, self.to_repo,
+            self.find_ghosts, reqd_revs_repr, ifp_revs_repr)
+
+    def get_search_result(self):
+        return self.to_repo.search_missing_revision_ids(
+            self.from_repo, revision_ids=self.required_ids,
+            if_present_ids=self.if_present_ids, find_ghosts=self.find_ghosts)
+
+
 def collapse_linear_regions(parent_map):
     """Collapse regions of the graph that are 'linear'.
 

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2010-06-04 03:09:35 +0000
+++ bzrlib/knit.py	2011-01-11 20:42:36 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -54,6 +54,7 @@
 
 from cStringIO import StringIO
 from itertools import izip
+import gzip
 import operator
 import os
 import sys
@@ -1879,7 +1880,7 @@
         :return: the header and the decompressor stream.
                  as (stream, header_record)
         """
-        df = tuned_gzip.GzipFile(mode='rb', fileobj=StringIO(raw_data))
+        df = gzip.GzipFile(mode='rb', fileobj=StringIO(raw_data))
         try:
             # Current serialise
             rec = self._check_header(key, df.readline())
@@ -1894,7 +1895,7 @@
         # 4168 calls in 2880 217 internal
         # 4168 calls to _parse_record_header in 2121
         # 4168 calls to readlines in 330
-        df = tuned_gzip.GzipFile(mode='rb', fileobj=StringIO(data))
+        df = gzip.GzipFile(mode='rb', fileobj=StringIO(data))
         try:
             record_contents = df.readlines()
         except Exception, e:

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2010-12-02 10:41:05 +0000
+++ bzrlib/lockdir.py	2011-01-12 20:31:15 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -430,8 +430,8 @@
         # there's a small race window between checking it and doing the
         # rename.
         broken_info_path = tmpname + self.__INFO_NAME
-        f = self.transport.get(broken_info_path)
-        broken_lines = f.readlines()
+        broken_content = self.transport.get_bytes(broken_info_path)
+        broken_lines = osutils.split_lines(broken_content)
         if broken_lines != corrupt_info_lines:
             raise LockBreakMismatch(self, broken_lines, corrupt_info_lines)
         self.transport.delete(broken_info_path)

=== modified file 'bzrlib/multiparent.py'
--- bzrlib/multiparent.py	2010-08-11 01:31:11 +0000
+++ bzrlib/multiparent.py	2011-01-11 18:48:11 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
     )
 from bzrlib import bencode
 """)
-from bzrlib.tuned_gzip import GzipFile
+from gzip import GzipFile
 
 
 def topo_iter_keys(vf, keys=None):
@@ -564,7 +564,8 @@
         zip_file = GzipFile(None, mode='rb', fileobj=sio)
         try:
             file_version_id = zip_file.readline()
-            return MultiParent.from_patch(zip_file.read())
+            content = zip_file.read()
+            return MultiParent.from_patch(content)
         finally:
             zip_file.close()
 

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2010-12-16 20:41:47 +0000
+++ bzrlib/mutabletree.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2010-11-06 02:32:43 +0000
+++ bzrlib/osutils.py	2011-01-12 21:06:32 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1461,10 +1461,16 @@
     # a similar effect.
 
     # If BZR_COLUMNS is set, take it, user is always right
+    # Except if they specified 0 in which case, impose no limit here
     try:
-        return int(os.environ['BZR_COLUMNS'])
+        width = int(os.environ['BZR_COLUMNS'])
     except (KeyError, ValueError):
-        pass
+        width = None
+    if width is not None:
+        if width > 0:
+            return width
+        else:
+            return None
 
     isatty = getattr(sys.stdout, 'isatty', None)
     if isatty is None or not isatty():
@@ -1995,6 +2001,14 @@
 # data at once.
 MAX_SOCKET_CHUNK = 64 * 1024
 
+_end_of_stream_errors = [errno.ECONNRESET]
+for _eno in ['WSAECONNRESET', 'WSAECONNABORTED']:
+    _eno = getattr(errno, _eno, None)
+    if _eno is not None:
+        _end_of_stream_errors.append(_eno)
+del _eno
+
+
 def read_bytes_from_socket(sock, report_activity=None,
         max_read_size=MAX_SOCKET_CHUNK):
     """Read up to max_read_size of bytes from sock and notify of progress.
@@ -2008,7 +2022,7 @@
             bytes = sock.recv(max_read_size)
         except socket.error, e:
             eno = e.args[0]
-            if eno == getattr(errno, "WSAECONNRESET", errno.ECONNRESET):
+            if eno in _end_of_stream_errors:
                 # The connection was closed by the other side.  Callers expect
                 # an empty string to signal end-of-stream.
                 return ""
@@ -2376,3 +2390,16 @@
         counter += 1
         name = "%s.~%d~" % (base, counter)
     return name
+
+
+def set_fd_cloexec(fd):
+    """Set a Unix file descriptor's FD_CLOEXEC flag.  Do nothing if platform
+    support for this is not available.
+    """
+    try:
+        import fcntl
+        old = fcntl.fcntl(fd, fcntl.F_GETFD)
+        fcntl.fcntl(fd, fcntl.F_SETFD, old | fcntl.FD_CLOEXEC)
+    except (ImportError, AttributeError):
+        # Either the fcntl module or specific constants are not present
+        pass

=== modified file 'bzrlib/plugins/launchpad/__init__.py'
--- bzrlib/plugins/launchpad/__init__.py	2010-12-07 09:06:39 +0000
+++ bzrlib/plugins/launchpad/__init__.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/plugins/launchpad/lp_api.py'
--- bzrlib/plugins/launchpad/lp_api.py	2010-12-02 10:41:05 +0000
+++ bzrlib/plugins/launchpad/lp_api.py	2011-01-19 01:03:43 +0000
@@ -47,11 +47,12 @@
     STAGING_SERVICE_ROOT,
     Launchpad,
     )
-try:
-    from launchpadlib.uris import LPNET_SERVICE_ROOT
-except ImportError:
-    LPNET_SERVICE_ROOT = 'https://api.launchpad.net/beta/'
+from launchpadlib import uris
 
+LPNET_SERVICE_ROOT = getattr(uris, 'LPNET_SERVICE_ROOT',
+                             'https://api.launchpad.net/beta/')
+QASTAGING_SERVICE_ROOT = getattr(uris, 'QASTAGING_SERVICE_ROOT',
+                                 'https://api.qastaging.launchpad.net/')
 
 # Declare the minimum version of launchpadlib that we need in order to work.
 # 1.5.1 is the version of launchpadlib packaged in Ubuntu 9.10, the most
@@ -80,6 +81,7 @@
 
 LAUNCHPAD_API_URLS = {
     'production': LPNET_SERVICE_ROOT,
+    'qastaging': QASTAGING_SERVICE_ROOT,
     'staging': STAGING_SERVICE_ROOT,
     'dev': 'https://api.launchpad.dev/beta/',
     }
@@ -189,12 +191,13 @@
     @staticmethod
     def tweak_url(url, launchpad):
         """Adjust a URL to work with staging, if needed."""
-        if str(launchpad._root_uri) != STAGING_SERVICE_ROOT:
-            return url
-        if url is None:
-            return None
-        return url.replace('bazaar.launchpad.net',
-                           'bazaar.staging.launchpad.net')
+        if str(launchpad._root_uri) == STAGING_SERVICE_ROOT:
+            return url.replace('bazaar.launchpad.net',
+                               'bazaar.staging.launchpad.net')
+        elif str(launchpad._root_uri) == QASTAGING_SERVICE_ROOT:
+            return url.replace('bazaar.launchpad.net',
+                               'bazaar.qastaging.launchpad.net')
+        return url
 
     @classmethod
     def from_bzr(cls, launchpad, bzr_branch, create_missing=True):
@@ -227,17 +230,26 @@
             raise errors.BzrError('%s is not registered on Launchpad' % url)
         return lp_branch
 
-    def get_dev_focus(self):
-        """Return the 'LaunchpadBranch' for the dev focus of this one."""
+    def get_target(self):
+        """Return the 'LaunchpadBranch' for the target of this one."""
         lp_branch = self.lp
-        if lp_branch.project is None:
-            raise errors.BzrError('%s has no product.' %
-                                  lp_branch.bzr_identity)
-        dev_focus = lp_branch.project.development_focus.branch
-        if dev_focus is None:
-            raise errors.BzrError('%s has no development focus.' %
-                                  lp_branch.bzr_identity)
-        return LaunchpadBranch(dev_focus, dev_focus.bzr_identity)
+        if lp_branch.project is not None:
+            dev_focus = lp_branch.project.development_focus.branch
+            if dev_focus is None:
+                raise errors.BzrError('%s has no development focus.' %
+                                  lp_branch.bzr_identity)
+            target = dev_focus.branch
+            if target is None:
+                raise errors.BzrError('development focus %s has no branch.' % dev_focus)
+        elif lp_branch.sourcepackage is not None:
+            target = lp_branch.sourcepackage.getBranch(pocket="Release")
+            if target is None:
+                raise errors.BzrError('source package %s has no branch.' %
+                                      lp_branch.sourcepackage)
+        else:
+            raise errors.BzrError('%s has no associated product or source package.' %
+                                  lp_branch.bzr_identity)
+        return LaunchpadBranch(target, target.bzr_identity)
 
     def update_lp(self):
         """Update the Launchpad copy of this branch."""

=== modified file 'bzrlib/plugins/launchpad/lp_propose.py'
--- bzrlib/plugins/launchpad/lp_propose.py	2010-12-02 10:41:05 +0000
+++ bzrlib/plugins/launchpad/lp_propose.py	2011-01-19 16:57:38 +0000
@@ -14,21 +14,22 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
-import webbrowser
-
 from bzrlib import (
     errors,
     hooks,
+    )
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import webbrowser
+
+from bzrlib import (
     msgeditor,
 )
 from bzrlib.plugins.launchpad import (
     lp_api,
     lp_registration,
 )
-from bzrlib.plugins.launchpad.lp_api import canonical_url
-
-from lazr.restfulclient import errors as restful_errors
+""")
 
 
 class ProposeMergeHooks(hooks.Hooks):
@@ -80,17 +81,14 @@
         self.source_branch = lp_api.LaunchpadBranch.from_bzr(
             self.launchpad, source_branch)
         if target_branch is None:
-            self.target_branch = self.source_branch.get_dev_focus()
+            self.target_branch = self.source_branch.get_target()
         else:
             self.target_branch = lp_api.LaunchpadBranch.from_bzr(
                 self.launchpad, target_branch)
         self.commit_message = message
         # XXX: this is where bug lp:583638 could be tackled.
         if reviews == []:
-            target_reviewer = self.target_branch.lp.reviewer
-            if target_reviewer is None:
-                raise errors.BzrCommandError('No reviewer specified')
-            self.reviews = [(target_reviewer, '')]
+            self.reviews = []
         else:
             self.reviews = [(self.launchpad.people[reviewer], review_type)
                             for reviewer, review_type in
@@ -153,7 +151,7 @@
             if mp.target_branch.self_link == self.target_branch.lp.self_link:
                 raise errors.BzrCommandError(
                     'There is already a branch merge proposal: %s' %
-                    canonical_url(mp))
+                    lp_api.canonical_url(mp))
 
     def _get_prerequisite_branch(self):
         hooks = self.hooks['get_prerequisite']
@@ -174,6 +172,7 @@
         :param **kwargs: **kwargs for the call.
         :return: The result of calling call(*args, *kwargs).
         """
+        from lazr.restfulclient import errors as restful_errors
         try:
             return call(*args, **kwargs)
         except restful_errors.HTTPError, e:
@@ -208,7 +207,7 @@
             review_types=review_types)
         if self.approve:
             self.call_webservice(mp.setStatus, status='Approved')
-        webbrowser.open(canonical_url(mp))
+        webbrowser.open(lp_api.canonical_url(mp))
 
 
 def modified_files(old_tree, new_tree):

=== modified file 'bzrlib/plugins/launchpad/lp_registration.py'
--- bzrlib/plugins/launchpad/lp_registration.py	2010-12-02 09:23:10 +0000
+++ bzrlib/plugins/launchpad/lp_registration.py	2011-01-17 06:26:35 +0000
@@ -86,6 +86,7 @@
 
     LAUNCHPAD_DOMAINS = {
         'production': 'launchpad.net',
+        'qastaging': 'qastaging.launchpad.net',
         'staging': 'staging.launchpad.net',
         'demo': 'demo.launchpad.net',
         'dev': 'launchpad.dev',

=== modified file 'bzrlib/plugins/launchpad/test_lp_api.py'
--- bzrlib/plugins/launchpad/test_lp_api.py	2010-03-05 08:55:12 +0000
+++ bzrlib/plugins/launchpad/test_lp_api.py	2011-01-17 06:26:35 +0000
@@ -15,8 +15,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-from bzrlib import commands, config, errors, osutils
-from bzrlib.plugins.launchpad import cmd_launchpad_mirror
+from bzrlib import config, errors, osutils
 from bzrlib.tests import (
     ModuleAvailableFeature,
     TestCase,

=== modified file 'bzrlib/plugins/launchpad/test_lp_directory.py'
--- bzrlib/plugins/launchpad/test_lp_directory.py	2010-12-03 11:16:25 +0000
+++ bzrlib/plugins/launchpad/test_lp_directory.py	2011-01-17 06:26:35 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -91,6 +91,19 @@
         self.assertEquals('https://xmlrpc.launchpad.net/bazaar/',
                           factory._service_url)
 
+    def test_qastaging(self):
+        """A launchpad url should map to a http url"""
+        factory = FakeResolveFactory(
+            self, 'apt', dict(urls=[
+                    'http://bazaar.qastaging.launchpad.net/~apt/apt/devel']))
+        url = 'lp://qastaging/apt'
+        directory = LaunchpadDirectory()
+        self.assertEquals('http://bazaar.qastaging.launchpad.net/~apt/apt/devel',
+                          directory._resolve(url, factory))
+        # Make sure that resolve went to the qastaging server.
+        self.assertEquals('https://xmlrpc.qastaging.launchpad.net/bazaar/',
+                          factory._service_url)
+
     def test_staging(self):
         """A launchpad url should map to a http url"""
         factory = FakeResolveFactory(
@@ -298,7 +311,7 @@
         self.server = self.server_class()
         self.server.start_server()
         # Ensure we don't clobber env
-        self._captureVar('BZR_LP_XMLRPC_URL', None)
+        self.overrideEnv('BZR_LP_XMLRPC_URL', None)
 
     def tearDown(self):
         self.server.stop_server()

=== modified file 'bzrlib/plugins/launchpad/test_lp_service.py'
--- bzrlib/plugins/launchpad/test_lp_service.py	2010-11-26 17:33:55 +0000
+++ bzrlib/plugins/launchpad/test_lp_service.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
     def setUp(self):
         super(LaunchpadServiceTests, self).setUp()
         # make sure we have a reproducible standard environment
-        self._captureVar('BZR_LP_XMLRPC_URL', None)
+        self.overrideEnv('BZR_LP_XMLRPC_URL', None)
 
     def test_default_service(self):
         service = LaunchpadService()

=== modified file 'bzrlib/plugins/launchpad/test_register.py'
--- bzrlib/plugins/launchpad/test_register.py	2010-12-07 09:06:39 +0000
+++ bzrlib/plugins/launchpad/test_register.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,18 +15,16 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import base64
-import os
 from StringIO import StringIO
 import urlparse
 import xmlrpclib
 
 from bzrlib import (
     config,
-    osutils,
     tests,
     ui,
     )
-from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.tests import TestCaseWithTransport
 
 # local import
 from bzrlib.plugins.launchpad.lp_registration import (
@@ -61,6 +59,25 @@
         """
         return (200, 'OK', [])
 
+    def getresponse(self, buffering=True):
+        """Fake the http reply.
+
+        This is used when running on Python 2.7, where xmlrpclib uses
+        httplib.HTTPConnection in a different way than before.
+        """
+        class FakeHttpResponse(object):
+
+            def __init__(self, status, reason, headers, body):
+                self.status = status
+                self.reason = reason
+                self.headers = headers
+                self.body = body
+
+            def read(self, size=-1):
+                return self.body.read(size)
+
+        return FakeHttpResponse(200, 'OK', [], self.getfile())
+
     def getfile(self):
         """Return a fake file containing the response content."""
         return StringIO('''\
@@ -85,6 +102,7 @@
     def __init__(self, testcase, expect_auth):
         self.testcase = testcase
         self.expect_auth = expect_auth
+        self._connection = (None, None)
 
     def make_connection(self, host):
         host, http_headers, x509 = self.get_host_info(host)
@@ -137,7 +155,7 @@
     def setUp(self):
         super(TestBranchRegistration, self).setUp()
         # make sure we have a reproducible standard environment
-        self._captureVar('BZR_LP_XMLRPC_URL', None)
+        self.overrideEnv('BZR_LP_XMLRPC_URL', None)
 
     def test_register_help(self):
         """register-branch accepts --help"""
@@ -296,7 +314,7 @@
     def setUp(self):
         super(TestGatherUserCredentials, self).setUp()
         # make sure we have a reproducible standard environment
-        self._captureVar('BZR_LP_XMLRPC_URL', None)
+        self.overrideEnv('BZR_LP_XMLRPC_URL', None)
 
     def test_gather_user_credentials_has_password(self):
         service = LaunchpadService()

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2010-12-14 23:14:44 +0000
+++ bzrlib/remote.py	2011-01-14 17:18:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1348,15 +1348,29 @@
         return result
 
     @needs_read_lock
-    def search_missing_revision_ids(self, other, revision_id=None, find_ghosts=True):
+    def search_missing_revision_ids(self, other,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None):
         """Return the revision ids that other has that this does not.
 
         These are returned in topological order.
 
         revision_id: only return revision ids included by revision_id.
         """
-        return repository.InterRepository.get(
-            other, self).search_missing_revision_ids(revision_id, find_ghosts)
+        if symbol_versioning.deprecated_passed(revision_id):
+            symbol_versioning.warn(
+                'search_missing_revision_ids(revision_id=...) was '
+                'deprecated in 2.3.  Use revision_ids=[...] instead.',
+                DeprecationWarning, stacklevel=2)
+            if revision_ids is not None:
+                raise AssertionError(
+                    'revision_ids is mutually exclusive with revision_id')
+            if revision_id is not None:
+                revision_ids = [revision_id]
+        inter_repo = repository.InterRepository.get(other, self)
+        return inter_repo.search_missing_revision_ids(
+            find_ghosts=find_ghosts, revision_ids=revision_ids,
+            if_present_ids=if_present_ids)
 
     def fetch(self, source, revision_id=None, pb=None, find_ghosts=False,
             fetch_spec=None):
@@ -1763,12 +1777,7 @@
         return '\n'.join((start_keys, stop_keys, count))
 
     def _serialise_search_result(self, search_result):
-        if isinstance(search_result, graph.PendingAncestryResult):
-            parts = ['ancestry-of']
-            parts.extend(search_result.heads)
-        else:
-            recipe = search_result.get_recipe()
-            parts = [recipe[0], self._serialise_search_recipe(recipe)]
+        parts = search_result.get_network_struct()
         return '\n'.join(parts)
 
     def autopack(self):
@@ -1968,6 +1977,7 @@
         candidate_verbs = [
             ('Repository.get_stream_1.19', (1, 19)),
             ('Repository.get_stream', (1, 13))]
+
         found_verb = False
         for verb, version in candidate_verbs:
             if medium._is_remote_before(version):
@@ -1977,6 +1987,17 @@
                     verb, args, search_bytes)
             except errors.UnknownSmartMethod:
                 medium._remember_remote_is_before(version)
+            except errors.UnknownErrorFromSmartServer, e:
+                if isinstance(search, graph.EverythingResult):
+                    error_verb = e.error_from_smart_server.error_verb
+                    if error_verb == 'BadSearch':
+                        # Pre-2.3 servers don't support this sort of search.
+                        # XXX: perhaps falling back to VFS on BadSearch is a
+                        # good idea in general?  It might provide a little bit
+                        # of protection against client-side bugs.
+                        medium._remember_remote_is_before((2, 3))
+                        break
+                raise
             else:
                 response_tuple, response_handler = response
                 found_verb = True

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2010-11-20 21:41:05 +0000
+++ bzrlib/repofmt/knitrepo.py	2010-12-21 06:10:11 +0000
@@ -43,6 +43,7 @@
     RepositoryFormat,
     RootCommitBuilder,
     )
+from bzrlib import symbol_versioning
 
 
 class _KnitParentsProvider(object):
@@ -534,16 +535,23 @@
         return are_knits and InterRepository._same_model(source, target)
 
     @needs_read_lock
-    def search_missing_revision_ids(self, revision_id=None, find_ghosts=True):
-        """See InterRepository.missing_revision_ids()."""
-        if revision_id is not None:
-            source_ids = self.source.get_ancestry(revision_id)
-            if source_ids[0] is not None:
-                raise AssertionError()
-            source_ids.pop(0)
-        else:
-            source_ids = self.source.all_revision_ids()
-        source_ids_set = set(source_ids)
+    def search_missing_revision_ids(self,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None):
+        """See InterRepository.search_missing_revision_ids()."""
+        if symbol_versioning.deprecated_passed(revision_id):
+            symbol_versioning.warn(
+                'search_missing_revision_ids(revision_id=...) was '
+                'deprecated in 2.3.  Use revision_ids=[...] instead.',
+                DeprecationWarning, stacklevel=2)
+            if revision_ids is not None:
+                raise AssertionError(
+                    'revision_ids is mutually exclusive with revision_id')
+            if revision_id is not None:
+                revision_ids = [revision_id]
+        del revision_id
+        source_ids_set = self._present_source_revisions_for(
+            revision_ids, if_present_ids)
         # source_ids is the worst possible case we may need to pull.
         # now we want to filter source_ids against what we actually
         # have in target, but don't try to check for existence where we know
@@ -553,7 +561,7 @@
         actually_present_revisions = set(
             self.target._eliminate_revisions_not_present(possibly_present_revisions))
         required_revisions = source_ids_set.difference(actually_present_revisions)
-        if revision_id is not None:
+        if revision_ids is not None:
             # we used get_ancestry to determine source_ids then we are assured all
             # revisions referenced are present as they are installed in topological order.
             # and the tip revision was validated by get_ancestry.

=== modified file 'bzrlib/repofmt/weaverepo.py'
--- bzrlib/repofmt/weaverepo.py	2010-11-20 21:41:05 +0000
+++ bzrlib/repofmt/weaverepo.py	2011-01-14 17:18:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 ghosts.
 """
 
+import gzip
 import os
 from cStringIO import StringIO
 import urllib
@@ -39,7 +40,9 @@
     lockable_files,
     lockdir,
     osutils,
+    symbol_versioning,
     trace,
+    tuned_gzip,
     urlutils,
     versionedfile,
     weave,
@@ -56,7 +59,6 @@
     RepositoryFormat,
     )
 from bzrlib.store.text import TextStore
-from bzrlib.tuned_gzip import GzipFile, bytes_to_gzip
 from bzrlib.versionedfile import (
     AbsentContentFactory,
     FulltextContentFactory,
@@ -339,6 +341,8 @@
     has been removed.
     """
 
+    supports_funky_characters = False
+
     _matchingbzrdir = bzrdir.BzrDirFormat4()
 
     def get_format_description(self):
@@ -388,6 +392,8 @@
 
     _versionedfile_class = weave.WeaveFile
     _matchingbzrdir = bzrdir.BzrDirFormat5()
+    supports_funky_characters = False
+
     @property
     def _serializer(self):
         return xml5.serializer_v5
@@ -433,6 +439,7 @@
 
     _versionedfile_class = weave.WeaveFile
     _matchingbzrdir = bzrdir.BzrDirFormat6()
+    supports_funky_characters = False
     @property
     def _serializer(self):
         return xml5.serializer_v5
@@ -482,6 +489,7 @@
     _versionedfile_class = weave.WeaveFile
     supports_ghosts = False
     supports_chks = False
+    supports_funky_characters = False
 
     _fetch_order = 'topological'
     _fetch_reconcile = True
@@ -589,7 +597,7 @@
             raise ValueError('bad idea to put / in %r' % (key,))
         text = ''.join(lines)
         if self._compressed:
-            text = bytes_to_gzip(text)
+            text = tuned_gzip.bytes_to_gzip(text)
         path = self._map(key)
         self._transport.put_bytes_non_atomic(path, text, create_parent_dir=True)
 
@@ -637,7 +645,7 @@
             else:
                 return None
         if compressed:
-            text = GzipFile(mode='rb', fileobj=StringIO(text)).read()
+            text = gzip.GzipFile(mode='rb', fileobj=StringIO(text)).read()
         return text
 
     def _map(self, key):
@@ -743,9 +751,6 @@
 
 class InterWeaveRepo(InterSameDataRepository):
     """Optimised code paths between Weave based repositories.
-
-    This should be in bzrlib/repofmt/weaverepo.py but we have not yet
-    implemented lazy inter-object optimisation.
     """
 
     @classmethod
@@ -803,8 +808,10 @@
             self.target.fetch(self.source, revision_id=revision_id)
 
     @needs_read_lock
-    def search_missing_revision_ids(self, revision_id=None, find_ghosts=True):
-        """See InterRepository.missing_revision_ids()."""
+    def search_missing_revision_ids(self,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None):
+        """See InterRepository.search_missing_revision_ids()."""
         # we want all revisions to satisfy revision_id in source.
         # but we don't want to stat every file here and there.
         # we want then, all revisions other needs to satisfy revision_id
@@ -816,14 +823,19 @@
         # disk format scales terribly for push anyway due to rewriting
         # inventory.weave, this is considered acceptable.
         # - RBC 20060209
-        if revision_id is not None:
-            source_ids = self.source.get_ancestry(revision_id)
-            if source_ids[0] is not None:
-                raise AssertionError()
-            source_ids.pop(0)
-        else:
-            source_ids = self.source._all_possible_ids()
-        source_ids_set = set(source_ids)
+        if symbol_versioning.deprecated_passed(revision_id):
+            symbol_versioning.warn(
+                'search_missing_revision_ids(revision_id=...) was '
+                'deprecated in 2.3.  Use revision_ids=[...] instead.',
+                DeprecationWarning, stacklevel=2)
+            if revision_ids is not None:
+                raise AssertionError(
+                    'revision_ids is mutually exclusive with revision_id')
+            if revision_id is not None:
+                revision_ids = [revision_id]
+        del revision_id
+        source_ids_set = self._present_source_revisions_for(
+            revision_ids, if_present_ids)
         # source_ids is the worst possible case we may need to pull.
         # now we want to filter source_ids against what we actually
         # have in target, but don't try to check for existence where we know
@@ -833,7 +845,7 @@
         actually_present_revisions = set(
             self.target._eliminate_revisions_not_present(possibly_present_revisions))
         required_revisions = source_ids_set.difference(actually_present_revisions)
-        if revision_id is not None:
+        if revision_ids is not None:
             # we used get_ancestry to determine source_ids then we are assured all
             # revisions referenced are present as they are installed in topological order.
             # and the tip revision was validated by get_ancestry.

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2010-12-02 10:41:05 +0000
+++ bzrlib/repository.py	2011-01-14 17:18:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,7 +42,6 @@
     pyutils,
     revision as _mod_revision,
     static_tuple,
-    symbol_versioning,
     trace,
     tsort,
     versionedfile,
@@ -57,6 +56,7 @@
 from bzrlib import (
     errors,
     registry,
+    symbol_versioning,
     ui,
     )
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
@@ -176,6 +176,41 @@
             self._validate_unicode_text(value,
                                         'revision property (%s)' % (key,))
 
+    def _ensure_fallback_inventories(self):
+        """Ensure that appropriate inventories are available.
+
+        This only applies to repositories that are stacked, and is about
+        enusring the stacking invariants. Namely, that for any revision that is
+        present, we either have all of the file content, or we have the parent
+        inventory and the delta file content.
+        """
+        if not self.repository._fallback_repositories:
+            return
+        if not self.repository._format.supports_chks:
+            raise errors.BzrError("Cannot commit directly to a stacked branch"
+                " in pre-2a formats. See "
+                "https://bugs.launchpad.net/bzr/+bug/375013 for details.")
+        # This is a stacked repo, we need to make sure we have the parent
+        # inventories for the parents.
+        parent_keys = [(p,) for p in self.parents]
+        parent_map = self.repository.inventories._index.get_parent_map(parent_keys)
+        missing_parent_keys = set([pk for pk in parent_keys
+                                       if pk not in parent_map])
+        fallback_repos = list(reversed(self.repository._fallback_repositories))
+        missing_keys = [('inventories', pk[0])
+                        for pk in missing_parent_keys]
+        resume_tokens = []
+        while missing_keys and fallback_repos:
+            fallback_repo = fallback_repos.pop()
+            source = fallback_repo._get_source(self.repository._format)
+            sink = self.repository._get_sink()
+            stream = source.get_stream_for_missing_keys(missing_keys)
+            missing_keys = sink.insert_stream_without_locking(stream,
+                self.repository._format)
+        if missing_keys:
+            raise errors.BzrError('Unable to fill in parent inventories for a'
+                                  ' stacked branch')
+
     def commit(self, message):
         """Make the actual commit.
 
@@ -193,6 +228,7 @@
         rev.parent_ids = self.parents
         self.repository.add_revision(self._new_revision_id, rev,
             self.new_inventory, self._config)
+        self._ensure_fallback_inventories()
         self.repository.commit_write_group()
         return self._new_revision_id
 
@@ -1561,15 +1597,28 @@
         return ret
 
     @needs_read_lock
-    def search_missing_revision_ids(self, other, revision_id=None, find_ghosts=True):
+    def search_missing_revision_ids(self, other,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None):
         """Return the revision ids that other has that this does not.
 
         These are returned in topological order.
 
         revision_id: only return revision ids included by revision_id.
         """
+        if symbol_versioning.deprecated_passed(revision_id):
+            symbol_versioning.warn(
+                'search_missing_revision_ids(revision_id=...) was '
+                'deprecated in 2.3.  Use revision_ids=[...] instead.',
+                DeprecationWarning, stacklevel=3)
+            if revision_ids is not None:
+                raise AssertionError(
+                    'revision_ids is mutually exclusive with revision_id')
+            if revision_id is not None:
+                revision_ids = [revision_id]
         return InterRepository.get(other, self).search_missing_revision_ids(
-            revision_id, find_ghosts)
+            find_ghosts=find_ghosts, revision_ids=revision_ids,
+            if_present_ids=if_present_ids)
 
     @staticmethod
     def open(base):
@@ -1761,9 +1810,9 @@
         :param revprops: Optional dictionary of revision properties.
         :param revision_id: Optional revision id.
         """
-        if self._fallback_repositories:
-            raise errors.BzrError("Cannot commit from a lightweight checkout "
-                "to a stacked branch. See "
+        if self._fallback_repositories and not self._format.supports_chks:
+            raise errors.BzrError("Cannot commit directly to a stacked branch"
+                " in pre-2a formats. See "
                 "https://bugs.launchpad.net/bzr/+bug/375013 for details.")
         result = self._commit_builder_class(self, parents, config,
             timestamp, timezone, committer, revprops, revision_id)
@@ -2822,39 +2871,6 @@
         raise NotImplementedError(self.revision_graph_can_have_wrong_parents)
 
 
-# remove these delegates a while after bzr 0.15
-def __make_delegated(name, from_module):
-    def _deprecated_repository_forwarder():
-        symbol_versioning.warn('%s moved to %s in bzr 0.15'
-            % (name, from_module),
-            DeprecationWarning,
-            stacklevel=2)
-        try:
-            return pyutils.get_named_object(from_module, name)
-        except AttributeError:
-            raise AttributeError('module %s has no name %s'
-                    % (sys.modules[from_module], name))
-    globals()[name] = _deprecated_repository_forwarder
-
-for _name in [
-        'AllInOneRepository',
-        'WeaveMetaDirRepository',
-        'PreSplitOutRepositoryFormat',
-        'RepositoryFormat4',
-        'RepositoryFormat5',
-        'RepositoryFormat6',
-        'RepositoryFormat7',
-        ]:
-    __make_delegated(_name, 'bzrlib.repofmt.weaverepo')
-
-for _name in [
-        'KnitRepository',
-        'RepositoryFormatKnit',
-        'RepositoryFormatKnit1',
-        ]:
-    __make_delegated(_name, 'bzrlib.repofmt.knitrepo')
-
-
 def install_revision(repository, rev, revision_tree):
     """Install all revision data into a repository."""
     install_revisions(repository, [(rev, revision_tree, None)])
@@ -3079,6 +3095,9 @@
     supports_tree_reference = None
     # Is the format experimental ?
     experimental = False
+    # Does this repository format escape funky characters, or does it create files with
+    # similar names as the versioned files in its contents on disk ?
+    supports_funky_characters = True
 
     def __repr__(self):
         return "%s()" % self.__class__.__name__
@@ -3430,7 +3449,7 @@
                                fetch_spec=fetch_spec,
                                find_ghosts=find_ghosts)
 
-    def _walk_to_common_revisions(self, revision_ids):
+    def _walk_to_common_revisions(self, revision_ids, if_present_ids=None):
         """Walk out from revision_ids in source to revisions target has.
 
         :param revision_ids: The start point for the search.
@@ -3438,10 +3457,14 @@
         """
         target_graph = self.target.get_graph()
         revision_ids = frozenset(revision_ids)
+        if if_present_ids:
+            all_wanted_revs = revision_ids.union(if_present_ids)
+        else:
+            all_wanted_revs = revision_ids
         missing_revs = set()
         source_graph = self.source.get_graph()
         # ensure we don't pay silly lookup costs.
-        searcher = source_graph._make_breadth_first_searcher(revision_ids)
+        searcher = source_graph._make_breadth_first_searcher(all_wanted_revs)
         null_set = frozenset([_mod_revision.NULL_REVISION])
         searcher_exhausted = False
         while True:
@@ -3483,30 +3506,79 @@
         return searcher.get_result()
 
     @needs_read_lock
-    def search_missing_revision_ids(self, revision_id=None, find_ghosts=True):
+    def search_missing_revision_ids(self,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None):
         """Return the revision ids that source has that target does not.
 
         :param revision_id: only return revision ids included by this
-                            revision_id.
+            revision_id.
+        :param revision_ids: return revision ids included by these
+            revision_ids.  NoSuchRevision will be raised if any of these
+            revisions are not present.
+        :param if_present_ids: like revision_ids, but will not cause
+            NoSuchRevision if any of these are absent, instead they will simply
+            not be in the result.  This is useful for e.g. finding revisions
+            to fetch for tags, which may reference absent revisions.
         :param find_ghosts: If True find missing revisions in deep history
             rather than just finding the surface difference.
         :return: A bzrlib.graph.SearchResult.
         """
+        if symbol_versioning.deprecated_passed(revision_id):
+            symbol_versioning.warn(
+                'search_missing_revision_ids(revision_id=...) was '
+                'deprecated in 2.3.  Use revision_ids=[...] instead.',
+                DeprecationWarning, stacklevel=2)
+            if revision_ids is not None:
+                raise AssertionError(
+                    'revision_ids is mutually exclusive with revision_id')
+            if revision_id is not None:
+                revision_ids = [revision_id]
+        del revision_id
         # stop searching at found target revisions.
-        if not find_ghosts and revision_id is not None:
-            return self._walk_to_common_revisions([revision_id])
+        if not find_ghosts and (revision_ids is not None or if_present_ids is
+                not None):
+            return self._walk_to_common_revisions(revision_ids,
+                    if_present_ids=if_present_ids)
         # generic, possibly worst case, slow code path.
         target_ids = set(self.target.all_revision_ids())
-        if revision_id is not None:
-            source_ids = self.source.get_ancestry(revision_id)
-            if source_ids[0] is not None:
-                raise AssertionError()
-            source_ids.pop(0)
-        else:
-            source_ids = self.source.all_revision_ids()
+        source_ids = self._present_source_revisions_for(
+            revision_ids, if_present_ids)
         result_set = set(source_ids).difference(target_ids)
         return self.source.revision_ids_to_search_result(result_set)
 
+    def _present_source_revisions_for(self, revision_ids, if_present_ids=None):
+        """Returns set of all revisions in ancestry of revision_ids present in
+        the source repo.
+
+        :param revision_ids: if None, all revisions in source are returned.
+        :param if_present_ids: like revision_ids, but if any/all of these are
+            absent no error is raised.
+        """
+        if revision_ids is not None or if_present_ids is not None:
+            # First, ensure all specified revisions exist.  Callers expect
+            # NoSuchRevision when they pass absent revision_ids here.
+            if revision_ids is None:
+                revision_ids = set()
+            if if_present_ids is None:
+                if_present_ids = set()
+            revision_ids = set(revision_ids)
+            if_present_ids = set(if_present_ids)
+            all_wanted_ids = revision_ids.union(if_present_ids)
+            graph = self.source.get_graph()
+            present_revs = set(graph.get_parent_map(all_wanted_ids))
+            missing = revision_ids.difference(present_revs)
+            if missing:
+                raise errors.NoSuchRevision(self.source, missing.pop())
+            found_ids = all_wanted_ids.intersection(present_revs)
+            source_ids = [rev_id for (rev_id, parents) in
+                          graph.iter_ancestry(found_ids)
+                          if rev_id != _mod_revision.NULL_REVISION
+                          and parents is not None]
+        else:
+            source_ids = self.source.all_revision_ids()
+        return set(source_ids)
+
     @staticmethod
     def _same_model(source, target):
         """True if source and target have the same data representation.
@@ -3830,7 +3902,13 @@
             fetch_spec=None):
         """See InterRepository.fetch()."""
         if fetch_spec is not None:
-            raise AssertionError("Not implemented yet...")
+            if (isinstance(fetch_spec, graph.NotInOtherForRevs) and
+                    len(fetch_spec.required_ids) == 1 and not
+                    fetch_spec.if_present_ids):
+                revision_id = list(fetch_spec.required_ids)[0]
+                del fetch_spec
+            else:
+                raise AssertionError("Not implemented yet...")
         ui.ui_factory.warn_experimental_format_fetch(self)
         if (not self.source.supports_rich_root()
             and self.target.supports_rich_root()):
@@ -3843,8 +3921,12 @@
             ui.ui_factory.show_user_warning('cross_format_fetch',
                 from_format=self.source._format,
                 to_format=self.target._format)
+        if revision_id:
+            search_revision_ids = [revision_id]
+        else:
+            search_revision_ids = None
         revision_ids = self.target.search_missing_revision_ids(self.source,
-            revision_id, find_ghosts=find_ghosts).get_keys()
+            revision_ids=search_revision_ids, find_ghosts=find_ghosts).get_keys()
         if not revision_ids:
             return 0, 0
         revision_ids = tsort.topo_sort(
@@ -4087,15 +4169,46 @@
                 is_resume = False
             try:
                 # locked_insert_stream performs a commit|suspend.
-                return self._locked_insert_stream(stream, src_format,
-                    is_resume)
+                missing_keys = self.insert_stream_without_locking(stream,
+                                    src_format, is_resume)
+                if missing_keys:
+                    # suspend the write group and tell the caller what we is
+                    # missing. We know we can suspend or else we would not have
+                    # entered this code path. (All repositories that can handle
+                    # missing keys can handle suspending a write group).
+                    write_group_tokens = self.target_repo.suspend_write_group()
+                    return write_group_tokens, missing_keys
+                hint = self.target_repo.commit_write_group()
+                to_serializer = self.target_repo._format._serializer
+                src_serializer = src_format._serializer
+                if (to_serializer != src_serializer and
+                    self.target_repo._format.pack_compresses):
+                    self.target_repo.pack(hint=hint)
+                return [], set()
             except:
                 self.target_repo.abort_write_group(suppress_errors=True)
                 raise
         finally:
             self.target_repo.unlock()
 
-    def _locked_insert_stream(self, stream, src_format, is_resume):
+    def insert_stream_without_locking(self, stream, src_format,
+                                      is_resume=False):
+        """Insert a stream's content into the target repository.
+
+        This assumes that you already have a locked repository and an active
+        write group.
+
+        :param src_format: a bzr repository format.
+        :param is_resume: Passed down to get_missing_parent_inventories to
+            indicate if we should be checking for missing texts at the same
+            time.
+
+        :return: A set of keys that are missing.
+        """
+        if not self.target_repo.is_write_locked():
+            raise errors.ObjectNotLocked(self)
+        if not self.target_repo.is_in_write_group():
+            raise errors.BzrError('you must already be in a write group')
         to_serializer = self.target_repo._format._serializer
         src_serializer = src_format._serializer
         new_pack = None
@@ -4180,19 +4293,7 @@
             # cannot even attempt suspending, and missing would have failed
             # during stream insertion.
             missing_keys = set()
-        else:
-            if missing_keys:
-                # suspend the write group and tell the caller what we is
-                # missing. We know we can suspend or else we would not have
-                # entered this code path. (All repositories that can handle
-                # missing keys can handle suspending a write group).
-                write_group_tokens = self.target_repo.suspend_write_group()
-                return write_group_tokens, missing_keys
-        hint = self.target_repo.commit_write_group()
-        if (to_serializer != src_serializer and
-            self.target_repo._format.pack_compresses):
-            self.target_repo.pack(hint=hint)
-        return [], set()
+        return missing_keys
 
     def _extract_and_insert_inventory_deltas(self, substream, serializer):
         target_rich_root = self.target_repo._format.rich_root_data

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2010-11-16 06:06:11 +0000
+++ bzrlib/smart/repository.py	2010-11-26 06:31:54 +0000
@@ -81,6 +81,8 @@
             recreate_search trusts that clients will look for missing things
             they expected and get it from elsewhere.
         """
+        if search_bytes == 'everything':
+            return graph.EverythingResult(repository), None
         lines = search_bytes.split('\n')
         if lines[0] == 'ancestry-of':
             heads = lines[1:]
@@ -412,6 +414,13 @@
     def do_repository_request(self, repository, to_network_name):
         """Get a stream for inserting into a to_format repository.
 
+        The request body is 'search_bytes', a description of the revisions
+        being requested.
+
+        In 2.3 this verb added support for search_bytes == 'everything'.  Older
+        implementations will respond with a BadSearch error, and clients should
+        catch this and fallback appropriately.
+
         :param repository: The repository to stream from.
         :param to_network_name: The network name of the format of the target
             repository.
@@ -489,6 +498,13 @@
 
 
 class SmartServerRepositoryGetStream_1_19(SmartServerRepositoryGetStream):
+    """The same as Repository.get_stream, but will return stream CHK formats to
+    clients.
+
+    See SmartServerRepositoryGetStream._should_fake_unknown.
+    
+    New in 1.19.
+    """
 
     def _should_fake_unknown(self):
         """Returns False; we don't need to workaround bugs in 1.19+ clients."""

=== modified file 'bzrlib/smart/server.py'
--- bzrlib/smart/server.py	2010-07-01 15:25:41 +0000
+++ bzrlib/smart/server.py	2011-01-14 22:36:45 +0000
@@ -18,7 +18,6 @@
 
 import errno
 import os.path
-import select
 import socket
 import sys
 import threading
@@ -27,7 +26,6 @@
 from bzrlib import (
     errors,
     trace,
-    transport,
 )
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -178,7 +176,7 @@
 
     def get_url(self):
         """Return the url of the server"""
-        return "bzr://%s:%d/" % self._sockname
+        return "bzr://%s:%s/" % (self._sockname[0], self._sockname[1])
 
     def serve_conn(self, conn, thread_name_suffix):
         # For WIN32, where the timeout value from the listening socket

=== modified file 'bzrlib/tag.py'
--- bzrlib/tag.py	2010-11-18 00:30:51 +0000
+++ bzrlib/tag.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,12 @@
 # NOTE: I was going to call this tags.py, but vim seems to think all files
 # called tags* are ctags files... mbp 20070220.
 
+from bzrlib.registry import Registry
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import itertools
+import re
+import sys
 
 from bzrlib import (
     bencode,
@@ -33,6 +39,7 @@
     symbol_versioning,
     trace,
     )
+""")
 
 
 class _Tags(object):
@@ -307,3 +314,50 @@
             DeprecationWarning)
         from_branch.tags.merge_to(to_branch.tags)
 
+
+def sort_natural(branch, tags):
+    """Sort tags, with numeric substrings as numbers.
+
+    :param branch: Branch
+    :param tags: List of tuples with tag name and revision id.
+    """
+    def natural_sort_key(tag):
+        return [f(s) for f,s in
+                zip(itertools.cycle((unicode.lower,int)),
+                                    re.split('([0-9]+)', tag[0]))]
+    tags.sort(key=natural_sort_key)
+
+
+def sort_alpha(branch, tags):
+    """Sort tags lexicographically, in place.
+
+    :param branch: Branch
+    :param tags: List of tuples with tag name and revision id.
+    """
+    tags.sort()
+
+
+def sort_time(branch, tags):
+    """Sort tags by time inline.
+
+    :param branch: Branch
+    :param tags: List of tuples with tag name and revision id.
+    """
+    timestamps = {}
+    for tag, revid in tags:
+        try:
+            revobj = branch.repository.get_revision(revid)
+        except errors.NoSuchRevision:
+            timestamp = sys.maxint # place them at the end
+        else:
+            timestamp = revobj.timestamp
+        timestamps[revid] = timestamp
+    tags.sort(key=lambda x: timestamps[x[1]])
+
+
+tag_sort_methods = Registry()
+tag_sort_methods.register("natural", sort_natural,
+    'Sort numeric substrings as numbers. (default)')
+tag_sort_methods.register("alpha", sort_alpha, 'Sort tags lexicographically.')
+tag_sort_methods.register("time", sort_time, 'Sort tags chronologically.')
+tag_sort_methods.default_key = "natural"

=== modified file 'bzrlib/tests/EncodingAdapter.py'
--- bzrlib/tests/EncodingAdapter.py	2010-05-24 01:05:14 +0000
+++ bzrlib/tests/EncodingAdapter.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010, 2011 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,8 +17,6 @@
 
 """Adapter for running test cases against multiple encodings."""
 
-from copy import deepcopy
-
 # prefix for micro (1/1000000)
 _mu = u'\xb5'
 

=== modified file 'bzrlib/tests/TestUtil.py'
--- bzrlib/tests/TestUtil.py	2010-09-21 03:20:09 +0000
+++ bzrlib/tests/TestUtil.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #       Author: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -29,9 +29,11 @@
 
 
 class LogCollector(logging.Handler):
+
     def __init__(self):
         logging.Handler.__init__(self)
         self.records=[]
+
     def emit(self, record):
         self.records.append(record.getMessage())
 
@@ -60,7 +62,8 @@
                 visitor.visitSuite(test)
                 visitTests(test, visitor)
             else:
-                print "unvisitable non-unittest.TestCase element %r (%r)" % (test, test.__class__)
+                print "unvisitable non-unittest.TestCase element %r (%r)" % (
+                    test, test.__class__)
 
 
 class TestSuite(unittest.TestSuite):
@@ -183,7 +186,9 @@
 
 class TestVisitor(object):
     """A visitor for Tests"""
+
     def visitSuite(self, aTestSuite):
         pass
+
     def visitCase(self, aTestCase):
         pass

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2010-12-16 14:39:57 +0000
+++ bzrlib/tests/__init__.py	2011-01-12 18:39:25 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -60,54 +60,40 @@
         % (testtools.__file__, _testtools_version))
 from testtools import content
 
+import bzrlib
 from bzrlib import (
     branchbuilder,
     bzrdir,
     chk_map,
+    commands as _mod_commands,
     config,
     debug,
     errors,
     hooks,
     lock as _mod_lock,
+    lockdir,
     memorytree,
     osutils,
+    plugin as _mod_plugin,
     pyutils,
     ui,
     urlutils,
     registry,
+    symbol_versioning,
+    trace,
     transport as _mod_transport,
     workingtree,
     )
-import bzrlib.branch
-import bzrlib.commands
-import bzrlib.timestamp
-import bzrlib.export
-import bzrlib.inventory
-import bzrlib.iterablefile
-import bzrlib.lockdir
 try:
     import bzrlib.lsprof
 except ImportError:
     # lsprof not available
     pass
-import bzrlib.merge3
-import bzrlib.plugin
 from bzrlib.smart import client, request
-import bzrlib.store
-from bzrlib import symbol_versioning
-from bzrlib.symbol_versioning import (
-    DEPRECATED_PARAMETER,
-    deprecated_function,
-    deprecated_in,
-    deprecated_method,
-    deprecated_passed,
-    )
-import bzrlib.trace
 from bzrlib.transport import (
     memory,
     pathfilter,
     )
-from bzrlib.trace import mutter, note
 from bzrlib.tests import (
     test_server,
     TestUtil,
@@ -115,7 +101,6 @@
     )
 from bzrlib.ui import NullProgressView
 from bzrlib.ui.text import TextUIFactory
-import bzrlib.version_info_formats.format_custom
 
 # Mark this python module as being part of the implementation
 # of unittest: this gives us better tracebacks where the last
@@ -138,6 +123,93 @@
 TestSuite = TestUtil.TestSuite
 TestLoader = TestUtil.TestLoader
 
+# Tests should run in a clean and clearly defined environment. The goal is to
+# keep them isolated from the running environment as mush as possible. The test
+# framework ensures the variables defined below are set (or deleted if the
+# value is None) before a test is run and reset to their original value after
+# the test is run. Generally if some code depends on an environment variable,
+# the tests should start without this variable in the environment. There are a
+# few exceptions but you shouldn't violate this rule lightly.
+isolated_environ = {
+    'BZR_HOME': None,
+    'HOME': None,
+    # bzr now uses the Win32 API and doesn't rely on APPDATA, but the
+    # tests do check our impls match APPDATA
+    'BZR_EDITOR': None, # test_msgeditor manipulates this variable
+    'VISUAL': None,
+    'EDITOR': None,
+    'BZR_EMAIL': None,
+    'BZREMAIL': None, # may still be present in the environment
+    'EMAIL': 'jrandom@example.com', # set EMAIL as bzr does not guess
+    'BZR_PROGRESS_BAR': None,
+    'BZR_LOG': None,
+    'BZR_PLUGIN_PATH': None,
+    'BZR_DISABLE_PLUGINS': None,
+    'BZR_PLUGINS_AT': None,
+    'BZR_CONCURRENCY': None,
+    # Make sure that any text ui tests are consistent regardless of
+    # the environment the test case is run in; you may want tests that
+    # test other combinations.  'dumb' is a reasonable guess for tests
+    # going to a pipe or a StringIO.
+    'TERM': 'dumb',
+    'LINES': '25',
+    'COLUMNS': '80',
+    'BZR_COLUMNS': '80',
+    # Disable SSH Agent
+    'SSH_AUTH_SOCK': None,
+    # Proxies
+    'http_proxy': None,
+    'HTTP_PROXY': None,
+    'https_proxy': None,
+    'HTTPS_PROXY': None,
+    'no_proxy': None,
+    'NO_PROXY': None,
+    'all_proxy': None,
+    'ALL_PROXY': None,
+    # Nobody cares about ftp_proxy, FTP_PROXY AFAIK. So far at
+    # least. If you do (care), please update this comment
+    # -- vila 20080401
+    'ftp_proxy': None,
+    'FTP_PROXY': None,
+    'BZR_REMOTE_PATH': None,
+    # Generally speaking, we don't want apport reporting on crashes in
+    # the test envirnoment unless we're specifically testing apport,
+    # so that it doesn't leak into the real system environment.  We
+    # use an env var so it propagates to subprocesses.
+    'APPORT_DISABLE': '1',
+    }
+
+
+def override_os_environ(test, env=None):
+    """Modify os.environ keeping a copy.
+    
+    :param test: A test instance
+
+    :param env: A dict containing variable definitions to be installed
+    """
+    if env is None:
+        env = isolated_environ
+    test._original_os_environ = dict([(var, value)
+                                      for var, value in os.environ.iteritems()])
+    for var, value in env.iteritems():
+        osutils.set_or_unset_env(var, value)
+        if var not in test._original_os_environ:
+            # The var is new, add it with a value of None, so
+            # restore_os_environ will delete it
+            test._original_os_environ[var] = None
+
+
+def restore_os_environ(test):
+    """Restore os.environ to its original state.
+
+    :param test: A test instance previously passed to override_os_environ.
+    """
+    for var, value in test._original_os_environ.iteritems():
+        # Restore the original value (or delete it if the value has been set to
+        # None in override_os_environ).
+        osutils.set_or_unset_env(var, value)
+
+
 class ExtendedTestResult(testtools.TextTestResult):
     """Accepts, reports and accumulates the results of running tests.
 
@@ -301,10 +373,9 @@
         addOnException = getattr(test, "addOnException", None)
         if addOnException is not None:
             addOnException(self._record_traceback_from_test)
-        # Only check for thread leaks if the test case supports cleanups
-        addCleanup = getattr(test, "addCleanup", None)
-        if addCleanup is not None:
-            addCleanup(self._check_leaked_threads, test)
+        # Only check for thread leaks on bzrlib derived test cases
+        if isinstance(test, TestCase):
+            test.addCleanup(self._check_leaked_threads, test)
 
     def startTests(self):
         self.report_tests_starting()
@@ -810,6 +881,25 @@
         return NullProgressView()
 
 
+def isolated_doctest_setUp(test):
+    override_os_environ(test)
+
+
+def isolated_doctest_tearDown(test):
+    restore_os_environ(test)
+
+
+def IsolatedDocTestSuite(*args, **kwargs):
+    """Overrides doctest.DocTestSuite to handle isolation.
+
+    The method is really a factory and users are expected to use it as such.
+    """
+    
+    kwargs['setUp'] = isolated_doctest_setUp
+    kwargs['tearDown'] = isolated_doctest_tearDown
+    return doctest.DocTestSuite(*args, **kwargs)
+
+
 class TestCase(testtools.TestCase):
     """Base class for bzr unit tests.
 
@@ -1126,7 +1216,7 @@
         except UnicodeError, e:
             # If we can't compare without getting a UnicodeError, then
             # obviously they are different
-            mutter('UnicodeError: %s', e)
+            trace.mutter('UnicodeError: %s', e)
         if message:
             message += '\n'
         raise AssertionError("%snot equal:\na = %s\nb = %s\n"
@@ -1192,7 +1282,6 @@
     def assertLogsError(self, exception_class, func, *args, **kwargs):
         """Assert that func(*args, **kwargs) quietly logs a specific exception.
         """
-        from bzrlib import trace
         captured = []
         orig_log_exception_quietly = trace.log_exception_quietly
         try:
@@ -1247,6 +1336,10 @@
         if haystack.find(needle) == -1:
             self.fail("string %r not found in '''%s'''" % (needle, haystack))
 
+    def assertNotContainsString(self, haystack, needle):
+        if haystack.find(needle) != -1:
+            self.fail("string %r found in '''%s'''" % (needle, haystack))
+
     def assertSubset(self, sublist, superlist):
         """Assert that every entry in sublist is present in superlist."""
         missing = set(sublist) - set(superlist)
@@ -1492,7 +1585,7 @@
         The file is removed as the test is torn down.
         """
         self._log_file = StringIO()
-        self._log_memento = bzrlib.trace.push_log_file(self._log_file)
+        self._log_memento = trace.push_log_file(self._log_file)
         self.addCleanup(self._finishLogFile)
 
     def _finishLogFile(self):
@@ -1500,10 +1593,10 @@
 
         Close the file and delete it, unless setKeepLogfile was called.
         """
-        if bzrlib.trace._trace_file:
+        if trace._trace_file:
             # flush the log file, to get all content
-            bzrlib.trace._trace_file.flush()
-        bzrlib.trace.pop_log_file(self._log_memento)
+            trace._trace_file.flush()
+        trace.pop_log_file(self._log_memento)
         # Cache the log result and delete the file on disk
         self._get_log(False)
 
@@ -1539,67 +1632,23 @@
             setattr(obj, attr_name, new)
         return value
 
+    def overrideEnv(self, name, new):
+        """Set an environment variable, and reset it after the test.
+
+        :param name: The environment variable name.
+
+        :param new: The value to set the variable to. If None, the 
+            variable is deleted from the environment.
+
+        :returns: The actual variable value.
+        """
+        value = osutils.set_or_unset_env(name, new)
+        self.addCleanup(osutils.set_or_unset_env, name, value)
+        return value
+
     def _cleanEnvironment(self):
-        new_env = {
-            'BZR_HOME': None, # Don't inherit BZR_HOME to all the tests.
-            'HOME': os.getcwd(),
-            # bzr now uses the Win32 API and doesn't rely on APPDATA, but the
-            # tests do check our impls match APPDATA
-            'BZR_EDITOR': None, # test_msgeditor manipulates this variable
-            'VISUAL': None,
-            'EDITOR': None,
-            'BZR_EMAIL': None,
-            'BZREMAIL': None, # may still be present in the environment
-            'EMAIL': 'jrandom@example.com', # set EMAIL as bzr does not guess
-            'BZR_PROGRESS_BAR': None,
-            'BZR_LOG': None,
-            'BZR_PLUGIN_PATH': None,
-            'BZR_DISABLE_PLUGINS': None,
-            'BZR_PLUGINS_AT': None,
-            'BZR_CONCURRENCY': None,
-            # Make sure that any text ui tests are consistent regardless of
-            # the environment the test case is run in; you may want tests that
-            # test other combinations.  'dumb' is a reasonable guess for tests
-            # going to a pipe or a StringIO.
-            'TERM': 'dumb',
-            'LINES': '25',
-            'COLUMNS': '80',
-            'BZR_COLUMNS': '80',
-            # SSH Agent
-            'SSH_AUTH_SOCK': None,
-            # Proxies
-            'http_proxy': None,
-            'HTTP_PROXY': None,
-            'https_proxy': None,
-            'HTTPS_PROXY': None,
-            'no_proxy': None,
-            'NO_PROXY': None,
-            'all_proxy': None,
-            'ALL_PROXY': None,
-            # Nobody cares about ftp_proxy, FTP_PROXY AFAIK. So far at
-            # least. If you do (care), please update this comment
-            # -- vila 20080401
-            'ftp_proxy': None,
-            'FTP_PROXY': None,
-            'BZR_REMOTE_PATH': None,
-            # Generally speaking, we don't want apport reporting on crashes in
-            # the test envirnoment unless we're specifically testing apport,
-            # so that it doesn't leak into the real system environment.  We
-            # use an env var so it propagates to subprocesses.
-            'APPORT_DISABLE': '1',
-        }
-        self._old_env = {}
-        self.addCleanup(self._restoreEnvironment)
-        for name, value in new_env.iteritems():
-            self._captureVar(name, value)
-
-    def _captureVar(self, name, newvalue):
-        """Set an environment variable, and reset it when finished."""
-        self._old_env[name] = osutils.set_or_unset_env(name, newvalue)
-
-    def _restoreEnvironment(self):
-        for name, value in self._old_env.iteritems():
-            osutils.set_or_unset_env(name, value)
+        for name, value in isolated_environ.iteritems():
+            self.overrideEnv(name, value)
 
     def _restoreHooks(self):
         for klass, (name, hooks) in self._preserved_hooks.items():
@@ -1699,7 +1748,7 @@
             self._benchtime += time.time() - start
 
     def log(self, *args):
-        mutter(*args)
+        trace.mutter(*args)
 
     def _get_log(self, keep_log_file=False):
         """Internal helper to get the log from bzrlib.trace for this test.
@@ -1790,9 +1839,10 @@
 
         try:
             try:
-                result = self.apply_redirected(ui.ui_factory.stdin,
+                result = self.apply_redirected(
+                    ui.ui_factory.stdin,
                     stdout, stderr,
-                    bzrlib.commands.run_bzr_catch_user_errors,
+                    _mod_commands.run_bzr_catch_user_errors,
                     args)
             except KeyboardInterrupt:
                 # Reraise KeyboardInterrupt with contents of redirected stdout
@@ -2048,8 +2098,8 @@
         if retcode is not None and retcode != process.returncode:
             if process_args is None:
                 process_args = "(unknown args)"
-            mutter('Output of bzr %s:\n%s', process_args, out)
-            mutter('Error for bzr %s:\n%s', process_args, err)
+            trace.mutter('Output of bzr %s:\n%s', process_args, out)
+            trace.mutter('Error for bzr %s:\n%s', process_args, err)
             self.fail('Command bzr %s failed with retcode %s != %s'
                       % (process_args, retcode, process.returncode))
         return [out, err]
@@ -2112,7 +2162,7 @@
 
         Tests that expect to provoke LockContention errors should call this.
         """
-        self.overrideAttr(bzrlib.lockdir, '_DEFAULT_TIMEOUT_SECONDS', 0)
+        self.overrideAttr(lockdir, '_DEFAULT_TIMEOUT_SECONDS', 0)
 
     def make_utf8_encoded_stringio(self, encoding_type=None):
         """Return a StringIOWrapper instance, that will encode Unicode
@@ -2442,8 +2492,8 @@
         test_home_dir = self.test_home_dir
         if isinstance(test_home_dir, unicode):
             test_home_dir = test_home_dir.encode(sys.getfilesystemencoding())
-        os.environ['HOME'] = test_home_dir
-        os.environ['BZR_HOME'] = test_home_dir
+        self.overrideEnv('HOME', test_home_dir)
+        self.overrideEnv('BZR_HOME', test_home_dir)
 
     def setUp(self):
         super(TestCaseWithMemoryTransport, self).setUp()
@@ -3599,8 +3649,9 @@
                 key, obj, help=help, info=info, override_existing=False)
         except KeyError:
             actual = self.get(key)
-            note('Test prefix alias %s is already used for %s, ignoring %s'
-                 % (key, actual, obj))
+            trace.note(
+                'Test prefix alias %s is already used for %s, ignoring %s'
+                % (key, actual, obj))
 
     def resolve_alias(self, id_start):
         """Replace the alias by the prefix in the given string.
@@ -3842,13 +3893,7 @@
         return []
     return [
         'bzrlib',
-        # FIXME: Fixing bug #690563 revealed an isolation problem in the single
-        # doctest for branchbuilder. Uncomment this when bug #321320 is fixed
-        # to ensure the issue is addressed (note that to reproduce the bug in
-        # the doctest below, one should comment the 'email' config var in
-        # bazaar.conf (or anywhere else). This means an setup where *no* user
-        # is being set at all in the environment.
-#       'bzrlib.branchbuilder',
+        'bzrlib.branchbuilder',
         'bzrlib.decorators',
         'bzrlib.export',
         'bzrlib.inventory',
@@ -3920,8 +3965,8 @@
         try:
             # note that this really does mean "report only" -- doctest
             # still runs the rest of the examples
-            doc_suite = doctest.DocTestSuite(mod,
-                optionflags=doctest.REPORT_ONLY_FIRST_FAILURE)
+            doc_suite = IsolatedDocTestSuite(
+                mod, optionflags=doctest.REPORT_ONLY_FIRST_FAILURE)
         except ValueError, e:
             print '**failed to get doctest for: %s\n%s' % (mod, e)
             raise
@@ -3930,7 +3975,7 @@
         suite.addTest(doc_suite)
 
     default_encoding = sys.getdefaultencoding()
-    for name, plugin in bzrlib.plugin.plugins().items():
+    for name, plugin in _mod_plugin.plugins().items():
         if not interesting_module(plugin.module.__name__):
             continue
         plugin_suite = plugin.test_suite()
@@ -3942,7 +3987,7 @@
         if plugin_suite is not None:
             suite.addTest(plugin_suite)
         if default_encoding != sys.getdefaultencoding():
-            bzrlib.trace.warning(
+            trace.warning(
                 'Plugin "%s" tried to reset default encoding to: %s', name,
                 sys.getdefaultencoding())
             reload(sys)
@@ -3963,9 +4008,9 @@
             # Some tests mentioned in the list are not in the test suite. The
             # list may be out of date, report to the tester.
             for id in not_found:
-                bzrlib.trace.warning('"%s" not found in the test suite', id)
+                trace.warning('"%s" not found in the test suite', id)
         for id in duplicates:
-            bzrlib.trace.warning('"%s" is used as an id by several tests', id)
+            trace.warning('"%s" is used as an id by several tests', id)
 
     return suite
 
@@ -4309,13 +4354,6 @@
         return self.module_name
 
 
-# This is kept here for compatibility, it is recommended to use
-# 'bzrlib.tests.feature.paramiko' instead
-ParamikoFeature = _CompatabilityThunkFeature(
-    deprecated_in((2,1,0)),
-    'bzrlib.tests.features', 'ParamikoFeature', 'paramiko')
-
-
 def probe_unicode_in_user_encoding():
     """Try to encode several unicode strings to use in unicode-aware tests.
     Return first successfull match.
@@ -4508,10 +4546,6 @@
 case_sensitive_filesystem_feature = _CaseSensitiveFilesystemFeature()
 
 
-# Kept for compatibility, use bzrlib.tests.features.subunit instead
-SubUnitFeature = _CompatabilityThunkFeature(
-    deprecated_in((2,1,0)),
-    'bzrlib.tests.features', 'SubUnitFeature', 'subunit')
 # Only define SubUnitBzrRunner if subunit is available.
 try:
     from subunit import TestProtocolClient

=== modified file 'bzrlib/tests/blackbox/test_add.py'
--- bzrlib/tests/blackbox/test_add.py	2010-12-16 20:41:47 +0000
+++ bzrlib/tests/blackbox/test_add.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2010-12-14 23:14:44 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/blackbox/test_break_lock.py'
--- bzrlib/tests/blackbox/test_break_lock.py	2010-09-14 07:19:22 +0000
+++ bzrlib/tests/blackbox/test_break_lock.py	2011-01-13 05:28:55 +0000
@@ -25,7 +25,6 @@
     tests,
     )
 from bzrlib.tests.script import (
-    ScriptRunner,
     run_script,
     )
 

=== modified file 'bzrlib/tests/blackbox/test_cat_revision.py'
--- bzrlib/tests/blackbox/test_cat_revision.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_cat_revision.py	2011-01-18 23:54:51 +0000
@@ -68,3 +68,11 @@
         output, errors = self.run_bzr(['cat-revision', '-d', 'a', u'abcd'])
         self.assertContainsRe(output, 'This revision')
         self.assertEqual('', errors)
+
+    def test_cat_tree_less_branch(self):
+        tree = self.make_branch_and_tree('.')
+        tree.commit('This revision', rev_id='abcd')
+        tree.bzrdir.destroy_workingtree()
+        output, errors = self.run_bzr(['cat-revision', '-d', 'a', u'abcd'])
+        self.assertContainsRe(output, 'This revision')
+        self.assertEqual('', errors)

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2010-12-02 09:23:10 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -719,13 +719,13 @@
             f = file('fed.bat', 'w')
             f.write('@rem dummy fed')
             f.close()
-            osutils.set_or_unset_env('BZR_EDITOR', "fed.bat")
+            self.overrideEnv('BZR_EDITOR', "fed.bat")
         else:
             f = file('fed.sh', 'wb')
             f.write('#!/bin/sh\n')
             f.close()
             os.chmod('fed.sh', 0755)
-            osutils.set_or_unset_env('BZR_EDITOR', "./fed.sh")
+            self.overrideEnv('BZR_EDITOR', "./fed.sh")
 
     def setup_commit_with_template(self):
         self.setup_editor()
@@ -756,8 +756,8 @@
         os.chdir('foo')
         open('foo.txt', 'w').write('hello')
         self.run_bzr(['add'])
-        osutils.set_or_unset_env('EMAIL', None)
-        osutils.set_or_unset_env('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
         out, err = self.run_bzr(['commit', '-m', 'initial'], 3)
         self.assertContainsRe(err, 'Unable to determine your name')
 

=== modified file 'bzrlib/tests/blackbox/test_diff.py'
--- bzrlib/tests/blackbox/test_diff.py	2010-12-20 02:27:04 +0000
+++ bzrlib/tests/blackbox/test_diff.py	2011-01-19 18:40:15 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -389,8 +389,7 @@
         # subprocess.py that we had to workaround).
         # However, if 'diff' may not be available
         self.make_example_branch()
-        # this will be automatically restored by the base bzr test class
-        os.environ['BZR_PROGRESS_BAR'] = 'none'
+        self.overrideEnv('BZR_PROGRESS_BAR', 'none')
         out, err = self.run_bzr_subprocess('diff -r 1 --diff-options -ub',
                                            universal_newlines=True,
                                            retcode=None)

=== modified file 'bzrlib/tests/blackbox/test_dpush.py'
--- bzrlib/tests/blackbox/test_dpush.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/blackbox/test_dpush.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/blackbox/test_exceptions.py'
--- bzrlib/tests/blackbox/test_exceptions.py	2010-11-06 15:32:00 +0000
+++ bzrlib/tests/blackbox/test_exceptions.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
 
 """Tests for display of exceptions."""
 
-from cStringIO import StringIO
-import os
 import sys
 
 from bzrlib import (
@@ -27,11 +25,9 @@
     osutils,
     repository,
     tests,
-    trace,
     )
 
-from bzrlib.tests import TestCaseInTempDir, TestCase
-from bzrlib.errors import NotBranchError
+from bzrlib.tests import TestCase
 
 
 class TestExceptionReporting(TestCase):

=== modified file 'bzrlib/tests/blackbox/test_help.py'
--- bzrlib/tests/blackbox/test_help.py	2010-08-29 14:32:45 +0000
+++ bzrlib/tests/blackbox/test_help.py	2011-01-12 00:58:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -104,6 +104,10 @@
         self.assertEquals(dash_help, qmark_long)
         self.assertEquals(dash_help, qmark_cmds)
 
+    def test_help_width_zero(self):
+        self.overrideEnv('BZR_COLUMNS', '0')
+        self.run_bzr('help commands')
+
     def test_hidden(self):
         help_commands = self.run_bzr('help commands')[0]
         help_hidden = self.run_bzr('help hidden-commands')[0]

=== modified file 'bzrlib/tests/blackbox/test_init.py'
--- bzrlib/tests/blackbox/test_init.py	2010-10-12 09:46:37 +0000
+++ bzrlib/tests/blackbox/test_init.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -226,9 +226,9 @@
         # and uses whoami only in a lock file. Without whoami the login name
         # is used. This test is to ensure that init passes even when whoami
         # is not available.
-        osutils.set_or_unset_env('EMAIL', None)
-        osutils.set_or_unset_env('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
         out, err = self.run_bzr(['init', 'foo'])
         self.assertEqual(err, '')
         self.assertTrue(os.path.exists('foo'))
-        
+

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@
 
         self.overrideAttr(osutils, '_cached_user_encoding', self.encoding)
         email = self.info['committer'] + ' <joe@foo.com>'
-        os.environ['BZR_EMAIL'] = email.encode(osutils.get_user_encoding())
+        self.overrideEnv('BZR_EMAIL', email.encode(osutils.get_user_encoding()))
         self.create_base()
 
     def run_bzr_decode(self, args, encoding=None, fail=False, retcode=None,

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2010-12-14 09:21:14 +0000
+++ bzrlib/tests/blackbox/test_push.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/blackbox/test_resolve.py'
--- bzrlib/tests/blackbox/test_resolve.py	2010-11-10 18:17:06 +0000
+++ bzrlib/tests/blackbox/test_resolve.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/blackbox/test_send.py'
--- bzrlib/tests/blackbox/test_send.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/blackbox/test_send.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 # Authors: Aaron Bentley
 #
 # This program is free software; you can redistribute it and/or modify

=== modified file 'bzrlib/tests/blackbox/test_serve.py'
--- bzrlib/tests/blackbox/test_serve.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/blackbox/test_serve.py	2011-01-14 22:12:08 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,16 +18,12 @@
 """Tests of the bzr serve command."""
 
 import os
-import os.path
 import signal
-import subprocess
-import sys
 import thread
 import threading
 
 from bzrlib import (
     builtins,
-    debug,
     errors,
     osutils,
     revision as _mod_revision,
@@ -37,11 +33,13 @@
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.smart import client, medium
-from bzrlib.smart.server import BzrServerFactory, SmartTCPServer
+from bzrlib.smart.server import (
+    BzrServerFactory,
+    SmartTCPServer,
+    )
 from bzrlib.tests import (
     TestCaseWithMemoryTransport,
     TestCaseWithTransport,
-    TestSkipped,
     )
 from bzrlib.trace import mutter
 from bzrlib.transport import remote
@@ -53,9 +51,9 @@
                                 *func_args, **func_kwargs):
         """Run 'bzr serve', and run the given func in a thread once the server
         has started.
-        
+
         When 'func' terminates, the server will be terminated too.
-        
+
         Returns stdout and stderr.
         """
         # install hook
@@ -164,7 +162,7 @@
         url = 'bzr://localhost:%d/' % port
         self.permit_url(url)
         return process, url
-    
+
     def test_bzr_serve_quiet(self):
         self.make_branch('.')
         args = ['--port', 'localhost:0', '--quiet']
@@ -225,7 +223,7 @@
         # -Dhpss, and does drop some hpss logging to the file.
         self.make_branch('.')
         log_fname = os.getcwd() + '/server.log'
-        self._captureVar('BZR_LOG', log_fname)
+        self.overrideEnv('BZR_LOG', log_fname)
         process, transport = self.start_server_inet(['-Dhpss'])
         branch = BzrDir.open_from_transport(transport).open_branch()
         self.make_read_requests(branch)
@@ -334,4 +332,3 @@
         self.assertEqual(base_url, self.bzr_serve_transport.base)
         self.assertEqual(base_dir,
             server_maker.get_base_path(self.bzr_serve_transport))
-

=== modified file 'bzrlib/tests/blackbox/test_shared_repository.py'
--- bzrlib/tests/blackbox/test_shared_repository.py	2010-05-05 14:02:53 +0000
+++ bzrlib/tests/blackbox/test_shared_repository.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -155,8 +155,8 @@
         # and uses whoami only in a lock file. Without whoami the login name
         # is used. This test is to ensure that init-repo passes even when whoami
         # is not available.
-        osutils.set_or_unset_env('EMAIL', None)
-        osutils.set_or_unset_env('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
         out, err = self.run_bzr(['init-repo', 'foo'])
         self.assertEqual(err, '')
         self.assertTrue(os.path.exists('foo'))

=== modified file 'bzrlib/tests/blackbox/test_tags.py'
--- bzrlib/tests/blackbox/test_tags.py	2010-11-18 00:22:24 +0000
+++ bzrlib/tests/blackbox/test_tags.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -278,6 +278,40 @@
             error_regexes=["bzr: ERROR: Requested revision: '123.123' "
                 "does not exist in branch:"])
 
+    def test_sort_tags_custom(self):
+        def sort_by_dots(branch, tags):
+            def sort_key((tag, revid)):
+                return tag.count(".")
+            tags.sort(key=sort_key)
+
+        # Register a custom sort method
+        tag.tag_sort_methods.register("dots", sort_by_dots, "Sort by dots.")
+        self.addCleanup(tag.tag_sort_methods.remove, "dots")
+
+        tree1 = self.make_branch_and_tree('branch1')
+        tree1.commit(allow_pointless=True, message='revision 1',
+                rev_id='revid-1', timestamp=10)
+        tree1.commit(allow_pointless=True, message='revision 2',
+                rev_id='revid-2', timestamp=15)
+
+        b1 = tree1.branch
+
+        b1.tags.set_tag(u'tag..', 'revid-2')
+        b1.tags.set_tag(u'tag....', 'missing') # not present in repository
+        b1.tags.set_tag(u'tag.', 'revid-1')
+        b1.tags.set_tag(u'tag...', 'revid-1')
+        b1.tags.set_tag(u'tag....', 'revid-1')
+
+        # sorted by number of dots
+        out, err = self.run_bzr('tags --sort=dots -d branch1')
+        self.assertEquals(err, '')
+        self.assertEquals([
+            'tag.                 1',
+            'tag..                2',
+            'tag...               1',
+            'tag....              1'],
+            out.splitlines())
+
     def _check_tag_filter(self, argstr, expected_revnos):
         #upper bound of laziness
         out, err = self.run_bzr('tags ' + argstr)

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -315,37 +315,28 @@
         cmd_name = 'test-command'
         if sys.platform == 'win32':
             cmd_name += '.bat'
-        oldpath = os.environ.get('BZRPATH', None)
-        try:
-            if 'BZRPATH' in os.environ:
-                del os.environ['BZRPATH']
-
-            f = file(cmd_name, 'wb')
-            if sys.platform == 'win32':
-                f.write('@echo off\n')
-            else:
-                f.write('#!/bin/sh\n')
-            # f.write('echo Hello from test-command')
-            f.close()
-            os.chmod(cmd_name, 0755)
-
-            # It should not find the command in the local
-            # directory by default, since it is not in my path
-            self.run_bzr(cmd_name, retcode=3)
-
-            # Now put it into my path
-            os.environ['BZRPATH'] = '.'
-
-            self.run_bzr(cmd_name)
-
-            # Make sure empty path elements are ignored
-            os.environ['BZRPATH'] = os.pathsep
-
-            self.run_bzr(cmd_name, retcode=3)
-
-        finally:
-            if oldpath:
-                os.environ['BZRPATH'] = oldpath
+        self.overrideEnv('BZRPATH', None)
+
+        f = file(cmd_name, 'wb')
+        if sys.platform == 'win32':
+            f.write('@echo off\n')
+        else:
+            f.write('#!/bin/sh\n')
+        # f.write('echo Hello from test-command')
+        f.close()
+        os.chmod(cmd_name, 0755)
+
+        # It should not find the command in the local
+        # directory by default, since it is not in my path
+        self.run_bzr(cmd_name, retcode=3)
+
+        # Now put it into my path
+        self.overrideEnv('BZRPATH', '.')
+        self.run_bzr(cmd_name)
+
+        # Make sure empty path elements are ignored
+        self.overrideEnv('BZRPATH', os.pathsep)
+        self.run_bzr(cmd_name, retcode=3)
 
 
 def listdir_sorted(dir):

=== modified file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	2010-08-02 14:09:16 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2011-01-13 15:54:03 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -48,42 +48,50 @@
 
     def make_format_5_branch(self):
         # setup a format 5 branch we can upgrade from.
-        self.make_branch_and_tree('format_5_branch',
-                                  format=bzrdir.BzrDirFormat5())
+        path = 'format_5_branch'
+        self.make_branch_and_tree(path, format=bzrdir.BzrDirFormat5())
+        return path
 
     def make_metadir_weave_branch(self):
         self.make_branch_and_tree('metadir_weave_branch', format='metaweave')
 
     def test_readonly_url_error(self):
-        self.make_format_5_branch()
+        path = self.make_format_5_branch()
         (out, err) = self.run_bzr(
-            ['upgrade', self.get_readonly_url('format_5_branch')], retcode=3)
-        self.assertEqual(out, "")
-        self.assertEqual(err, "bzr: ERROR: Upgrade URL cannot work with readonly URLs.\n")
+            ['upgrade', self.get_readonly_url(path)], retcode=3)
+        err_msg = 'Upgrade URL cannot work with readonly URLs.'
+        self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
+                             % (err_msg, err_msg),
+                             err)
 
     def test_upgrade_up_to_date(self):
         self.make_current_format_branch_and_checkout()
         # when up to date we should get a message to that effect
         (out, err) = self.run_bzr('upgrade current_format_branch', retcode=3)
-        self.assertEqual("", out)
-        self.assertEqualDiff("bzr: ERROR: The branch format Meta "
-                             "directory format 1 is already at the most "
-                             "recent format.\n", err)
+        err_msg = ('The branch format %s is already at the most recent format.'
+                   % ('Meta directory format 1'))
+        self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
+                             % (err_msg, err_msg),
+                             err)
 
     def test_upgrade_up_to_date_checkout_warns_branch_left_alone(self):
         self.make_current_format_branch_and_checkout()
         # when upgrading a checkout, the branch location and a suggestion
         # to upgrade it should be emitted even if the checkout is up to
         # date
+        burl = self.get_transport('current_format_branch').base
+        curl = self.get_transport('current_format_checkout').base
         (out, err) = self.run_bzr('upgrade current_format_checkout', retcode=3)
-        self.assertEqual("This is a checkout. The branch (%s) needs to be "
-                         "upgraded separately.\n"
-                         % transport.get_transport(
-                self.get_url('current_format_branch')).base,
-                         out)
-        self.assertEqualDiff("bzr: ERROR: The branch format Meta "
-                             "directory format 1 is already at the most "
-                             "recent format.\n", err)
+        self.assertEqual(
+            'Upgrading branch %s ...\nThis is a checkout.'
+            ' The branch (%s) needs to be upgraded separately.\n'
+            % (curl, burl),
+            out)
+        msg = 'The branch format %s is already at the most recent format.' % (
+            'Meta directory format 1')
+        self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
+                             % (msg, msg),
+                             err)
 
     def test_upgrade_checkout(self):
         # upgrading a checkout should work
@@ -94,20 +102,21 @@
         # anonymous branch
         pass
 
-    def test_ugrade_branch_in_repo(self):
+    def test_upgrade_branch_in_repo(self):
         # upgrading a branch in a repo should warn about not upgrading the repo
         pass
 
     def test_upgrade_explicit_metaformat(self):
         # users can force an upgrade to metadir format.
-        self.make_format_5_branch()
-        url = transport.get_transport(self.get_url('format_5_branch')).base
+        path = self.make_format_5_branch()
+        url = transport.get_transport(self.get_url(path)).base
         # check --format takes effect
         controldir.ControlDirFormat._set_default_format(bzrdir.BzrDirFormat5())
         backup_dir = 'backup.bzr.~1~'
         (out, err) = self.run_bzr(
             ['upgrade', '--format=metaweave', url])
-        self.assertEqualDiff("""starting upgrade of %s
+        self.assertEqualDiff("""Upgrading branch %s ...
+starting upgrade of %s
 making backup of %s.bzr
   to %s%s
 starting upgrade from format 5 to 6
@@ -115,10 +124,10 @@
 adding prefixes to revision-store
 starting upgrade from format 6 to metadir
 finished
-""" % (url, url, url, backup_dir), out)
+""" % (url, url, url, url, backup_dir), out)
         self.assertEqualDiff("", err)
         self.assertTrue(isinstance(
-            bzrdir.BzrDir.open(self.get_url('format_5_branch'))._format,
+            bzrdir.BzrDir.open(self.get_url(path))._format,
             bzrdir.BzrDirMetaFormat1))
 
     def test_upgrade_explicit_knit(self):
@@ -131,13 +140,15 @@
         backup_dir = 'backup.bzr.~1~'
         (out, err) = self.run_bzr(
             ['upgrade', '--format=knit', url])
-        self.assertEqualDiff("""starting upgrade of %s
+        self.assertEqualDiff("""Upgrading branch %s ...
+starting upgrade of %s
 making backup of %s.bzr
   to %s%s
 starting repository conversion
 repository converted
 finished
-""" % (url, url, url, backup_dir), out)
+""" % (url, url, url, url, backup_dir),
+                             out)
         self.assertEqualDiff("", err)
         converted_dir = bzrdir.BzrDir.open(self.get_url('metadir_weave_branch'))
         self.assertTrue(isinstance(converted_dir._format,
@@ -149,6 +160,29 @@
         self.run_bzr('init-repository --format=metaweave repo')
         self.run_bzr('upgrade --format=knit repo')
 
+    def assertLegalOption(self, option_str):
+        # Confirm that an option is legal. (Lower level tests are
+        # expected to validate the actual functionality.)
+        self.run_bzr('init --format=pack-0.92 branch-foo')
+        self.run_bzr('upgrade --format=2a branch-foo %s' % (option_str,))
+
+    def assertBranchFormat(self, dir, format):
+        branch = bzrdir.BzrDir.open_tree_or_branch(self.get_url(dir))[1]
+        branch_format = branch._format
+        meta_format = bzrdir.format_registry.make_bzrdir(format)
+        expected_format = meta_format.get_branch_format()
+        self.assertEqual(expected_format, branch_format)
+
+    def test_upgrade_clean_supported(self):
+        self.assertLegalOption('--clean')
+        self.assertBranchFormat('branch-foo', '2a')
+        backup_bzr_dir = os.path.join("branch-foo", "backup.bzr.~1~")
+        self.assertFalse(os.path.exists(backup_bzr_dir))
+
+    def test_upgrade_dry_run_supported(self):
+        self.assertLegalOption('--dry-run')
+        self.assertBranchFormat('branch-foo', 'pack-0.92')
+
     def test_upgrade_permission_check(self):
         """'backup.bzr' should retain permissions of .bzr. Bug #262450"""
         self.requireFeature(features.posix_permissions_feature)
@@ -160,10 +194,9 @@
         new_perms = os.stat(backup_dir).st_mode & 0777
         self.assertTrue(new_perms == old_perms)
 
-
     def test_upgrade_with_existing_backup_dir(self):
-        self.make_format_5_branch()
-        t = transport.get_transport(self.get_url('format_5_branch'))
+        path = self.make_format_5_branch()
+        t = transport.get_transport(self.get_url(path))
         url = t.base
         controldir.ControlDirFormat._set_default_format(bzrdir.BzrDirFormat5())
         backup_dir1 = 'backup.bzr.~1~'
@@ -173,7 +206,8 @@
         t.mkdir(backup_dir1)
         (out, err) = self.run_bzr(
             ['upgrade', '--format=metaweave', url])
-        self.assertEqualDiff("""starting upgrade of %s
+        self.assertEqualDiff("""Upgrading branch %s ...
+starting upgrade of %s
 making backup of %s.bzr
   to %s%s
 starting upgrade from format 5 to 6
@@ -181,10 +215,10 @@
 adding prefixes to revision-store
 starting upgrade from format 6 to metadir
 finished
-""" % (url, url, url, backup_dir2), out)
+""" % (url, url, url, url, backup_dir2), out)
         self.assertEqualDiff("", err)
         self.assertTrue(isinstance(
-            bzrdir.BzrDir.open(self.get_url('format_5_branch'))._format,
+            bzrdir.BzrDir.open(self.get_url(path))._format,
             bzrdir.BzrDirMetaFormat1))
         self.assertTrue(t.has(backup_dir2))
 
@@ -197,14 +231,15 @@
         url = t.base
         out, err = self.run_bzr(['upgrade', '--format=knit', url])
         backup_dir = 'backup.bzr.~1~'
-        self.assertEqualDiff("""starting upgrade of %s
+        self.assertEqualDiff("""Upgrading branch %s ...
+starting upgrade of %s
 making backup of %s.bzr
   to %s%s
 starting upgrade from format 6 to metadir
 starting repository conversion
 repository converted
 finished
-""" % (url, url, url,backup_dir), out)
+""" % (url, url, url, url,backup_dir), out)
         self.assertEqual('', err)
 
 

=== modified file 'bzrlib/tests/blackbox/test_version.py'
--- bzrlib/tests/blackbox/test_version.py	2009-12-02 16:43:09 +0000
+++ bzrlib/tests/blackbox/test_version.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008 Canonical Ltd
+# Copyright (C) 2007, 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -95,7 +95,7 @@
             raise TestSkipped('Cannot find a unicode character that works in'
                               ' encoding %s' % (osutils.get_user_encoding(),))
 
-        osutils.set_or_unset_env('BZR_HOME', str_val)
+        self.overrideEnv('BZR_HOME', str_val)
         self.permit_source_tree_branch_repo()
         out = self.run_bzr("version")[0]
         self.assertTrue(len(out) > 0)
@@ -106,7 +106,7 @@
 
     def test_simple(self):
         bzr_log = 'my.bzr.log'
-        osutils.set_or_unset_env('BZR_LOG', bzr_log)
+        self.overrideEnv('BZR_LOG', bzr_log)
         default_log = os.path.join(os.environ['BZR_HOME'], '.bzr.log')
         self.failIfExists([default_log, bzr_log])
         out = self.run_bzr_subprocess('version')[0]
@@ -122,7 +122,7 @@
             bzr_log = 'NUL'
         else:
             bzr_log = '/dev/null'
-        osutils.set_or_unset_env('BZR_LOG', bzr_log)
+        self.overrideEnv('BZR_LOG', bzr_log)
         default_log = os.path.join(os.environ['BZR_HOME'], '.bzr.log')
         self.failIfExists(default_log)
         out = self.run_bzr_subprocess('version')[0]

=== modified file 'bzrlib/tests/blackbox/test_whoami.py'
--- bzrlib/tests/blackbox/test_whoami.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_whoami.py	2011-01-18 22:30:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,11 +17,8 @@
 
 """Black-box tests for bzr whoami."""
 
-import os
-
 import bzrlib
 from bzrlib import (
-    osutils,
     config,
     )
 from bzrlib.tests import TestCaseWithTransport
@@ -39,6 +36,11 @@
         self.assertTrue(len(out) > 0)
         self.assertEquals(1, out.count('@'))
 
+    def test_whoami_email_arg(self):
+        # whoami --email is mutually exclusive with any arguments
+        out = self.run_bzr("whoami --email 'foo <foo@example.com>'", 3)[0]
+        self.assertEquals("", out)
+
     def test_whoami_branch(self):
         """branch specific user identity works."""
         wt = self.make_branch_and_tree('.')
@@ -52,7 +54,7 @@
 
         # Verify that the environment variable overrides the value
         # in the file
-        os.environ['BZR_EMAIL'] = 'Different ID <other@environ.ment>'
+        self.overrideEnv('BZR_EMAIL', 'Different ID <other@environ.ment>')
         whoami = self.run_bzr("whoami")[0]
         self.assertEquals('Different ID <other@environ.ment>\n', whoami)
         whoami_email = self.run_bzr("whoami --email")[0]
@@ -94,8 +96,8 @@
     def test_whoami_not_set(self):
         """Ensure whoami error if username is not set.
         """
-        osutils.set_or_unset_env('EMAIL', None)
-        osutils.set_or_unset_env('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
         out, err = self.run_bzr(['whoami'], 3)
         self.assertContainsRe(err, 'Unable to determine your name')
 

=== modified file 'bzrlib/tests/doc_generate/__init__.py'
--- bzrlib/tests/doc_generate/__init__.py	2010-12-09 21:34:01 +0000
+++ bzrlib/tests/doc_generate/__init__.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/doc_generate/builders/test_texinfo.py'
--- bzrlib/tests/doc_generate/builders/test_texinfo.py	2010-12-09 15:04:24 +0000
+++ bzrlib/tests/doc_generate/builders/test_texinfo.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/doc_generate/writers/test_texinfo.py'
--- bzrlib/tests/doc_generate/writers/test_texinfo.py	2010-12-09 15:04:24 +0000
+++ bzrlib/tests/doc_generate/writers/test_texinfo.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/features.py'
--- bzrlib/tests/features.py	2010-10-11 09:53:16 +0000
+++ bzrlib/tests/features.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 import stat
 
 from bzrlib import tests
-from bzrlib.symbol_versioning import deprecated_in
 
 
 class _NotRunningAsRoot(tests.Feature):

=== modified file 'bzrlib/tests/http_server.py'
--- bzrlib/tests/http_server.py	2010-08-24 13:01:06 +0000
+++ bzrlib/tests/http_server.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,23 +20,12 @@
 import posixpath
 import random
 import re
-import select
 import SimpleHTTPServer
 import socket
-import SocketServer
-import sys
-import threading
-import time
 import urllib
 import urlparse
 
-from bzrlib import (
-    osutils,
-    tests,
-    transport,
-    )
 from bzrlib.tests import test_server
-from bzrlib.transport import local
 
 
 class BadWebserverPath(ValueError):

=== modified file 'bzrlib/tests/http_utils.py'
--- bzrlib/tests/http_utils.py	2010-06-23 16:43:16 +0000
+++ bzrlib/tests/http_utils.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,13 +15,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from cStringIO import StringIO
-import errno
 import re
-import socket
-import threading
-import time
 import urllib2
-import urlparse
 
 
 from bzrlib import (
@@ -32,7 +27,6 @@
     )
 from bzrlib.smart import (
     medium,
-    protocol,
     )
 from bzrlib.tests import http_server
 from bzrlib.transport import chroot

=== modified file 'bzrlib/tests/https_server.py'
--- bzrlib/tests/https_server.py	2010-12-07 16:18:50 +0000
+++ bzrlib/tests/https_server.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/per_branch/test_branch.py'
--- bzrlib/tests/per_branch/test_branch.py	2010-12-07 09:06:39 +0000
+++ bzrlib/tests/per_branch/test_branch.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/per_branch/test_locking.py'
--- bzrlib/tests/per_branch/test_locking.py	2010-06-11 07:59:43 +0000
+++ bzrlib/tests/per_branch/test_locking.py	2011-01-13 05:30:51 +0000
@@ -17,7 +17,6 @@
 """Test locks across all branch implemenations"""
 
 from bzrlib import (
-    branch as _mod_branch,
     errors,
     tests,
     )
@@ -501,10 +500,6 @@
         self.assertThat(branch.lock_read, ReturnsUnlockable(branch))
 
     def test_lock_write_locks_repo_too(self):
-        if isinstance(self.branch_format, _mod_branch.BzrBranchFormat4):
-            # Branch format 4 is combined with the repository, so this test
-            # doesn't apply.
-            return
         branch = self.make_branch('b')
         branch = branch.bzrdir.open_branch()
         branch.lock_write()

=== modified file 'bzrlib/tests/per_branch/test_parent.py'
--- bzrlib/tests/per_branch/test_parent.py	2010-12-07 09:06:39 +0000
+++ bzrlib/tests/per_branch/test_parent.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/per_branch/test_push.py'
--- bzrlib/tests/per_branch/test_push.py	2010-09-09 00:32:58 +0000
+++ bzrlib/tests/per_branch/test_push.py	2011-01-14 20:52:48 +0000
@@ -24,11 +24,9 @@
     builtins,
     bzrdir,
     check,
-    debug,
     errors,
     memorytree,
     push,
-    repository,
     revision,
     symbol_versioning,
     tests,
@@ -36,8 +34,6 @@
     )
 from bzrlib.smart import (
     client,
-    server,
-    repository as _mod_smart_repo,
     )
 from bzrlib.tests import (
     per_branch,
@@ -174,6 +170,21 @@
         self.assertEqual(tree.branch.last_revision(),
                          to_branch.last_revision())
 
+    def test_push_overwrite_with_older_mainline_rev(self):
+        """Pushing an older mainline revision with overwrite.
+
+        This was <https://bugs.launchpad.net/bzr/+bug/386576>.
+        """
+        source = self.make_branch_and_tree('source')
+        target = self.make_branch('target')
+
+        source.commit('1st commit')
+        source.commit('2nd commit', rev_id='rev-2')
+        source.commit('3rd commit')
+        source.branch.push(target)
+        source.branch.push(target, stop_revision='rev-2', overwrite=True)
+        self.assertEqual('rev-2', target.last_revision())
+
     def test_push_overwrite_of_non_tip_with_stop_revision(self):
         """Combining the stop_revision and overwrite options works.
 

=== modified file 'bzrlib/tests/per_branch/test_sprout.py'
--- bzrlib/tests/per_branch/test_sprout.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_branch/test_sprout.py	2011-01-13 05:36:39 +0000
@@ -182,9 +182,9 @@
                 source.last_revision(), possible_transports=[target_transport],
                 source_branch=source, stacked=True)
         except errors.UnstackableBranchFormat:
-            if isinstance(self.branch_format, _mod_branch.BzrBranchFormat4):
-                raise tests.KnownFailure(
-                    "Format 4 doesn't auto stack successfully.")
+            if not self.branch_format.supports_stacking():
+                raise tests.TestNotApplicable(
+                    "Format doesn't auto stack successfully.")
             else:
                 raise
         result = dir.open_branch()

=== modified file 'bzrlib/tests/per_branch/test_stacking.py'
--- bzrlib/tests/per_branch/test_stacking.py	2010-07-01 03:41:20 +0000
+++ bzrlib/tests/per_branch/test_stacking.py	2011-01-13 01:02:53 +0000
@@ -154,8 +154,6 @@
         self.assertRevisionInRepository('newbranch', new_branch_revid)
 
     def test_sprout_stacked_from_smart_server(self):
-        if isinstance(self.branch_format, branch.BzrBranchFormat4):
-            raise TestNotApplicable('Branch format 4 is not usable via HPSS.')
         # We have a mainline
         trunk_tree = self.make_branch_and_tree('mainline')
         trunk_revid = trunk_tree.commit('mainline')

=== modified file 'bzrlib/tests/per_branch/test_update.py'
--- bzrlib/tests/per_branch/test_update.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_update.py	2011-01-10 21:31:59 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +16,10 @@
 
 
 from bzrlib import (
+    branch,
     errors,
     revision as _mod_revision,
+    tests,
     )
 from bzrlib.tests import per_branch
 
@@ -115,3 +117,23 @@
 
         tree1.branch.update_revisions(tree2.branch, overwrite=True)
         self.assertEqual((1, rev1), tree1.branch.last_revision_info())
+
+    def test_update_in_checkout_of_readonly(self):
+        tree1 = self.make_branch_and_tree('tree1')
+        rev1 = tree1.commit('one')
+        try:
+            tree1.branch.tags.set_tag('test-tag', rev1)
+        except errors.TagsNotSupported:
+            # Tags not supported
+            raise tests.TestNotApplicable("only triggered from branches with"
+                " tags")
+        readonly_branch1 = branch.Branch.open('readonly+' + tree1.branch.base)
+        tree2 = tree1.bzrdir.sprout('tree2').open_workingtree()
+        try:
+            tree2.branch.bind(readonly_branch1)
+        except errors.UpgradeRequired:
+            # old branch, cant test.
+            raise tests.TestNotApplicable("only triggered in bound branches")
+        rev2 = tree1.commit('two')
+        tree2.update()
+        self.assertEqual(rev2, tree2.branch.last_revision())

=== modified file 'bzrlib/tests/per_controldir/test_controldir.py'
--- bzrlib/tests/per_controldir/test_controldir.py	2010-11-26 17:59:08 +0000
+++ bzrlib/tests/per_controldir/test_controldir.py	2011-01-13 05:28:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 """Tests for control directory implementations - tests a controldir format."""
 
 from itertools import izip
-import os
 
 import bzrlib.branch
 from bzrlib import (
@@ -34,16 +33,19 @@
     )
 import bzrlib.revision
 from bzrlib.tests import (
-                          ChrootedTestCase,
-                          TestNotApplicable,
-                          TestSkipped,
-                          )
+    ChrootedTestCase,
+    TestNotApplicable,
+    TestSkipped,
+    )
 from bzrlib.tests.per_controldir import TestCaseWithControlDir
 from bzrlib.transport.local import LocalTransport
 from bzrlib.ui import (
     CannedInputUIFactory,
     )
-from bzrlib.remote import RemoteBzrDir, RemoteRepository
+from bzrlib.remote import (
+    RemoteBzrDir,
+    RemoteRepository,
+    )
 from bzrlib.repofmt import weaverepo
 
 
@@ -257,7 +259,7 @@
 
     def test_clone_bzrdir_branch_and_repo_fixed_user_id(self):
         # Bug #430868 is about an email containing '.sig'
-        os.environ['BZR_EMAIL'] = 'murphy@host.sighup.org'
+        self.overrideEnv('BZR_EMAIL', 'murphy@host.sighup.org')
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -755,12 +757,11 @@
         tree.commit('revision 1', rev_id='1')
         tree.commit('revision 2', rev_id='2', allow_pointless=True)
         dir = tree.bzrdir
-        if isinstance(dir, (bzrdir.BzrDirPreSplitOut,)):
-            self.assertRaises(errors.MustHaveWorkingTree, dir.sprout,
-                              self.get_url('target'),
-                              create_tree_if_local=False)
-            return
-        target = dir.sprout(self.get_url('target'), create_tree_if_local=False)
+        try:
+            target = dir.sprout(self.get_url('target'),
+                create_tree_if_local=False)
+        except errors.MustHaveWorkingTree:
+            raise TestNotApplicable("control dir format requires working tree")
         self.failIfExists('target/foo')
         self.assertEqual(tree.branch.last_revision(),
                          target.open_branch().last_revision())
@@ -1206,7 +1207,7 @@
         self.assertTrue(isinstance(dir.get_repository_transport(None),
                                    transport.Transport))
         # with a given format, either the bzr dir supports identifiable
-        # repositories, or it supports anonymous  repository formats, but not both.
+        # repositories, or it supports anonymous repository formats, but not both.
         anonymous_format = weaverepo.RepositoryFormat6()
         identifiable_format = weaverepo.RepositoryFormat7()
         try:

=== modified file 'bzrlib/tests/per_interbranch/test_push.py'
--- bzrlib/tests/per_interbranch/test_push.py	2010-09-07 23:32:42 +0000
+++ bzrlib/tests/per_interbranch/test_push.py	2011-01-13 05:42:30 +0000
@@ -26,10 +26,8 @@
 
 from bzrlib import (
     branch,
-    builtins,
     bzrdir,
     check,
-    debug,
     errors,
     push,
     repository,
@@ -40,7 +38,6 @@
 from bzrlib.bzrdir import BzrDir
 from bzrlib.memorytree import MemoryTree
 from bzrlib.revision import NULL_REVISION
-from bzrlib.smart import client, server
 from bzrlib.smart.repository import SmartServerRepositoryGetParentMap
 from bzrlib.tests.per_interbranch import (
     TestCaseWithInterBranch,
@@ -204,8 +201,6 @@
         default for the branch), and will be stacked when the repo format
         allows (which means that the branch format isn't necessarly preserved).
         """
-        if isinstance(self.branch_format_from, branch.BzrBranchFormat4):
-            raise tests.TestNotApplicable('Not a metadir format.')
         if isinstance(self.branch_format_from, branch.BranchReferenceFormat):
             # This test could in principle apply to BranchReferenceFormat, but
             # make_branch_builder doesn't support it.

=== modified file 'bzrlib/tests/per_interrepository/test_interrepository.py'
--- bzrlib/tests/per_interrepository/test_interrepository.py	2009-07-10 06:46:10 +0000
+++ bzrlib/tests/per_interrepository/test_interrepository.py	2011-01-14 17:18:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,17 +19,11 @@
 import sys
 
 import bzrlib
-import bzrlib.bzrdir as bzrdir
-from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 import bzrlib.errors as errors
 import bzrlib.gpg
 from bzrlib.inventory import Inventory
-import bzrlib.repofmt.weaverepo as weaverepo
-import bzrlib.repository as repository
-from bzrlib.revision import NULL_REVISION, Revision
+from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import (
-    TestCase,
-    TestCaseWithTransport,
     TestNotApplicable,
     TestSkipped,
     )
@@ -64,12 +58,9 @@
 
 
 def check_repo_format_for_funky_id_on_win32(repo):
-    if (isinstance(repo, (weaverepo.AllInOneRepository,
-                          weaverepo.WeaveMetaDirRepository))
-        and sys.platform == 'win32'):
-            raise TestSkipped("funky chars does not permitted"
-                              " on this platform in repository"
-                              " %s" % repo.__class__.__name__)
+    if not repo._format.supports_funky_characters and sys.platform == 'win32':
+        raise TestSkipped("funky chars not allowed on this platform in repository"
+                          " %s" % repo.__class__.__name__)
 
 
 class TestInterRepository(TestCaseWithInterRepository):
@@ -110,7 +101,8 @@
         # and sign 'rev2'
         tree_a.branch.repository.lock_write()
         tree_a.branch.repository.start_write_group()
-        tree_a.branch.repository.sign_revision('rev2', bzrlib.gpg.LoopbackGPGStrategy(None))
+        tree_a.branch.repository.sign_revision('rev2',
+            bzrlib.gpg.LoopbackGPGStrategy(None))
         tree_a.branch.repository.commit_write_group()
         tree_a.branch.repository.unlock()
 
@@ -139,9 +131,15 @@
         self.assertFalse(repo_b.has_revision('pizza'))
         # Asking specifically for an absent revision errors.
         self.assertRaises(errors.NoSuchRevision,
-            repo_b.search_missing_revision_ids, repo_a, revision_id='pizza',
+            repo_b.search_missing_revision_ids, repo_a, revision_ids=['pizza'],
             find_ghosts=True)
         self.assertRaises(errors.NoSuchRevision,
+            repo_b.search_missing_revision_ids, repo_a, revision_ids=['pizza'],
+            find_ghosts=False)
+        self.callDeprecated(
+            ['search_missing_revision_ids(revision_id=...) was deprecated in '
+             '2.3.  Use revision_ids=[...] instead.'],
+            self.assertRaises, errors.NoSuchRevision,
             repo_b.search_missing_revision_ids, repo_a, revision_id='pizza',
             find_ghosts=False)
 
@@ -151,7 +149,8 @@
         # make a repository to compare against that is empty
         repo_b = self.make_to_repository('empty')
         repo_a = self.bzrdir.open_repository()
-        result = repo_b.search_missing_revision_ids(repo_a, revision_id='rev1')
+        result = repo_b.search_missing_revision_ids(
+            repo_a, revision_ids=['rev1'])
         self.assertEqual(set(['rev1']), result.get_keys())
         self.assertEqual(('search', set(['rev1']), set([NULL_REVISION]), 1),
             result.get_recipe())

=== modified file 'bzrlib/tests/per_merger.py'
--- bzrlib/tests/per_merger.py	2010-03-05 08:55:12 +0000
+++ bzrlib/tests/per_merger.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
     errors,
     merge as _mod_merge,
     option,
-    progress,
     )
 from bzrlib.tests import (
     multiply_tests,

=== modified file 'bzrlib/tests/per_pack_repository.py'
--- bzrlib/tests/per_pack_repository.py	2010-05-06 23:41:35 +0000
+++ bzrlib/tests/per_pack_repository.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 These tests are repeated for all pack-based repository formats.
 """
 
-from cStringIO import StringIO
 from stat import S_ISDIR
 
 from bzrlib.btree_index import BTreeGraphIndex
@@ -29,35 +28,24 @@
     errors,
     inventory,
     osutils,
-    progress,
     repository,
     revision as _mod_revision,
-    symbol_versioning,
     tests,
     ui,
-    upgrade,
-    workingtree,
-    )
-from bzrlib.repofmt import (
-    pack_repo,
-    groupcompress_repo,
     )
 from bzrlib.repofmt.groupcompress_repo import RepositoryFormat2a
 from bzrlib.smart import (
     client,
     )
 from bzrlib.tests import (
-    TestCase,
     TestCaseWithTransport,
     TestNotApplicable,
-    TestSkipped,
     )
 from bzrlib.transport import (
     get_transport,
     memory,
     )
 from bzrlib.tests import test_server
-from bzrlib.tests.per_repository import TestCaseWithRepository
 
 
 class TestPackRepository(TestCaseWithTransport):

=== modified file 'bzrlib/tests/per_repository/test_check.py'
--- bzrlib/tests/per_repository/test_check.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/per_repository/test_check.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -135,7 +135,7 @@
         repo.lock_write()
         self.addCleanup(repo.unlock)
         config = _mod_config.Config()
-        os.environ['BZR_EMAIL'] = 'foo@sample.com'
+        self.overrideEnv('BZR_EMAIL', 'foo@sample.com')
         builder = repo.get_commit_builder(None, [], config)
         list(builder.record_iter_changes(None, _mod_revision.NULL_REVISION, [
             ('TREE_ROOT', (None, ''), True, (False, True), (None, None),

=== modified file 'bzrlib/tests/per_repository/test_commit_builder.py'
--- bzrlib/tests/per_repository/test_commit_builder.py	2010-10-15 16:43:03 +0000
+++ bzrlib/tests/per_repository/test_commit_builder.py	2011-01-12 16:41:08 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1300,8 +1300,8 @@
 
     def test_stacked_repositories_reject_commit_builder(self):
         # As per bug 375013, committing to stacked repositories is currently
-        # broken, so repositories with fallbacks refuse to hand out a commit
-        # builder.
+        # broken if we aren't in a chk repository. So old repositories with
+        # fallbacks refuse to hand out a commit builder.
         repo_basis = self.make_repository('basis')
         branch = self.make_branch('local')
         repo_local = branch.repository
@@ -1309,15 +1309,19 @@
             repo_local.add_fallback_repository(repo_basis)
         except errors.UnstackableRepositoryFormat:
             raise tests.TestNotApplicable("not a stackable format.")
-        repo_local.lock_write()
-        self.addCleanup(repo_local.unlock)
-        self.assertRaises(errors.BzrError, repo_local.get_commit_builder,
-            branch, [], branch.get_config())
+        self.addCleanup(repo_local.lock_write().unlock)
+        if not repo_local._format.supports_chks:
+            self.assertRaises(errors.BzrError, repo_local.get_commit_builder,
+                branch, [], branch.get_config())
+        else:
+            builder = repo_local.get_commit_builder(branch, [],
+                                                    branch.get_config())
+            builder.abort()
 
     def test_committer_no_username(self):
         # Ensure that when no username is available but a committer is
         # supplied, commit works.
-        del os.environ['EMAIL']
+        self.overrideEnv('EMAIL', None)
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:

=== modified file 'bzrlib/tests/per_repository/test_repository.py'
--- bzrlib/tests/per_repository/test_repository.py	2010-05-06 23:41:35 +0000
+++ bzrlib/tests/per_repository/test_repository.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,10 +25,8 @@
     delta as _mod_delta,
     errors,
     gpg,
-    graph,
     info,
     inventory,
-    osutils,
     remote,
     repository,
     revision as _mod_revision,
@@ -37,7 +35,6 @@
     upgrade,
     versionedfile,
     workingtree,
-    xml_serializer,
     )
 from bzrlib.repofmt import (
     pack_repo,
@@ -854,9 +851,6 @@
 
     def test_sprout_from_hpss_preserves_format(self):
         """repo.sprout from a smart server preserves the repository format."""
-        if self.repository_format == weaverepo.RepositoryFormat7():
-            raise tests.TestNotApplicable(
-                "Cannot fetch weaves over smart protocol.")
         remote_repo = self.make_remote_repository('remote')
         local_bzrdir = self.make_bzrdir('local')
         try:

=== modified file 'bzrlib/tests/per_repository/test_write_group.py'
--- bzrlib/tests/per_repository/test_write_group.py	2010-02-11 09:27:55 +0000
+++ bzrlib/tests/per_repository/test_write_group.py	2011-01-10 23:52:14 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -363,6 +363,35 @@
         sink.insert_stream((), repo._format, tokens)
         self.assertEqual([True], call_log)
 
+    def test_insert_stream_without_locking_fails_without_lock(self):
+        repo = self.make_repository('test-repo')
+        sink = repo._get_sink()
+        stream = [('texts', [versionedfile.FulltextContentFactory(
+            ('file-id', 'rev-id'), (), None, 'lines\n')])]
+        self.assertRaises(errors.ObjectNotLocked,
+            sink.insert_stream_without_locking, stream, repo._format)
+
+    def test_insert_stream_without_locking_fails_without_write_group(self):
+        repo = self.make_repository('test-repo')
+        self.addCleanup(repo.lock_write().unlock)
+        sink = repo._get_sink()
+        stream = [('texts', [versionedfile.FulltextContentFactory(
+            ('file-id', 'rev-id'), (), None, 'lines\n')])]
+        self.assertRaises(errors.BzrError,
+            sink.insert_stream_without_locking, stream, repo._format)
+
+    def test_insert_stream_without_locking(self):
+        repo = self.make_repository('test-repo')
+        self.addCleanup(repo.lock_write().unlock)
+        repo.start_write_group()
+        sink = repo._get_sink()
+        stream = [('texts', [versionedfile.FulltextContentFactory(
+            ('file-id', 'rev-id'), (), None, 'lines\n')])]
+        missing_keys = sink.insert_stream_without_locking(stream, repo._format)
+        repo.commit_write_group()
+        self.assertEqual(set(), missing_keys)
+
+
 
 class TestResumeableWriteGroup(per_repository.TestCaseWithRepository):
 

=== modified file 'bzrlib/tests/per_repository_reference/__init__.py'
--- bzrlib/tests/per_repository_reference/__init__.py	2010-08-25 10:20:41 +0000
+++ bzrlib/tests/per_repository_reference/__init__.py	2011-01-10 22:47:59 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,12 +24,9 @@
 
 from bzrlib import (
     errors,
-    repository,
     remote,
     )
-from bzrlib.branch import BzrBranchFormat7
 from bzrlib.bzrdir import BzrDir
-from bzrlib.repofmt.pack_repo import RepositoryFormatKnitPack6
 from bzrlib.tests import multiply_tests
 from bzrlib.tests.per_repository import (
     all_repository_format_scenarios,
@@ -89,18 +86,8 @@
     result = []
     for test_name, scenario_info in all_repository_format_scenarios():
         format = scenario_info['repository_format']
-        if isinstance(format, remote.RemoteRepositoryFormat):
-            # This is a RemoteRepositoryFormat scenario.  Force the scenario to
-            # use real branch and repository formats that support references.
-            scenario_info = dict(scenario_info)
-            format = remote.RemoteRepositoryFormat()
-            format._custom_format = RepositoryFormatKnitPack6()
-            scenario_info['repository_format'] = format
-            bzrdir_format = remote.RemoteBzrDirFormat()
-            bzrdir_format.repository_format = format
-            bzrdir_format.set_branch_format(BzrBranchFormat7())
-            scenario_info['bzrdir_format'] = bzrdir_format
-        if format.supports_external_lookups:
+        if (isinstance(format, remote.RemoteRepositoryFormat)
+            or format.supports_external_lookups):
             result.append((test_name, scenario_info))
     return result
 
@@ -113,6 +100,7 @@
         'bzrlib.tests.per_repository_reference.test_all_revision_ids',
         'bzrlib.tests.per_repository_reference.test_break_lock',
         'bzrlib.tests.per_repository_reference.test_check',
+        'bzrlib.tests.per_repository_reference.test_commit_with_stacking',
         'bzrlib.tests.per_repository_reference.test_default_stacking',
         'bzrlib.tests.per_repository_reference.test_fetch',
         'bzrlib.tests.per_repository_reference.test_get_record_stream',

=== added file 'bzrlib/tests/per_repository_reference/test_commit_with_stacking.py'
--- bzrlib/tests/per_repository_reference/test_commit_with_stacking.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_reference/test_commit_with_stacking.py	2011-01-11 00:36:10 +0000
@@ -0,0 +1,220 @@
+# Copyright (C) 2010, 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+from bzrlib import (
+    errors,
+    remote,
+    tests,
+    urlutils,
+    )
+from bzrlib.tests.per_repository import TestCaseWithRepository
+
+
+class TestCaseWithStackedTarget(TestCaseWithRepository):
+
+    r1_key = ('rev1-id',)
+    r2_key = ('rev2-id',)
+
+    def make_stacked_target(self):
+        base_tree = self.make_branch_and_tree('base')
+        self.build_tree(['base/f1.txt'])
+        base_tree.add(['f1.txt'], ['f1.txt-id'])
+        base_tree.commit('initial', rev_id=self.r1_key[0])
+        self.build_tree(['base/f2.txt'])
+        base_tree.add(['f2.txt'], ['f2.txt-id'])
+        base_tree.commit('base adds f2', rev_id=self.r2_key[0])
+        stacked_url = urlutils.join(base_tree.branch.base, '../stacked')
+        stacked_bzrdir = base_tree.bzrdir.sprout(stacked_url,
+            stacked=True)
+        if isinstance(stacked_bzrdir, remote.RemoteBzrDir):
+            stacked_branch = stacked_bzrdir.open_branch()
+            stacked_tree = stacked_branch.create_checkout('stacked',
+                lightweight=True)
+        else:
+            stacked_tree = stacked_bzrdir.open_workingtree()
+        return base_tree, stacked_tree
+
+
+class TestCommitWithStacking(TestCaseWithStackedTarget):
+
+    def setUp(self):
+        super(TestCommitWithStacking, self).setUp()
+        format = self.repository_format
+        if (not (isinstance(format, remote.RemoteRepositoryFormat)
+                 or format.supports_chks)):
+            raise tests.TestNotApplicable('stacked commit only supported'
+                ' for chk repositories')
+
+    def get_only_repo(self, tree):
+        """Open just the repository used by this tree.
+
+        This returns a read locked Repository object without any stacking
+        fallbacks.
+        """
+        repo = tree.branch.repository.bzrdir.open_repository()
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        return repo
+
+    def assertPresent(self, expected, vf, keys):
+        """Check which of the supplied keys are present."""
+        parent_map = vf.get_parent_map(keys)
+        self.assertEqual(sorted(expected), sorted(parent_map))
+
+    def test_simple_commit(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        self.assertEqual(1,
+                len(stacked_tree.branch.repository._fallback_repositories))
+        self.build_tree_contents([('stacked/f1.txt', 'new content\n')])
+        stacked_tree.commit('new content', rev_id='new-rev-id')
+        # We open the repository without fallbacks to ensure the data is
+        # locally true
+        stacked_only_repo = self.get_only_repo(stacked_tree)
+        # We should have the immediate parent inventory available, but not the
+        # grandparent's
+        self.assertPresent([self.r2_key],
+            stacked_only_repo.inventories, [self.r1_key, self.r2_key])
+        # And we should be able to pull this revision into another stacked
+        # branch
+        stacked2_branch = base_tree.bzrdir.sprout('stacked2',
+                                                  stacked=True).open_branch()
+        stacked2_branch.repository.fetch(stacked_only_repo,
+                                         revision_id='new-rev-id')
+
+    def test_merge_commit(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        self.build_tree_contents([('base/f1.txt', 'new content\n')])
+        r3_key = ('rev3-id',)
+        base_tree.commit('second base', rev_id=r3_key[0])
+        to_be_merged_tree = base_tree.bzrdir.sprout('merged'
+            ).open_workingtree()
+        self.build_tree(['merged/f2.txt'])
+        to_be_merged_tree.add(['f2.txt'], ['f2.txt-id'])
+        to_merge_key = ('to-merge-rev-id',)
+        to_be_merged_tree.commit('new-to-be-merged', rev_id=to_merge_key[0])
+        stacked_tree.merge_from_branch(to_be_merged_tree.branch)
+        merged_key = ('merged-rev-id',)
+        stacked_tree.commit('merge', rev_id=merged_key[0])
+        # to-merge isn't in base, so it should be in stacked.
+        # rev3-id is a parent of a revision we have, so we should have the
+        # inventory, but not the revision.
+        # merged has a parent of r2, so we should also have r2's
+        # inventory-but-not-revision.
+        # Nothing has r1 directly, so we shouldn't have anything present for it
+        stacked_only_repo = self.get_only_repo(stacked_tree)
+        all_keys = [self.r1_key, self.r2_key, r3_key, to_merge_key, merged_key]
+        self.assertPresent([to_merge_key, merged_key],
+                           stacked_only_repo.revisions, all_keys)
+        self.assertPresent([self.r2_key, r3_key, to_merge_key, merged_key],
+                           stacked_only_repo.inventories, all_keys)
+
+    def test_merge_from_master(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        self.build_tree_contents([('base/f1.txt', 'new content\n')])
+        r3_key = ('rev3-id',)
+        base_tree.commit('second base', rev_id=r3_key[0])
+        stacked_tree.merge_from_branch(base_tree.branch)
+        merged_key = ('merged-rev-id',)
+        stacked_tree.commit('merge', rev_id=merged_key[0])
+        all_keys = [self.r1_key, self.r2_key, r3_key, merged_key]
+        # We shouldn't have any of the base revisions in the local repo, but we
+        # should have both base inventories.
+        stacked_only_repo = self.get_only_repo(stacked_tree)
+        self.assertPresent([merged_key],
+                           stacked_only_repo.revisions, all_keys)
+        self.assertPresent([self.r2_key, r3_key, merged_key],
+                           stacked_only_repo.inventories, all_keys)
+
+    def test_multi_stack(self):
+        """base + stacked + stacked-on-stacked"""
+        base_tree, stacked_tree = self.make_stacked_target()
+        self.build_tree(['stacked/f3.txt'])
+        stacked_tree.add(['f3.txt'], ['f3.txt-id'])
+        stacked_key = ('stacked-rev-id',)
+        stacked_tree.commit('add f3', rev_id=stacked_key[0])
+        stacked_only_repo = self.get_only_repo(stacked_tree)
+        self.assertPresent([self.r2_key], stacked_only_repo.inventories,
+                           [self.r1_key, self.r2_key])
+        # This ensures we get a Remote URL, rather than a local one.
+        stacked2_url = urlutils.join(base_tree.branch.base, '../stacked2')
+        stacked2_bzrdir = stacked_tree.bzrdir.sprout(stacked2_url,
+                            revision_id=self.r1_key[0],
+                            stacked=True)
+        if isinstance(stacked2_bzrdir, remote.RemoteBzrDir):
+            stacked2_branch = stacked2_bzrdir.open_branch()
+            stacked2_tree = stacked2_branch.create_checkout('stacked2',
+                lightweight=True)
+        else:
+            stacked2_tree = stacked2_bzrdir.open_workingtree()
+        # stacked2 is stacked on stacked, but its content is rev1, so
+        # it needs to pull the basis information from a fallback-of-fallback.
+        self.build_tree(['stacked2/f3.txt'])
+        stacked2_only_repo = self.get_only_repo(stacked2_tree)
+        self.assertPresent([], stacked2_only_repo.inventories,
+                           [self.r1_key, self.r2_key])
+        stacked2_tree.add(['f3.txt'], ['f3.txt-id'])
+        stacked2_tree.commit('add f3', rev_id='stacked2-rev-id')
+        # We added data to this read-locked repo, so refresh it
+        stacked2_only_repo.refresh_data()
+        self.assertPresent([self.r1_key], stacked2_only_repo.inventories,
+                           [self.r1_key, self.r2_key])
+
+    def test_commit_with_ghosts_fails(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        stacked_tree.set_parent_ids([stacked_tree.last_revision(),
+                                     'ghost-rev-id'])
+        self.assertRaises(errors.BzrError,
+            stacked_tree.commit, 'failed_commit')
+
+    def test_commit_with_ghost_in_ancestry(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        self.build_tree_contents([('base/f1.txt', 'new content\n')])
+        r3_key = ('rev3-id',)
+        base_tree.commit('second base', rev_id=r3_key[0])
+        to_be_merged_tree = base_tree.bzrdir.sprout('merged'
+            ).open_workingtree()
+        self.build_tree(['merged/f2.txt'])
+        to_be_merged_tree.add(['f2.txt'], ['f2.txt-id'])
+        ghost_key = ('ghost-rev-id',)
+        to_be_merged_tree.set_parent_ids([r3_key[0], ghost_key[0]])
+        to_merge_key = ('to-merge-rev-id',)
+        to_be_merged_tree.commit('new-to-be-merged', rev_id=to_merge_key[0])
+        stacked_tree.merge_from_branch(to_be_merged_tree.branch)
+        merged_key = ('merged-rev-id',)
+        stacked_tree.commit('merge', rev_id=merged_key[0])
+        # vs test_merge_commit, the fetch for 'merge_from_branch' should
+        # already have handled that 'ghost-rev-id' is a ghost, and commit
+        # should not try to fill it in at this point.
+        stacked_only_repo = self.get_only_repo(stacked_tree)
+        all_keys = [self.r1_key, self.r2_key, r3_key, to_merge_key, merged_key,
+                    ghost_key]
+        self.assertPresent([to_merge_key, merged_key],
+                           stacked_only_repo.revisions, all_keys)
+        self.assertPresent([self.r2_key, r3_key, to_merge_key, merged_key],
+                           stacked_only_repo.inventories, all_keys)
+
+
+class TestCommitStackedFailsAppropriately(TestCaseWithStackedTarget):
+
+    def test_stacked_commit_fails_on_old_formats(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        format = stacked_tree.branch.repository._format
+        if format.supports_chks:
+            stacked_tree.commit('should succeed')
+        else:
+            self.assertRaises(errors.BzrError,
+                stacked_tree.commit, 'unsupported format')

=== modified file 'bzrlib/tests/per_repository_reference/test_fetch.py'
--- bzrlib/tests/per_repository_reference/test_fetch.py	2009-06-01 18:13:46 +0000
+++ bzrlib/tests/per_repository_reference/test_fetch.py	2010-11-18 01:25:20 +0000
@@ -18,6 +18,7 @@
 from bzrlib import (
     branch,
     errors,
+    graph,
     )
 from bzrlib.smart import (
     server,
@@ -25,7 +26,7 @@
 from bzrlib.tests.per_repository import TestCaseWithRepository
 
 
-class TestFetch(TestCaseWithRepository):
+class TestFetchBase(TestCaseWithRepository):
 
     def make_source_branch(self):
         # It would be nice if there was a way to force this to be memory-only
@@ -51,6 +52,9 @@
         self.addCleanup(source_b.unlock)
         return content, source_b
 
+
+class TestFetch(TestFetchBase):
+
     def test_sprout_from_stacked_with_short_history(self):
         content, source_b = self.make_source_branch()
         # Split the generated content into a base branch, and a stacked branch
@@ -149,3 +153,38 @@
         source_b.lock_read()
         self.addCleanup(source_b.unlock)
         stacked.pull(source_b, stop_revision='B-id')
+
+
+class TestFetchFromRepoWithUnconfiguredFallbacks(TestFetchBase):
+
+    def make_stacked_source_repo(self):
+        _, source_b = self.make_source_branch()
+        # Use 'make_branch' which gives us a bzr:// branch when appropriate,
+        # rather than creating a branch-on-disk
+        stack_b = self.make_branch('stack-on')
+        stack_b.pull(source_b, stop_revision='B-id')
+        stacked_b = self.make_branch('stacked')
+        stacked_b.set_stacked_on_url('../stack-on')
+        stacked_b.pull(source_b, stop_revision='C-id')
+        return stacked_b.repository
+
+    def test_fetch_everything_includes_parent_invs(self):
+        stacked = self.make_stacked_source_repo()
+        repo_missing_fallbacks = stacked.bzrdir.open_repository()
+        self.addCleanup(repo_missing_fallbacks.lock_read().unlock)
+        target = self.make_repository('target')
+        self.addCleanup(target.lock_write().unlock)
+        target.fetch(
+            repo_missing_fallbacks,
+            fetch_spec=graph.EverythingResult(repo_missing_fallbacks))
+        self.assertEqual(repo_missing_fallbacks.revisions.keys(),
+            target.revisions.keys())
+        self.assertEqual(repo_missing_fallbacks.inventories.keys(),
+            target.inventories.keys())
+        self.assertEqual(['C-id'],
+            sorted(k[-1] for k in target.revisions.keys()))
+        self.assertEqual(['B-id', 'C-id'],
+            sorted(k[-1] for k in target.inventories.keys()))
+
+
+

=== modified file 'bzrlib/tests/per_transport.py'
--- bzrlib/tests/per_transport.py	2010-09-21 03:20:09 +0000
+++ bzrlib/tests/per_transport.py	2011-01-12 21:50:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -98,11 +98,11 @@
 
     def setUp(self):
         super(TransportTests, self).setUp()
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
 
     def check_transport_contents(self, content, transport, relpath):
-        """Check that transport.get(relpath).read() == content."""
-        self.assertEqualDiff(content, transport.get(relpath).read())
+        """Check that transport.get_bytes(relpath) == content."""
+        self.assertEqualDiff(content, transport.get_bytes(relpath))
 
     def test_ensure_base_missing(self):
         """.ensure_base() should create the directory if it doesn't exist"""
@@ -256,7 +256,7 @@
         handle = t.open_write_stream('foo')
         try:
             handle.write('b')
-            self.assertEqual('b', t.get('foo').read())
+            self.assertEqual('b', t.get_bytes('foo'))
         finally:
             handle.close()
 
@@ -268,7 +268,11 @@
         try:
             handle.write('b')
             self.assertEqual('b', t.get_bytes('foo'))
-            self.assertEqual('b', t.get('foo').read())
+            f = t.get('foo')
+            try:
+                self.assertEqual('b', f.read())
+            finally:
+                f.close()
         finally:
             handle.close()
 
@@ -640,7 +644,7 @@
             self.build_tree(files, transport=transport_from)
             self.assertEqual(4, transport_from.copy_to(files, transport_to))
             for f in files:
-                self.check_transport_contents(transport_to.get(f).read(),
+                self.check_transport_contents(transport_to.get_bytes(f),
                                               transport_from, f)
 
         t = self.get_transport()
@@ -669,7 +673,7 @@
         files = ['a', 'b', 'c', 'd']
         t.copy_to(iter(files), temp_transport)
         for f in files:
-            self.check_transport_contents(temp_transport.get(f).read(),
+            self.check_transport_contents(temp_transport.get_bytes(f),
                                           t, f)
         del temp_transport
 

=== modified file 'bzrlib/tests/per_uifactory/__init__.py'
--- bzrlib/tests/per_uifactory/__init__.py	2010-09-09 07:49:00 +0000
+++ bzrlib/tests/per_uifactory/__init__.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -190,7 +190,7 @@
 
         # Remove 'TERM' == 'dumb' which causes us to *not* treat output as a
         # real terminal, even though isatty returns True
-        self._captureVar('TERM', None)
+        self.overrideEnv('TERM', None)
         self.stderr = TTYStringIO()
         self.stdout = TTYStringIO()
         self.factory = ui.text.TextUIFactory(self.stdin, self.stdout,

=== modified file 'bzrlib/tests/per_versionedfile.py'
--- bzrlib/tests/per_versionedfile.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/per_versionedfile.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # Authors:
 #   Johan Rydberg <jrydberg@gnu.org>
@@ -21,6 +21,7 @@
 # TODO: might be nice to create a versionedfile with some type of corruption
 # considered typical and check that it can be detected/corrected.
 
+from gzip import GzipFile
 from itertools import chain, izip
 from StringIO import StringIO
 
@@ -37,14 +38,11 @@
 from bzrlib.errors import (
                            RevisionNotPresent,
                            RevisionAlreadyPresent,
-                           WeaveParentMismatch
                            )
 from bzrlib.knit import (
     cleanup_pack_knit,
     make_file_factory,
     make_pack_factory,
-    KnitAnnotateFactory,
-    KnitPlainFactory,
     )
 from bzrlib.tests import (
     TestCase,
@@ -53,10 +51,7 @@
     TestSkipped,
     )
 from bzrlib.tests.http_utils import TestCaseWithWebserver
-from bzrlib.trace import mutter
 from bzrlib.transport.memory import MemoryTransport
-from bzrlib.tsort import topo_sort
-from bzrlib.tuned_gzip import GzipFile
 import bzrlib.versionedfile as versionedfile
 from bzrlib.versionedfile import (
     ConstantMapper,
@@ -66,7 +61,7 @@
     make_versioned_files_factory,
     )
 from bzrlib.weave import WeaveFile
-from bzrlib.weavefile import read_weave, write_weave
+from bzrlib.weavefile import write_weave
 from bzrlib.tests.scenarios import load_tests_apply_scenarios
 
 

=== modified file 'bzrlib/tests/per_workingtree/test_add_reference.py'
--- bzrlib/tests/per_workingtree/test_add_reference.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_add_reference.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,13 +16,9 @@
 
 import os
 
-from bzrlib import errors, tests, workingtree, workingtree_4
+from bzrlib import errors, tests, workingtree
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
-TREES_NOT_SUPPORTING_REFERENCES = (workingtree.WorkingTree2,
-                                   workingtree.WorkingTree3,
-                                   workingtree_4.WorkingTree4)
-
 
 class TestBasisInventory(TestCaseWithWorkingTree):
 
@@ -37,7 +33,7 @@
         return tree, sub_tree
 
     def _references_unsupported(self, tree):
-        if tree.__class__ in TREES_NOT_SUPPORTING_REFERENCES:
+        if not tree.supports_tree_reference():
             raise tests.TestSkipped('Tree format does not support references')
         else:
             self.fail('%r does not support references but should'

=== modified file 'bzrlib/tests/per_workingtree/test_workingtree.py'
--- bzrlib/tests/per_workingtree/test_workingtree.py	2010-11-26 17:59:08 +0000
+++ bzrlib/tests/per_workingtree/test_workingtree.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 # Authors:  Robert Collins <robert.collins@canonical.com>
 #           and others
 #
@@ -19,7 +19,6 @@
 from cStringIO import StringIO
 import errno
 import os
-import sys
 
 from bzrlib import (
     branch,
@@ -29,17 +28,22 @@
     osutils,
     tests,
     urlutils,
-    workingtree,
-    )
-from bzrlib.errors import (NotBranchError, NotVersionedError,
-                           UnsupportedOperation, PathsNotVersionedError)
+    )
+from bzrlib.errors import (
+    UnsupportedOperation,
+    PathsNotVersionedError,
+    )
 from bzrlib.inventory import Inventory
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
 from bzrlib.tests import TestSkipped, TestNotApplicable
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
-from bzrlib.trace import mutter
-from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
-                                WorkingTree, WorkingTree2)
+from bzrlib.workingtree import (
+    TreeDirectory,
+    TreeFile,
+    TreeLink,
+    WorkingTree,
+    WorkingTree2,
+    )
 from bzrlib.conflicts import ConflictList, TextConflict, ContentsConflict
 
 

=== modified file 'bzrlib/tests/script.py'
--- bzrlib/tests/script.py	2010-11-08 09:58:04 +0000
+++ bzrlib/tests/script.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,10 +25,8 @@
 import os
 import shlex
 import textwrap
-from cStringIO import StringIO
 
 from bzrlib import (
-    errors,
     osutils,
     tests,
     )

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2010-09-02 19:19:54 +0000
+++ bzrlib/tests/stub_sftp.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008, 2009, 2010 Robey Pointer <robey@lag.net>, Canonical Ltd
+# Copyright (C) 2005, 2006, 2008-2011 Robey Pointer <robey@lag.net>, Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,11 +21,9 @@
 
 import os
 import paramiko
-import select
 import socket
 import SocketServer
 import sys
-import threading
 import time
 
 from bzrlib import (

=== modified file 'bzrlib/tests/test__annotator.py'
--- bzrlib/tests/test__annotator.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__annotator.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 from bzrlib import (
     annotate,
-    _annotator_py,
     errors,
     knit,
     revision,

=== modified file 'bzrlib/tests/test__chk_map.py'
--- bzrlib/tests/test__chk_map.py	2010-05-11 10:45:26 +0000
+++ bzrlib/tests/test__chk_map.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 from bzrlib import (
     chk_map,
-    inventory,
     tests,
     )
 from bzrlib.static_tuple import StaticTuple

=== modified file 'bzrlib/tests/test__dirstate_helpers.py'
--- bzrlib/tests/test__dirstate_helpers.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test__dirstate_helpers.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test__groupcompress.py'
--- bzrlib/tests/test__groupcompress.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test__groupcompress.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,13 +17,11 @@
 """Tests for the python and pyrex extensions of groupcompress"""
 
 from bzrlib import (
-    groupcompress,
     _groupcompress_py,
     tests,
     )
 from bzrlib.tests.scenarios import (
     load_tests_apply_scenarios,
-    multiply_scenarios,
     )
 
 

=== modified file 'bzrlib/tests/test__known_graph.py'
--- bzrlib/tests/test__known_graph.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test__known_graph.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
 from bzrlib import (
     errors,
-    graph as _mod_graph,
     _known_graph_py,
     tests,
     )

=== modified file 'bzrlib/tests/test__simple_set.py'
--- bzrlib/tests/test__simple_set.py	2010-08-02 19:20:30 +0000
+++ bzrlib/tests/test__simple_set.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
 import sys
 
 from bzrlib import (
-    errors,
-    osutils,
     tests,
     )
 

=== modified file 'bzrlib/tests/test__static_tuple.py'
--- bzrlib/tests/test__static_tuple.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test__static_tuple.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,13 +17,11 @@
 """Tests for the StaticTuple type."""
 
 import cPickle
-import gc
 import sys
 
 from bzrlib import (
     _static_tuple_py,
     debug,
-    errors,
     osutils,
     static_tuple,
     tests,

=== modified file 'bzrlib/tests/test_ancestry.py'
--- bzrlib/tests/test_ancestry.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_ancestry.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007 Canonical Ltd
+# Copyright (C) 2005-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,12 +15,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-from cStringIO import StringIO
-import os
-import sys
-
 from bzrlib.tests import TestCaseWithMemoryTransport
-from bzrlib.branch import Branch
 from bzrlib.branchbuilder import BranchBuilder
 
 

=== modified file 'bzrlib/tests/test_annotate.py'
--- bzrlib/tests/test_annotate.py	2009-07-10 07:43:15 +0000
+++ bzrlib/tests/test_annotate.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,10 +21,7 @@
 
 from bzrlib import (
     annotate,
-    conflicts,
-    errors,
     tests,
-    trace,
     )
 
 

=== modified file 'bzrlib/tests/test_bad_files.py'
--- bzrlib/tests/test_bad_files.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_bad_files.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 import os
 
 from bzrlib import (
-    add,
     errors,
     )
 from bzrlib.status import show_tree_status

=== modified file 'bzrlib/tests/test_bisect_multi.py'
--- bzrlib/tests/test_bisect_multi.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_bisect_multi.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 """Tests for bisect_multi."""
 
-from bzrlib import errors
 from bzrlib.bisect_multi import bisect_multi_bytes
 from bzrlib.tests import TestCase
 

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2010-12-14 23:14:44 +0000
+++ bzrlib/tests/test_branch.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_branchbuilder.py'
--- bzrlib/tests/test_branchbuilder.py	2010-11-18 08:13:01 +0000
+++ bzrlib/tests/test_branchbuilder.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2009 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 from bzrlib import (
     branch as _mod_branch,
-    errors,
     revision as _mod_revision,
     tests,
     )

=== modified file 'bzrlib/tests/test_btree_index.py'
--- bzrlib/tests/test_btree_index.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test_btree_index.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2010-08-05 04:21:03 +0000
+++ bzrlib/tests/test_bundle.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
     inventory,
     merge,
     osutils,
-    repository,
     revision as _mod_revision,
     tests,
     treebuilder,
@@ -35,13 +34,11 @@
 from bzrlib.bundle import read_mergeable_from_url
 from bzrlib.bundle.apply_bundle import install_bundle, merge_bundle
 from bzrlib.bundle.bundle_data import BundleTree
-from bzrlib.bzrdir import BzrDir
 from bzrlib.directory_service import directories
 from bzrlib.bundle.serializer import write_bundle, read_bundle, v09, v4
 from bzrlib.bundle.serializer.v08 import BundleSerializerV08
 from bzrlib.bundle.serializer.v09 import BundleSerializerV09
 from bzrlib.bundle.serializer.v4 import BundleSerializerV4
-from bzrlib.branch import Branch
 from bzrlib.repofmt import knitrepo
 from bzrlib.tests import (
     test_read_bundle,

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2010-12-15 00:38:15 +0000
+++ bzrlib/tests/test_bzrdir.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_chk_serializer.py'
--- bzrlib/tests/test_chk_serializer.py	2009-06-04 21:50:04 +0000
+++ bzrlib/tests/test_chk_serializer.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,13 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
-
-from bzrlib import (
-    errors,
-    revision,
-    serializer,
-    )
 from bzrlib.chk_serializer import (
     chk_bencode_serializer,
     )

=== modified file 'bzrlib/tests/test_clean_tree.py'
--- bzrlib/tests/test_clean_tree.py	2010-08-23 17:54:07 +0000
+++ bzrlib/tests/test_clean_tree.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 import errno
 import os
 import shutil
-from StringIO import StringIO
 import sys
 import types
 

=== modified file 'bzrlib/tests/test_commands.py'
--- bzrlib/tests/test_commands.py	2010-08-28 16:56:36 +0000
+++ bzrlib/tests/test_commands.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 import errno
 import inspect
 import sys

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2010-10-15 16:43:03 +0000
+++ bzrlib/tests/test_commit.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,18 +21,14 @@
 from bzrlib import (
     bzrdir,
     errors,
-    lockdir,
-    osutils,
-    tests,
     )
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
+from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.commit import Commit, NullCommitReporter
 from bzrlib.config import BranchConfig
 from bzrlib.errors import (PointlessCommit, BzrError, SigningFailed,
                            LockContention)
 from bzrlib.tests import SymlinkFeature, TestCaseWithTransport
-from bzrlib.workingtree import WorkingTree
 
 
 # TODO: Test commit with some added, and added-but-missing files

=== modified file 'bzrlib/tests/test_commit_merge.py'
--- bzrlib/tests/test_commit_merge.py	2009-08-05 02:30:59 +0000
+++ bzrlib/tests/test_commit_merge.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,16 +16,13 @@
 
 
 import os
-import shutil
 
 from bzrlib import check, osutils
-from bzrlib.branch import Branch
-from bzrlib.errors import PointlessCommit, BzrError
+from bzrlib.errors import PointlessCommit
 from bzrlib.tests import (
     SymlinkFeature,
     TestCaseWithTransport,
     )
-from bzrlib.tests.test_revision import make_branches
 
 
 class TestCommitMerge(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2010-11-29 01:23:53 +0000
+++ bzrlib/tests/test_config.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -402,11 +402,11 @@
 
     def setUp(self):
         super(TestConfigPath, self).setUp()
-        os.environ['HOME'] = '/home/bogus'
-        os.environ['XDG_CACHE_DIR'] = ''
+        self.overrideEnv('HOME', '/home/bogus')
+        self.overrideEnv('XDG_CACHE_DIR', '')
         if sys.platform == 'win32':
-            os.environ['BZR_HOME'] = \
-                r'C:\Documents and Settings\bogus\Application Data'
+            self.overrideEnv(
+                'BZR_HOME', r'C:\Documents and Settings\bogus\Application Data')
             self.bzr_home = \
                 'C:/Documents and Settings/bogus/Application Data/bazaar/2.0'
         else:
@@ -441,9 +441,9 @@
             raise tests.TestNotApplicable(
                 'XDG config dir not used on this platform')
         super(TestXDGConfigDir, self).setUp()
-        os.environ['HOME'] = self.test_home_dir
+        self.overrideEnv('HOME', self.test_home_dir)
         # BZR_HOME overrides everything we want to test so unset it.
-        del os.environ['BZR_HOME']
+        self.overrideEnv('BZR_HOME', None)
 
     def test_xdg_config_dir_exists(self):
         """When ~/.config/bazaar exists, use it as the config dir."""
@@ -454,7 +454,7 @@
     def test_xdg_config_home(self):
         """When XDG_CONFIG_HOME is set, use it."""
         xdgconfigdir = osutils.pathjoin(self.test_home_dir, 'xdgconfig')
-        os.environ['XDG_CONFIG_HOME'] = xdgconfigdir
+        self.overrideEnv('XDG_CONFIG_HOME', xdgconfigdir)
         newdir = osutils.pathjoin(xdgconfigdir, 'bazaar')
         os.makedirs(newdir)
         self.assertEqual(config.config_dir(), newdir)
@@ -1308,7 +1308,7 @@
         self.assertEqual('bzr', my_config.get_bzr_remote_path())
         my_config.set_user_option('bzr_remote_path', '/path-bzr')
         self.assertEqual('/path-bzr', my_config.get_bzr_remote_path())
-        os.environ['BZR_REMOTE_PATH'] = '/environ-bzr'
+        self.overrideEnv('BZR_REMOTE_PATH', '/environ-bzr')
         self.assertEqual('/environ-bzr', my_config.get_bzr_remote_path())
 
 
@@ -1360,7 +1360,7 @@
         self.assertEqual("John", my_config._get_user_id())
 
     def test_BZR_EMAIL_OVERRIDES(self):
-        os.environ['BZR_EMAIL'] = "Robert Collins <robertc@example.org>"
+        self.overrideEnv('BZR_EMAIL', "Robert Collins <robertc@example.org>")
         branch = FakeBranch()
         my_config = config.BranchConfig(branch)
         self.assertEqual("Robert Collins <robertc@example.org>",

=== modified file 'bzrlib/tests/test_conflicts.py'
--- bzrlib/tests/test_conflicts.py	2010-11-10 16:24:31 +0000
+++ bzrlib/tests/test_conflicts.py	2011-01-15 17:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,13 +18,11 @@
 import os
 
 from bzrlib import (
-    branchbuilder,
     bzrdir,
     conflicts,
     errors,
     option,
     tests,
-    workingtree,
     )
 from bzrlib.tests import (
     script,
@@ -429,6 +427,15 @@
               dict(actions='modify_file', check='file_has_more_content')),
              ('file_deleted',
               dict(actions='delete_file', check='file_doesnt_exist')),),
+            # File modified/deleted in dir
+            (dict(_base_actions='create_file_in_dir',
+                  _path='dir/file', _file_id='file-id'),
+             ('file_modified_in_dir',
+              dict(actions='modify_file_in_dir',
+                   check='file_in_dir_has_more_content')),
+             ('file_deleted_in_dir',
+              dict(actions='delete_file',
+                   check='file_in_dir_doesnt_exist')),),
             ])
 
     def do_create_file(self):
@@ -446,6 +453,19 @@
     def check_file_doesnt_exist(self):
         self.failIfExists('branch/file')
 
+    def do_create_file_in_dir(self):
+        return [('add', ('dir', 'dir-id', 'directory', '')),
+                ('add', ('dir/file', 'file-id', 'file', 'trunk content\n'))]
+
+    def do_modify_file_in_dir(self):
+        return [('modify', ('file-id', 'trunk content\nmore content\n'))]
+
+    def check_file_in_dir_has_more_content(self):
+        self.assertFileEqual('trunk content\nmore content\n', 'branch/dir/file')
+
+    def check_file_in_dir_doesnt_exist(self):
+        self.failIfExists('branch/dir/file')
+
     def _get_resolve_path_arg(self, wt, action):
         return self._path
 
@@ -477,6 +497,16 @@
                    # PathConflicts deletion handling requires a special
                    # hard-coded value
                    path='<deleted>', file_id='file-id')),),
+            # File renamed/deleted in dir
+            (dict(_base_actions='create_file_in_dir'),
+             ('file_renamed_in_dir',
+              dict(actions='rename_file_in_dir', check='file_in_dir_renamed',
+                   path='dir/new-file', file_id='file-id')),
+             ('file_deleted',
+              dict(actions='delete_file', check='file_in_dir_doesnt_exist',
+                   # PathConflicts deletion handling requires a special
+                   # hard-coded value
+                   path='<deleted>', file_id='file-id')),),
             # File renamed/renamed differently
             (dict(_base_actions='create_file'),
              ('file_renamed',
@@ -551,6 +581,20 @@
     def check_dir_doesnt_exist(self):
         self.failIfExists('branch/dir')
 
+    def do_create_file_in_dir(self):
+        return [('add', ('dir', 'dir-id', 'directory', '')),
+                ('add', ('dir/file', 'file-id', 'file', 'trunk content\n'))]
+
+    def do_rename_file_in_dir(self):
+        return [('rename', ('dir/file', 'dir/new-file'))]
+
+    def check_file_in_dir_renamed(self):
+        self.failIfExists('branch/dir/file')
+        self.failUnlessExists('branch/dir/new-file')
+
+    def check_file_in_dir_doesnt_exist(self):
+        self.failIfExists('branch/dir/file')
+
     def _get_resolve_path_arg(self, wt, action):
         tpath = self._this['path']
         opath = self._other['path']

=== modified file 'bzrlib/tests/test_crash.py'
--- bzrlib/tests/test_crash.py	2010-04-08 04:34:03 +0000
+++ bzrlib/tests/test_crash.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
     config,
     crash,
     osutils,
-    symbol_versioning,
     tests,
     )
 
@@ -40,7 +39,7 @@
     def test_apport_report(self):
         crash_dir = osutils.joinpath((self.test_base_dir, 'crash'))
         os.mkdir(crash_dir)
-        os.environ['APPORT_CRASH_DIR'] = crash_dir
+        self.overrideEnv('APPORT_CRASH_DIR', crash_dir)
         self.assertEquals(crash_dir, config.crash_dir())
 
         stderr = StringIO()

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2010-05-26 15:58:08 +0000
+++ bzrlib/tests/test_diff.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -144,31 +144,22 @@
         self.check_patch(lines)
 
     def test_external_diff_binary_lang_c(self):
-        old_env = {}
         for lang in ('LANG', 'LC_ALL', 'LANGUAGE'):
-            old_env[lang] = osutils.set_or_unset_env(lang, 'C')
-        try:
-            lines = external_udiff_lines(['\x00foobar\n'], ['foo\x00bar\n'])
-            # Older versions of diffutils say "Binary files", newer
-            # versions just say "Files".
-            self.assertContainsRe(lines[0],
-                                  '(Binary f|F)iles old and new differ\n')
-            self.assertEquals(lines[1:], ['\n'])
-        finally:
-            for lang, old_val in old_env.iteritems():
-                osutils.set_or_unset_env(lang, old_val)
+            self.overrideEnv(lang, 'C')
+        lines = external_udiff_lines(['\x00foobar\n'], ['foo\x00bar\n'])
+        # Older versions of diffutils say "Binary files", newer
+        # versions just say "Files".
+        self.assertContainsRe(lines[0], '(Binary f|F)iles old and new differ\n')
+        self.assertEquals(lines[1:], ['\n'])
 
     def test_no_external_diff(self):
         """Check that NoDiff is raised when diff is not available"""
-        # Use os.environ['PATH'] to make sure no 'diff' command is available
-        orig_path = os.environ['PATH']
-        try:
-            os.environ['PATH'] = ''
-            self.assertRaises(errors.NoDiff, diff.external_diff,
-                              'old', ['boo\n'], 'new', ['goo\n'],
-                              StringIO(), diff_opts=['-u'])
-        finally:
-            os.environ['PATH'] = orig_path
+        # Make sure no 'diff' command is available
+        # XXX: Weird, using None instead of '' breaks the test -- vila 20101216
+        self.overrideEnv('PATH', '')
+        self.assertRaises(errors.NoDiff, diff.external_diff,
+                          'old', ['boo\n'], 'new', ['goo\n'],
+                          StringIO(), diff_opts=['-u'])
 
     def test_internal_diff_default(self):
         # Default internal diff encoding is utf8

=== modified file 'bzrlib/tests/test_dirstate.py'
--- bzrlib/tests/test_dirstate.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test_dirstate.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 """Tests of the dirstate functionality being built for WorkingTreeFormat4."""
 
-import bisect
 import os
 
 from bzrlib import (

=== modified file 'bzrlib/tests/test_eol_filters.py'
--- bzrlib/tests/test_eol_filters.py	2009-06-10 23:29:48 +0000
+++ bzrlib/tests/test_eol_filters.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 
 from bzrlib import (
     errors,
-    rules,
     )
 from bzrlib.filters import _get_filter_stack_for
 from bzrlib.filters.eol import (

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2010-12-02 09:23:10 +0000
+++ bzrlib/tests/test_errors.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,10 +25,13 @@
     bzrdir,
     errors,
     osutils,
-    symbol_versioning,
     urlutils,
     )
-from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
+from bzrlib.tests import (
+    TestCase,
+    TestCaseWithTransport,
+    TestSkipped,
+    )
 
 
 class TestErrors(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_extract.py'
--- bzrlib/tests/test_extract.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_extract.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 from bzrlib import (
     branch,
-    bzrdir,
     errors,
     )
 from bzrlib.tests import TestCaseWithTransport

=== modified file 'bzrlib/tests/test_fetch.py'
--- bzrlib/tests/test_fetch.py	2010-02-09 20:18:32 +0000
+++ bzrlib/tests/test_fetch.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007, 2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,25 +14,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import os
-import re
-import sys
-
-import bzrlib
 from bzrlib import (
     bzrdir,
     errors,
     osutils,
-    merge,
-    repository,
     versionedfile,
     )
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
 from bzrlib.repofmt import knitrepo
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests.test_revision import make_branches
-from bzrlib.trace import mutter
 from bzrlib.upgrade import Convert
 from bzrlib.workingtree import WorkingTree
 

=== modified file 'bzrlib/tests/test_foreign.py'
--- bzrlib/tests/test_foreign.py	2010-11-30 02:32:37 +0000
+++ bzrlib/tests/test_foreign.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_ftp_transport.py'
--- bzrlib/tests/test_ftp_transport.py	2010-02-27 01:34:49 +0000
+++ bzrlib/tests/test_ftp_transport.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,10 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 import ftplib
 import getpass
-import sys
 
 from bzrlib import (
     config,

=== modified file 'bzrlib/tests/test_generate_docs.py'
--- bzrlib/tests/test_generate_docs.py	2009-06-19 09:06:56 +0000
+++ bzrlib/tests/test_generate_docs.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 from cStringIO import StringIO
 
 import bzrlib.commands
-from bzrlib.tests import TestCase, TestSkipped
+from bzrlib.tests import TestCase
 
 
 class Options:

=== modified file 'bzrlib/tests/test_generate_ids.py'
--- bzrlib/tests/test_generate_ids.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_generate_ids.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
 
 """Tests for bzrlib/generate_ids.py"""
 
-import re
-
 from bzrlib import (
     generate_ids,
     tests,

=== modified file 'bzrlib/tests/test_globbing.py'
--- bzrlib/tests/test_globbing.py	2010-08-07 16:42:09 +0000
+++ bzrlib/tests/test_globbing.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,6 @@
     )
 from bzrlib.tests import (
     TestCase,
-    TestCaseInTempDir,
     )
 
 

=== modified file 'bzrlib/tests/test_gpg.py'
--- bzrlib/tests/test_gpg.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_gpg.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2011 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -18,12 +18,11 @@
 """Tests for signing and verifying blobs of data via gpg."""
 
 # import system imports here
-import os
 import sys
 
 from bzrlib import errors, ui
 import bzrlib.gpg as gpg
-from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib.tests import TestCase
 
 class FakeConfig(object):
 

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2010-12-11 22:06:52 +0000
+++ bzrlib/tests/test_graph.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 from bzrlib import (
     errors,
     graph as _mod_graph,
-    symbol_versioning,
     tests,
     )
 from bzrlib.revision import NULL_REVISION

=== modified file 'bzrlib/tests/test_groupcompress.py'
--- bzrlib/tests/test_groupcompress.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test_groupcompress.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_hashcache.py'
--- bzrlib/tests/test_hashcache.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_hashcache.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,13 +16,15 @@
 
 import os
 import stat
-import sys
 import time
 
 from bzrlib import osutils
 from bzrlib.errors import BzrError
 from bzrlib.hashcache import HashCache
-from bzrlib.tests import OsFifoFeature, TestCaseInTempDir, TestCase
+from bzrlib.tests import (
+    OsFifoFeature,
+    TestCaseInTempDir,
+    )
 
 
 def sha1(t):

=== modified file 'bzrlib/tests/test_help.py'
--- bzrlib/tests/test_help.py	2010-05-24 05:50:53 +0000
+++ bzrlib/tests/test_help.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
 
 """Unit tests for the bzrlib.help module."""
 
-from cStringIO import StringIO
-
 from bzrlib import (
     builtins,
     commands,

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2010-12-15 16:39:25 +0000
+++ bzrlib/tests/test_http.py	2011-01-12 22:21:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1073,19 +1073,6 @@
     Only the urllib implementation is tested here.
     """
 
-    def setUp(self):
-        tests.TestCase.setUp(self)
-        self._http_saved_env = {}
-        self.addCleanup(self._restore_env)
-
-    def _install_env(self, env):
-        for name, value in env.iteritems():
-            self._http_saved_env[name] = osutils.set_or_unset_env(name, value)
-
-    def _restore_env(self):
-        for name, value in self._http_saved_env.iteritems():
-            osutils.set_or_unset_env(name, value)
-
     def _proxied_request(self):
         handler = _urllib2_wrappers.ProxyHandler()
         request = _urllib2_wrappers.Request('GET','http://baz/buzzle')
@@ -1093,13 +1080,13 @@
         return request
 
     def test_empty_user(self):
-        self._install_env({'http_proxy': 'http://bar.com'})
+        self.overrideEnv('http_proxy', 'http://bar.com')
         request = self._proxied_request()
         self.assertFalse(request.headers.has_key('Proxy-authorization'))
 
     def test_invalid_proxy(self):
         """A proxy env variable without scheme"""
-        self._install_env({'http_proxy': 'host:1234'})
+        self.overrideEnv('http_proxy', 'host:1234')
         self.assertRaises(errors.InvalidURL, self._proxied_request)
 
 
@@ -1136,39 +1123,23 @@
             self.no_proxy_host = self.server_host_port
         # The secondary server is the proxy
         self.proxy_url = self.get_secondary_url()
-        self._http_saved_env = {}
 
     def _testing_pycurl(self):
         # TODO: This is duplicated for lots of the classes in this file
         return (features.pycurl.available()
                 and self._transport == PyCurlTransport)
 
-    def _install_env(self, env):
-        for name, value in env.iteritems():
-            self._http_saved_env[name] = osutils.set_or_unset_env(name, value)
-
-    def _restore_env(self):
-        for name, value in self._http_saved_env.iteritems():
-            osutils.set_or_unset_env(name, value)
-
-    def proxied_in_env(self, env):
-        self._install_env(env)
-        t = self.get_readonly_transport()
-        try:
-            self.assertEqual('proxied contents of foo\n', t.get('foo').read())
-        finally:
-            self._restore_env()
-
-    def not_proxied_in_env(self, env):
-        self._install_env(env)
-        t = self.get_readonly_transport()
-        try:
-            self.assertEqual('contents of foo\n', t.get('foo').read())
-        finally:
-            self._restore_env()
+    def assertProxied(self):
+        t = self.get_readonly_transport()
+        self.assertEqual('proxied contents of foo\n', t.get('foo').read())
+
+    def assertNotProxied(self):
+        t = self.get_readonly_transport()
+        self.assertEqual('contents of foo\n', t.get('foo').read())
 
     def test_http_proxy(self):
-        self.proxied_in_env({'http_proxy': self.proxy_url})
+        self.overrideEnv('http_proxy', self.proxy_url)
+        self.assertProxied()
 
     def test_HTTP_PROXY(self):
         if self._testing_pycurl():
@@ -1177,43 +1148,49 @@
             # about. Should we ?)
             raise tests.TestNotApplicable(
                 'pycurl does not check HTTP_PROXY for security reasons')
-        self.proxied_in_env({'HTTP_PROXY': self.proxy_url})
+        self.overrideEnv('HTTP_PROXY', self.proxy_url)
+        self.assertProxied()
 
     def test_all_proxy(self):
-        self.proxied_in_env({'all_proxy': self.proxy_url})
+        self.overrideEnv('all_proxy', self.proxy_url)
+        self.assertProxied()
 
     def test_ALL_PROXY(self):
-        self.proxied_in_env({'ALL_PROXY': self.proxy_url})
+        self.overrideEnv('ALL_PROXY', self.proxy_url)
+        self.assertProxied()
 
     def test_http_proxy_with_no_proxy(self):
-        self.not_proxied_in_env({'http_proxy': self.proxy_url,
-                                 'no_proxy': self.no_proxy_host})
+        self.overrideEnv('no_proxy', self.no_proxy_host)
+        self.overrideEnv('http_proxy', self.proxy_url)
+        self.assertNotProxied()
 
     def test_HTTP_PROXY_with_NO_PROXY(self):
         if self._testing_pycurl():
             raise tests.TestNotApplicable(
                 'pycurl does not check HTTP_PROXY for security reasons')
-        self.not_proxied_in_env({'HTTP_PROXY': self.proxy_url,
-                                 'NO_PROXY': self.no_proxy_host})
+        self.overrideEnv('NO_PROXY', self.no_proxy_host)
+        self.overrideEnv('HTTP_PROXY', self.proxy_url)
+        self.assertNotProxied()
 
     def test_all_proxy_with_no_proxy(self):
-        self.not_proxied_in_env({'all_proxy': self.proxy_url,
-                                 'no_proxy': self.no_proxy_host})
+        self.overrideEnv('no_proxy', self.no_proxy_host)
+        self.overrideEnv('all_proxy', self.proxy_url)
+        self.assertNotProxied()
 
     def test_ALL_PROXY_with_NO_PROXY(self):
-        self.not_proxied_in_env({'ALL_PROXY': self.proxy_url,
-                                 'NO_PROXY': self.no_proxy_host})
+        self.overrideEnv('NO_PROXY', self.no_proxy_host)
+        self.overrideEnv('ALL_PROXY', self.proxy_url)
+        self.assertNotProxied()
 
     def test_http_proxy_without_scheme(self):
+        self.overrideEnv('http_proxy', self.server_host_port)
         if self._testing_pycurl():
             # pycurl *ignores* invalid proxy env variables. If that ever change
             # in the future, this test will fail indicating that pycurl do not
             # ignore anymore such variables.
-            self.not_proxied_in_env({'http_proxy': self.server_host_port})
+            self.assertNotProxied()
         else:
-            self.assertRaises(errors.InvalidURL,
-                              self.proxied_in_env,
-                              {'http_proxy': self.server_host_port})
+            self.assertRaises(errors.InvalidURL, self.assertProxied)
 
 
 class TestRanges(http_utils.TestCaseWithWebserver):
@@ -1707,8 +1684,6 @@
 
     def setUp(self):
         super(TestProxyAuth, self).setUp()
-        self._http_saved_env = {}
-        self.addCleanup(self._restore_env)
         # Override the contents to avoid false positives
         self.build_tree_contents([('a', 'not proxied contents of a\n'),
                                   ('b', 'not proxied contents of b\n'),
@@ -1717,17 +1692,9 @@
                                   ])
 
     def get_user_transport(self, user, password):
-        self._install_env({'all_proxy': self.get_user_url(user, password)})
+        self.overrideEnv('all_proxy', self.get_user_url(user, password))
         return TestAuth.get_user_transport(self, user, password)
 
-    def _install_env(self, env):
-        for name, value in env.iteritems():
-            self._http_saved_env[name] = osutils.set_or_unset_env(name, value)
-
-    def _restore_env(self):
-        for name, value in self._http_saved_env.iteritems():
-            osutils.set_or_unset_env(name, value)
-
     def test_empty_pass(self):
         if self._testing_pycurl():
             import pycurl
@@ -1770,7 +1737,7 @@
     def setUp(self):
         super(SmartHTTPTunnellingTest, self).setUp()
         # We use the VFS layer as part of HTTP tunnelling tests.
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         self.transport_readonly_server = http_utils.HTTPServerWithSmarts
         self.http_server = self.get_readonly_server()
 

=== modified file 'bzrlib/tests/test_https_ca_bundle.py'
--- bzrlib/tests/test_https_ca_bundle.py	2010-12-15 16:39:25 +0000
+++ bzrlib/tests/test_https_ca_bundle.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
 import os
 import sys
 
-import bzrlib
-from bzrlib import osutils
 from bzrlib.tests import (
     TestCaseInTempDir,
     TestSkipped,
@@ -31,19 +29,9 @@
 class TestGetCAPath(TestCaseInTempDir):
 
     def setUp(self):
-        TestCaseInTempDir.setUp(self)
-        new_env = {
-            'CURL_CA_BUNDLE': None,
-            'PATH': None,
-            }
-        self._http_saved_env = {}
-        self.addCleanup(self._restore)
-        for name, value in new_env.iteritems():
-            self._http_saved_env[name] = osutils.set_or_unset_env(name, None)
-
-    def _restore(self):
-        for name, value in self._http_saved_env.iteritems():
-            osutils.set_or_unset_env(name, value)
+        super(TestGetCAPath, self).setUp()
+        self.overrideEnv('CURL_CA_BUNDLE', None)
+        self.overrideEnv('PATH', None)
 
     def _make_file(self, in_dir='.'):
         fname = os.path.join(in_dir, 'curl-ca-bundle.crt')
@@ -55,7 +43,7 @@
         self.assertEqual('', ca_bundle.get_ca_path(use_cache=False))
 
     def test_env_var(self):
-        osutils.set_or_unset_env('CURL_CA_BUNDLE', 'foo.bar')
+        self.overrideEnv('CURL_CA_BUNDLE', 'foo.bar')
         self._make_file()
         self.assertEqual('foo.bar', ca_bundle.get_ca_path(use_cache=False))
 
@@ -65,6 +53,6 @@
         os.mkdir('foo')
         in_dir = os.path.join(os.getcwd(), 'foo')
         self._make_file(in_dir=in_dir)
-        osutils.set_or_unset_env('PATH', in_dir)
+        self.overrideEnv('PATH', in_dir)
         shouldbe = os.path.join(in_dir, 'curl-ca-bundle.crt')
         self.assertEqual(shouldbe, ca_bundle.get_ca_path(use_cache=False))

=== modified file 'bzrlib/tests/test_ignores.py'
--- bzrlib/tests/test_ignores.py	2010-05-03 04:51:58 +0000
+++ bzrlib/tests/test_ignores.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,15 @@
 
 from cStringIO import StringIO
 
-from bzrlib import config, errors, ignores
-from bzrlib.tests import TestCase, TestCaseInTempDir, TestCaseWithTransport
+from bzrlib import (
+    config,
+    ignores,
+    )
+from bzrlib.tests import (
+    TestCase,
+    TestCaseInTempDir,
+    TestCaseWithTransport,
+    )
 
 
 class TestParseIgnoreFile(TestCase):

=== modified file 'bzrlib/tests/test_import_tariff.py'
--- bzrlib/tests/test_import_tariff.py	2010-10-20 11:16:16 +0000
+++ bzrlib/tests/test_import_tariff.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 """Tests for how many modules are loaded in executing various commands."""
 
+import os
 from testtools import content
 
 from bzrlib.plugin import (
@@ -29,12 +30,19 @@
 
 
 class TestImportTariffs(TestCaseWithTransport):
-
     """Check how many modules are loaded for some representative scenarios.
 
     See the Testing Guide in the developer documentation for more explanation.
     """
 
+    def setUp(self):
+        # Preserve some env vars as we want to escape the isolation for them
+        self.preserved_env_vars = {}
+        for name in ('BZR_HOME', 'BZR_PLUGIN_PATH', 'BZR_DISABLE_PLUGINS',
+                     'BZR_PLUGINS_AT', 'HOME'):
+            self.preserved_env_vars[name] = os.environ.get(name)
+        super(TestImportTariffs, self).setUp()
+
     def run_command_check_imports(self, args, forbidden_imports):
         """Run bzr ARGS in a subprocess and check its imports.
 
@@ -50,15 +58,11 @@
         # more likely to always show everything.  And we use the environment
         # variable rather than 'python -v' in the hope it will work even if
         # bzr is frozen and python is not explicitly specified. -- mbp 20100208
-        #
+
         # Normally we want test isolation from the real $HOME but here we
         # explicitly do want to test against things installed there, therefore
         # we pass it through.
-        env_changes = dict(PYTHONVERBOSE='1')
-        for name in ['BZR_HOME', 'BZR_PLUGIN_PATH',
-                     'BZR_DISABLE_PLUGINS', 'BZR_PLUGINS_AT',
-                     'HOME',]:
-            env_changes[name] = self._old_env.get(name)
+        env_changes = dict(PYTHONVERBOSE='1', **self.preserved_env_vars)
         out, err = self.run_bzr_subprocess(args,
             allow_plugins=(not are_plugins_disabled()),
             env_changes=env_changes)

=== modified file 'bzrlib/tests/test_info.py'
--- bzrlib/tests/test_info.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_info.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,7 +15,6 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import sys
-from urllib import quote
 
 from bzrlib import (
     branch as _mod_branch,

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test_inv.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,13 +25,18 @@
     revision,
     tests,
     )
-from bzrlib.inventory import (CHKInventory, Inventory, ROOT_ID, InventoryFile,
-    InventoryDirectory, InventoryEntry, TreeReference)
+from bzrlib.inventory import (
+    CHKInventory,
+    Inventory,
+    ROOT_ID,
+    InventoryFile,
+    InventoryDirectory,
+    InventoryEntry,
+    TreeReference,
+    )
 from bzrlib.tests import (
     TestCase,
     TestCaseWithTransport,
-    condition_isinstance,
-    multiply_tests,
     )
 from bzrlib.tests.per_workingtree import workingtree_formats
 from bzrlib.tests.scenarios import load_tests_apply_scenarios

=== modified file 'bzrlib/tests/test_inventory_delta.py'
--- bzrlib/tests/test_inventory_delta.py	2010-08-23 17:54:07 +0000
+++ bzrlib/tests/test_inventory_delta.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 from cStringIO import StringIO
 
 from bzrlib import (
-    errors,
     inventory,
     inventory_delta,
     )

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_knit.py	2011-01-11 17:54:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 """Tests for Knit data structure"""
 
 from cStringIO import StringIO
+import gzip
 import sys
 
 from bzrlib import (
@@ -27,7 +28,6 @@
     pack,
     tests,
     transport,
-    tuned_gzip,
     )
 from bzrlib.errors import (
     KnitHeaderError,
@@ -692,7 +692,7 @@
 
     def create_gz_content(self, text):
         sio = StringIO()
-        gz_file = tuned_gzip.GzipFile(mode='wb', fileobj=sio)
+        gz_file = gzip.GzipFile(mode='wb', fileobj=sio)
         gz_file.write(text)
         gz_file.close()
         return sio.getvalue()

=== modified file 'bzrlib/tests/test_lazy_import.py'
--- bzrlib/tests/test_lazy_import.py	2010-09-25 20:08:01 +0000
+++ bzrlib/tests/test_lazy_import.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,10 @@
     lazy_import,
     osutils,
     )
-from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib.tests import (
+    TestCase,
+    TestCaseInTempDir,
+    )
 
 
 class InstrumentedReplacer(lazy_import.ScopeReplacer):

=== modified file 'bzrlib/tests/test_library_state.py'
--- bzrlib/tests/test_library_state.py	2010-06-26 01:07:16 +0000
+++ bzrlib/tests/test_library_state.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 """Tests for BzrLibraryState."""
 
-import bzrlib
 from bzrlib import (
     library_state,
     tests,

=== modified file 'bzrlib/tests/test_lock.py'
--- bzrlib/tests/test_lock.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test_lock.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_lockable_files.py'
--- bzrlib/tests/test_lockable_files.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_lockable_files.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,8 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from StringIO import StringIO
-
 import bzrlib
 from bzrlib import (
     errors,
@@ -23,11 +21,7 @@
     osutils,
     transport,
     )
-from bzrlib.errors import BzrBadParameterNotString, NoSuchFile, ReadOnlyError
 from bzrlib.lockable_files import LockableFiles, TransportLock
-from bzrlib.symbol_versioning import (
-    deprecated_in,
-    )
 from bzrlib.tests import (
     TestCaseInTempDir,
     TestNotApplicable,

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2010-12-02 09:23:10 +0000
+++ bzrlib/tests/test_lockdir.py	2011-01-12 20:31:15 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 """Tests for LockDir"""
 
-from cStringIO import StringIO
 import os
 from threading import Thread, Lock
 import time
@@ -34,12 +33,14 @@
     LockBreakMismatch,
     LockBroken,
     LockContention,
-    LockError,
     LockFailed,
     LockNotHeld,
     )
 from bzrlib.lockdir import LockDir
-from bzrlib.tests import (features, TestCaseWithTransport)
+from bzrlib.tests import (
+    features,
+    TestCaseWithTransport,
+    )
 from bzrlib.trace import note
 
 # These tests sometimes use threads to test the behaviour of lock files with
@@ -580,16 +581,12 @@
                 self.prompts.append(('boolean', prompt))
                 return True
         ui = LoggingUIFactory()
-        orig_factory = bzrlib.ui.ui_factory
-        bzrlib.ui.ui_factory = ui
-        try:
-            ld2.break_lock()
-            self.assertLength(1, ui.prompts)
-            self.assertEqual('boolean', ui.prompts[0][0])
-            self.assertStartsWith(ui.prompts[0][1], 'Break (corrupt LockDir')
-            self.assertRaises(LockBroken, ld.unlock)
-        finally:
-            bzrlib.ui.ui_factory = orig_factory
+        self.overrideAttr(bzrlib.ui, 'ui_factory', ui)
+        ld2.break_lock()
+        self.assertLength(1, ui.prompts)
+        self.assertEqual('boolean', ui.prompts[0][0])
+        self.assertStartsWith(ui.prompts[0][1], 'Break (corrupt LockDir')
+        self.assertRaises(LockBroken, ld.unlock)
 
     def test_break_lock_missing_info(self):
         """break_lock works even if the info file is missing (and tells the UI

=== modified file 'bzrlib/tests/test_lsprof.py'
--- bzrlib/tests/test_lsprof.py	2010-07-05 08:10:03 +0000
+++ bzrlib/tests/test_lsprof.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 
 import cPickle
-import os
 import threading
 
 import bzrlib

=== modified file 'bzrlib/tests/test_merge3.py'
--- bzrlib/tests/test_merge3.py	2010-04-26 16:59:56 +0000
+++ bzrlib/tests/test_merge3.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,11 +16,9 @@
 
 
 from bzrlib import (
-    errors,
     merge3,
     tests,
     )
-from bzrlib.merge3 import Merge3
 from bzrlib.errors import CantReprocessAndShowBase, BinaryFile
 
 def split_lines(t):

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_merge_core.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,11 @@
     merge_directive,
     osutils,
     )
-from bzrlib.conflicts import ContentsConflict, TextConflict, PathConflict
+from bzrlib.conflicts import (
+    ContentsConflict,
+    TextConflict,
+    PathConflict,
+    )
 from bzrlib.merge import (
     Merge3Merger,
     Diff3Merger,
@@ -32,7 +36,6 @@
     Merger,
     )
 from bzrlib.osutils import getcwd, pathjoin
-from bzrlib import progress
 from bzrlib.transform import TreeTransform
 from bzrlib.tests import TestCaseWithTransport, TestSkipped
 from bzrlib.workingtree import WorkingTree

=== modified file 'bzrlib/tests/test_missing.py'
--- bzrlib/tests/test_missing.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_missing.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,9 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import os
-
-
 from bzrlib import (
     missing,
     tests,
@@ -25,7 +22,6 @@
     iter_log_revisions,
     )
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.workingtree import WorkingTree
 
 
 class TestMissing(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2010-11-11 13:45:02 +0000
+++ bzrlib/tests/test_msgeditor.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,8 +29,6 @@
     tests,
     trace,
     )
-from bzrlib.branch import Branch
-from bzrlib.config import ensure_config_dir_exists, config_filename
 from bzrlib.msgeditor import (
     make_commit_message_template_encoded,
     edit_commit_message_encoded
@@ -157,7 +155,7 @@
             return './fed.sh'
 
     def test_run_editor(self):
-        os.environ['BZR_EDITOR'] = self.make_do_nothing_editor()
+        self.overrideEnv('BZR_EDITOR', self.make_do_nothing_editor())
         self.assertEqual(True, msgeditor._run_editor(''),
                          'Unable to run dummy fake editor')
 
@@ -191,11 +189,11 @@
 "%s" fed.py %%1
 """ % sys.executable)
             f.close()
-            os.environ['BZR_EDITOR'] = 'fed.bat'
+            self.overrideEnv('BZR_EDITOR', 'fed.bat')
         else:
             # [non-win32] make python script executable and set BZR_EDITOR
             os.chmod('fed.py', 0755)
-            os.environ['BZR_EDITOR'] = './fed.py'
+            self.overrideEnv('BZR_EDITOR', './fed.py')
 
     def test_edit_commit_message(self):
         working_tree = self.make_uncommitted_tree()
@@ -230,16 +228,17 @@
         working_tree = self.make_uncommitted_tree()
 
         if sys.platform == 'win32':
-            os.environ['BZR_EDITOR'] = 'cmd.exe /c del'
+            editor = 'cmd.exe /c del'
         else:
-            os.environ['BZR_EDITOR'] = 'rm'
+            editor = 'rm'
+        self.overrideEnv('BZR_EDITOR', editor)
 
         self.assertRaises((IOError, OSError), msgeditor.edit_commit_message, '')
 
     def test__get_editor(self):
-        os.environ['BZR_EDITOR'] = 'bzr_editor'
-        os.environ['VISUAL'] = 'visual'
-        os.environ['EDITOR'] = 'editor'
+        self.overrideEnv('BZR_EDITOR', 'bzr_editor')
+        self.overrideEnv('VISUAL', 'visual')
+        self.overrideEnv('EDITOR', 'editor')
 
         conf = config.GlobalConfig.from_string('editor = config_editor\n',
                                                save=True)
@@ -259,7 +258,7 @@
 
     def test__run_editor_EACCES(self):
         """If running a configured editor raises EACESS, the user is warned."""
-        os.environ['BZR_EDITOR'] = 'eacces.py'
+        self.overrideEnv('BZR_EDITOR', 'eacces.py')
         f = file('eacces.py', 'wb')
         f.write('# Not a real editor')
         f.close()
@@ -267,7 +266,7 @@
         os.chmod('eacces.py', 0)
         # Set $EDITOR so that _run_editor will terminate before trying real
         # editors.
-        os.environ['EDITOR'] = self.make_do_nothing_editor()
+        self.overrideEnv('EDITOR', self.make_do_nothing_editor())
         # Call _run_editor, capturing mutter.warning calls.
         warnings = []
         def warning(*args):
@@ -319,23 +318,20 @@
         self.assertFileEqual('', msgfilename)
 
     def test_unsupported_encoding_commit_message(self):
-        old_env = osutils.set_or_unset_env('LANG', 'C')
-        try:
-            # LANG env variable has no effect on Windows
-            # but some characters anyway cannot be represented
-            # in default user encoding
-            char = probe_bad_non_ascii(osutils.get_user_encoding())
-            if char is None:
-                raise TestSkipped('Cannot find suitable non-ascii character '
-                    'for user_encoding (%s)' % osutils.get_user_encoding())
-
-            self.make_fake_editor(message=char)
-
-            working_tree = self.make_uncommitted_tree()
-            self.assertRaises(errors.BadCommitMessageEncoding,
-                              msgeditor.edit_commit_message, '')
-        finally:
-            osutils.set_or_unset_env('LANG', old_env)
+        self.overrideEnv('LANG', 'C')
+        # LANG env variable has no effect on Windows
+        # but some characters anyway cannot be represented
+        # in default user encoding
+        char = probe_bad_non_ascii(osutils.get_user_encoding())
+        if char is None:
+            raise TestSkipped('Cannot find suitable non-ascii character '
+                'for user_encoding (%s)' % osutils.get_user_encoding())
+
+        self.make_fake_editor(message=char)
+
+        working_tree = self.make_uncommitted_tree()
+        self.assertRaises(errors.BadCommitMessageEncoding,
+                          msgeditor.edit_commit_message, '')
 
     def test_generate_commit_message_template_no_hooks(self):
         commit_obj = commit.Commit()

=== modified file 'bzrlib/tests/test_multiparent.py'
--- bzrlib/tests/test_multiparent.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_multiparent.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 from unittest import TestCase
 
 from bzrlib import (
-    knit,
     multiparent,
     patiencediff,
     tests,

=== modified file 'bzrlib/tests/test_nonascii.py'
--- bzrlib/tests/test_nonascii.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_nonascii.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 from bzrlib import osutils
 from bzrlib.osutils import pathjoin
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
-from bzrlib.workingtree import WorkingTree
 
 
 class NonAsciiTest(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2011-01-14 05:14:04 +0000
+++ bzrlib/tests/test_options.py	2011-01-19 18:40:15 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2010-12-10 00:12:42 +0000
+++ bzrlib/tests/test_osutils.py	2011-01-12 15:50:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1899,17 +1899,17 @@
         self.assertIsInstance(concurrency, int)
 
     def test_local_concurrency_environment_variable(self):
-        os.environ['BZR_CONCURRENCY'] = '2'
+        self.overrideEnv('BZR_CONCURRENCY', '2')
         self.assertEqual(2, osutils.local_concurrency(use_cache=False))
-        os.environ['BZR_CONCURRENCY'] = '3'
+        self.overrideEnv('BZR_CONCURRENCY', '3')
         self.assertEqual(3, osutils.local_concurrency(use_cache=False))
-        os.environ['BZR_CONCURRENCY'] = 'foo'
+        self.overrideEnv('BZR_CONCURRENCY', 'foo')
         self.assertEqual(1, osutils.local_concurrency(use_cache=False))
 
     def test_option_concurrency(self):
-        os.environ['BZR_CONCURRENCY'] = '1'
+        self.overrideEnv('BZR_CONCURRENCY', '1')
         self.run_bzr('rocks --concurrency 42')
-        # Command line overrides envrionment variable
+        # Command line overrides environment variable
         self.assertEquals('42', os.environ['BZR_CONCURRENCY'])
         self.assertEquals(42, osutils.local_concurrency(use_cache=False))
 
@@ -1985,19 +1985,23 @@
     def test_defaults_to_BZR_COLUMNS(self):
         # BZR_COLUMNS is set by the test framework
         self.assertNotEqual('12', os.environ['BZR_COLUMNS'])
-        os.environ['BZR_COLUMNS'] = '12'
+        self.overrideEnv('BZR_COLUMNS', '12')
         self.assertEqual(12, osutils.terminal_width())
 
+    def test_BZR_COLUMNS_0_no_limit(self):
+        self.overrideEnv('BZR_COLUMNS', '0')
+        self.assertEqual(None, osutils.terminal_width())
+
     def test_falls_back_to_COLUMNS(self):
-        del os.environ['BZR_COLUMNS']
+        self.overrideEnv('BZR_COLUMNS', None)
         self.assertNotEqual('42', os.environ['COLUMNS'])
         self.set_fake_tty()
-        os.environ['COLUMNS'] = '42'
+        self.overrideEnv('COLUMNS', '42')
         self.assertEqual(42, osutils.terminal_width())
 
     def test_tty_default_without_columns(self):
-        del os.environ['BZR_COLUMNS']
-        del os.environ['COLUMNS']
+        self.overrideEnv('BZR_COLUMNS', None)
+        self.overrideEnv('COLUMNS', None)
 
         def terminal_size(w, h):
             return 42, 42
@@ -2010,8 +2014,8 @@
         self.assertEqual(42, osutils.terminal_width())
 
     def test_non_tty_default_without_columns(self):
-        del os.environ['BZR_COLUMNS']
-        del os.environ['COLUMNS']
+        self.overrideEnv('BZR_COLUMNS', None)
+        self.overrideEnv('COLUMNS', None)
         self.replace_stdout(None)
         self.assertEqual(None, osutils.terminal_width())
 
@@ -2027,8 +2031,8 @@
         else:
             self.overrideAttr(termios, 'TIOCGWINSZ')
             del termios.TIOCGWINSZ
-        del os.environ['BZR_COLUMNS']
-        del os.environ['COLUMNS']
+        self.overrideEnv('BZR_COLUMNS', None)
+        self.overrideEnv('COLUMNS', None)
         # Whatever the result is, if we don't raise an exception, it's ok.
         osutils.terminal_width()
 
@@ -2070,7 +2074,7 @@
 class TestGetuserUnicode(tests.TestCase):
 
     def test_ascii_user(self):
-        osutils.set_or_unset_env('LOGNAME', 'jrandom')
+        self.overrideEnv('LOGNAME', 'jrandom')
         self.assertEqual(u'jrandom', osutils.getuser_unicode())
 
     def test_unicode_user(self):
@@ -2082,9 +2086,9 @@
                 % (osutils.get_user_encoding(),))
         uni_username = u'jrandom' + uni_val
         encoded_username = uni_username.encode(ue)
-        osutils.set_or_unset_env('LOGNAME', encoded_username)
+        self.overrideEnv('LOGNAME', encoded_username)
         self.assertEqual(uni_username, osutils.getuser_unicode())
-        osutils.set_or_unset_env('LOGNAME', u'jrandom\xb6'.encode(ue))
+        self.overrideEnv('LOGNAME', u'jrandom\xb6'.encode(ue))
         self.assertEqual(u'jrandom\xb6', osutils.getuser_unicode())
 
 class TestBackupNames(tests.TestCase):

=== modified file 'bzrlib/tests/test_osutils_encodings.py'
--- bzrlib/tests/test_osutils_encodings.py	2010-06-29 16:21:13 +0000
+++ bzrlib/tests/test_osutils_encodings.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,11 +18,9 @@
 
 import codecs
 import locale
-import os
 import sys
 
 from bzrlib import (
-    errors,
     osutils,
     )
 from bzrlib.tests import (
@@ -171,8 +169,6 @@
     def setUp(self):
         TestCase.setUp(self)
         self.overrideAttr(locale, 'getpreferredencoding')
-        self.addCleanup(osutils.set_or_unset_env,
-                        'LANG', os.environ.get('LANG'))
         self.overrideAttr(sys, 'stderr', StringIOWrapper())
 
     def test_get_user_encoding(self):
@@ -181,7 +177,8 @@
 
         locale.getpreferredencoding = f
         fake_codec.add('user_encoding')
-        self.assertEquals('user_encoding', osutils.get_user_encoding(use_cache=False))
+        self.assertEquals('user_encoding',
+                          osutils.get_user_encoding(use_cache=False))
         self.assertEquals('', sys.stderr.getvalue())
 
     def test_user_cp0(self):
@@ -216,7 +213,7 @@
             raise locale.Error, 'unsupported locale'
 
         locale.getpreferredencoding = f
-        os.environ['LANG'] = 'BOGUS'
+        self.overrideEnv('LANG', 'BOGUS')
         self.assertEquals('ascii', osutils.get_user_encoding(use_cache=False))
         self.assertEquals('bzr: warning: unsupported locale\n'
                           '  Could not determine what text encoding to use.\n'

=== modified file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_permissions.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -32,8 +32,6 @@
 
 import os
 import sys
-import stat
-from cStringIO import StringIO
 import urllib
 
 from bzrlib import transport

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2010-06-11 08:02:42 +0000
+++ bzrlib/tests/test_plugins.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -656,7 +656,7 @@
                     self.fail('No path to global plugins')
 
     def test_get_standard_plugins_path_env(self):
-        os.environ['BZR_PLUGIN_PATH'] = 'foo/'
+        self.overrideEnv('BZR_PLUGIN_PATH', 'foo/')
         path = plugin.get_standard_plugins_path()
         for directory in path:
             self.assertNotContainsRe(directory, r'\\/$')
@@ -692,7 +692,7 @@
 
     def _set_path(self, *args):
         path = os.pathsep.join(self._list2paths(*args))
-        osutils.set_or_unset_env('BZR_PLUGIN_PATH', path)
+        self.overrideEnv('BZR_PLUGIN_PATH', path)
 
     def check_path(self, expected_dirs, setting_dirs):
         if setting_dirs:
@@ -779,7 +779,7 @@
         self.addCleanup(self._unregister_plugin, 'test_foo')
 
     def test_cannot_import(self):
-        osutils.set_or_unset_env('BZR_DISABLE_PLUGINS', 'test_foo')
+        self.overrideEnv('BZR_DISABLE_PLUGINS', 'test_foo')
         plugin.set_plugins_path(['.'])
         try:
             import bzrlib.plugins.test_foo
@@ -801,7 +801,7 @@
         self.overrideAttr(trace, 'warning', captured_warning)
         # Reset the flag that protect against double loading
         self.overrideAttr(plugin, '_loaded', False)
-        osutils.set_or_unset_env('BZR_DISABLE_PLUGINS', 'test_foo')
+        self.overrideEnv('BZR_DISABLE_PLUGINS', 'test_foo')
         plugin.load_plugins(['.'])
         self.assertPluginUnknown('test_foo')
         # Make sure we don't warn about the plugin ImportError since this has
@@ -859,7 +859,7 @@
         self.assertTestFooLoadedFrom('standard/test_foo')
 
     def test_import(self):
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.set_plugins_path(['standard'])
         try:
             import bzrlib.plugins.test_foo
@@ -868,12 +868,12 @@
         self.assertTestFooLoadedFrom('non-standard-dir')
 
     def test_loading(self):
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.load_plugins(['standard'])
         self.assertTestFooLoadedFrom('non-standard-dir')
 
     def test_compiled_loaded(self):
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.load_plugins(['standard'])
         self.assertTestFooLoadedFrom('non-standard-dir')
         self.assertIsSameRealPath('non-standard-dir/__init__.py',
@@ -896,7 +896,7 @@
         self.create_plugin_package('test_bar', dir='non-standard-dir/test_bar')
         self.addCleanup(self._unregister_plugin_submodule,
                         'test_foo', 'test_bar')
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.set_plugins_path(['standard'])
         import bzrlib.plugins.test_foo
         self.assertEqual('bzrlib.plugins.test_foo',
@@ -913,7 +913,7 @@
         self.create_plugin_package('test_bar', dir='another-dir/test_bar')
         self.addCleanup(self._unregister_plugin_submodule,
                         'test_foo', 'test_bar')
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@another-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@another-dir')
         plugin.set_plugins_path(['standard'])
         import bzrlib.plugins.test_foo
         self.assertEqual('bzrlib.plugins.test_foo',
@@ -928,7 +928,7 @@
         random = 'non-standard-dir/setup.py'
         os.rename(init, random)
         self.addCleanup(os.rename, random, init)
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.load_plugins(['standard'])
         self.assertPluginUnknown('test_foo')
 
@@ -942,6 +942,6 @@
 ''' % ('test_foo', plugin_path)
         self.create_plugin('test_foo', source=source,
                            dir=plugin_dir, file_name=plugin_file_name)
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@%s' % plugin_path)
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@%s' % plugin_path)
         plugin.load_plugins(['standard'])
         self.assertTestFooLoadedFrom(plugin_path)

=== modified file 'bzrlib/tests/test_progress.py'
--- bzrlib/tests/test_progress.py	2010-08-05 08:18:06 +0000
+++ bzrlib/tests/test_progress.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,16 +15,11 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-import os
 from StringIO import StringIO
 
-from bzrlib import errors
 from bzrlib.progress import (
     ProgressTask,
     )
-from bzrlib.symbol_versioning import (
-    deprecated_in,
-    )
 from bzrlib.tests import TestCase
 from bzrlib.ui.text import (
     TextProgressView,

=== modified file 'bzrlib/tests/test_read_bundle.py'
--- bzrlib/tests/test_read_bundle.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test_read_bundle.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,18 +17,15 @@
 """Test read_bundle works properly across various transports."""
 
 import cStringIO
-import os
 
 import bzrlib.bundle
 from bzrlib.bundle.serializer import write_bundle
 import bzrlib.bzrdir
 import bzrlib.errors as errors
-from bzrlib.symbol_versioning import deprecated_in
 from bzrlib import tests
 from bzrlib.tests.test_transport import TestTransportImplementation
 from bzrlib.tests.per_transport import transport_test_permutations
 import bzrlib.transport
-from bzrlib.transport.memory import MemoryTransport
 import bzrlib.urlutils
 from bzrlib.tests.scenarios import load_tests_apply_scenarios
 
@@ -69,7 +66,7 @@
         # from the one we want to test, so we must inject a correct transport
         # into possible_transports first).
         self.possible_transports = [self.get_transport(self.bundle_name)]
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         wt = self.create_test_bundle()
 
     def read_mergeable_from_url(self, url):

=== modified file 'bzrlib/tests/test_reconcile.py'
--- bzrlib/tests/test_reconcile.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_reconcile.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,9 +23,7 @@
     tests,
     )
 from bzrlib.reconcile import reconcile, Reconciler
-from bzrlib.revision import Revision
 from bzrlib.tests import per_repository
-from bzrlib.workingtree import WorkingTree
 
 
 class TestWorksWithSharedRepositories(per_repository.TestCaseWithRepository):

=== modified file 'bzrlib/tests/test_registry.py'
--- bzrlib/tests/test_registry.py	2010-11-05 20:54:32 +0000
+++ bzrlib/tests/test_registry.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 
 from bzrlib import (
     branch,
-    errors,
     osutils,
     registry,
     tests,

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test_remote.py	2011-01-14 17:18:23 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -57,12 +57,13 @@
     )
 from bzrlib.repofmt import groupcompress_repo, pack_repo
 from bzrlib.revision import NULL_REVISION
-from bzrlib.smart import medium
+from bzrlib.smart import medium, request
 from bzrlib.smart.client import _SmartClient
-from bzrlib.smart.repository import SmartServerRepositoryGetParentMap
+from bzrlib.smart.repository import (
+    SmartServerRepositoryGetParentMap,
+    SmartServerRepositoryGetStream_1_19,
+    )
 from bzrlib.tests import (
-    condition_isinstance,
-    multiply_tests,
     test_server,
     )
 from bzrlib.tests.scenarios import load_tests_apply_scenarios
@@ -71,7 +72,7 @@
     RemoteTransport,
     RemoteSSHTransport,
     RemoteTCPTransport,
-)
+    )
 
 
 load_tests = load_tests_apply_scenarios
@@ -3182,11 +3183,63 @@
 
     def test_copy_content_into_avoids_revision_history(self):
         local = self.make_branch('local')
-        remote_backing_tree = self.make_branch_and_tree('remote')
-        remote_backing_tree.commit("Commit.")
+        builder = self.make_branch_builder('remote')
+        builder.build_commit(message="Commit.")
         remote_branch_url = self.smart_server.get_url() + 'remote'
         remote_branch = bzrdir.BzrDir.open(remote_branch_url).open_branch()
         local.repository.fetch(remote_branch.repository)
         self.hpss_calls = []
         remote_branch.copy_content_into(local)
         self.assertFalse('Branch.revision_history' in self.hpss_calls)
+
+    def test_fetch_everything_needs_just_one_call(self):
+        local = self.make_branch('local')
+        builder = self.make_branch_builder('remote')
+        builder.build_commit(message="Commit.")
+        remote_branch_url = self.smart_server.get_url() + 'remote'
+        remote_branch = bzrdir.BzrDir.open(remote_branch_url).open_branch()
+        self.hpss_calls = []
+        local.repository.fetch(remote_branch.repository,
+                fetch_spec=graph.EverythingResult(remote_branch.repository))
+        self.assertEqual(['Repository.get_stream_1.19'], self.hpss_calls)
+
+    def override_verb(self, verb_name, verb):
+        request_handlers = request.request_handlers
+        orig_verb = request_handlers.get(verb_name)
+        request_handlers.register(verb_name, verb, override_existing=True)
+        self.addCleanup(request_handlers.register, verb_name, orig_verb,
+                override_existing=True)
+
+    def test_fetch_everything_backwards_compat(self):
+        """Can fetch with EverythingResult even with pre 2.3 servers.
+        
+        Pre-2.3 do not support 'everything' searches with the
+        Repository.get_stream_1.19 verb.
+        """
+        verb_log = []
+        class OldGetStreamVerb(SmartServerRepositoryGetStream_1_19):
+            """A version of the Repository.get_stream_1.19 verb patched to
+            reject 'everything' searches the way 2.2 and earlier do.
+            """
+            def recreate_search(self, repository, search_bytes, discard_excess=False):
+                verb_log.append(search_bytes.split('\n', 1)[0])
+                if search_bytes == 'everything':
+                    return (None, request.FailedSmartServerResponse(('BadSearch',)))
+                return super(OldGetStreamVerb,
+                        self).recreate_search(repository, search_bytes,
+                            discard_excess=discard_excess)
+        self.override_verb('Repository.get_stream_1.19', OldGetStreamVerb)
+        local = self.make_branch('local')
+        builder = self.make_branch_builder('remote')
+        builder.build_commit(message="Commit.")
+        remote_branch_url = self.smart_server.get_url() + 'remote'
+        remote_branch = bzrdir.BzrDir.open(remote_branch_url).open_branch()
+        self.hpss_calls = []
+        local.repository.fetch(remote_branch.repository,
+                fetch_spec=graph.EverythingResult(remote_branch.repository))
+        # make sure the overridden verb was used
+        self.assertLength(1, verb_log)
+        # more than one HPSS call is needed, but because it's a VFS callback
+        # its hard to predict exactly how many.
+        self.assertTrue(len(self.hpss_calls) > 1)
+

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2010-11-22 22:27:58 +0000
+++ bzrlib/tests/test_repository.py	2010-11-24 07:09:55 +0000
@@ -1659,7 +1659,7 @@
         self.orig_pack = target.pack
         target.pack = self.log_pack
         search = target.search_missing_revision_ids(
-            source_tree.branch.repository, tip)
+            source_tree.branch.repository, revision_ids=[tip])
         stream = source.get_stream(search)
         from_format = source_tree.branch.repository._format
         sink = target._get_sink()

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2010-07-15 12:37:32 +0000
+++ bzrlib/tests/test_revision.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,26 +15,19 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-import os
 import warnings
 
 from bzrlib import (
     bugtracker,
     revision,
-    symbol_versioning,
     )
-from bzrlib.branch import Branch
 from bzrlib.errors import (
     InvalidBugStatus,
     InvalidLineInBugsProperty,
-    NoSuchRevision,
     )
-from bzrlib.deprecated_graph import Graph
-from bzrlib.revision import (find_present_ancestors,
-                             NULL_REVISION)
+from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCase, TestCaseWithTransport
 from bzrlib.trace import mutter
-from bzrlib.workingtree import WorkingTree
 
 # We're allowed to test deprecated interfaces
 warnings.filterwarnings('ignore',

=== modified file 'bzrlib/tests/test_revisionspec.py'
--- bzrlib/tests/test_revisionspec.py	2010-09-24 02:19:53 +0000
+++ bzrlib/tests/test_revisionspec.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,20 +15,15 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import datetime
-import os
 import time
 
 from bzrlib import (
-    branch,
-    bzrdir,
     errors,
-    repository,
     revision as _mod_revision,
     )
-from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.revisionspec import (
     RevisionSpec,
-    RevisionSpec_revno,
     RevisionSpec_tag,
     )
 

=== modified file 'bzrlib/tests/test_revisiontree.py'
--- bzrlib/tests/test_revisiontree.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_revisiontree.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2008-2011 Canonical Ltd
 # Authors:  Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -21,8 +21,6 @@
     errors,
     revision,
     )
-import bzrlib
-from bzrlib.inventory import ROOT_ID
 from bzrlib.tests import TestCaseWithTransport
 
 

=== modified file 'bzrlib/tests/test_rio.py'
--- bzrlib/tests/test_rio.py	2010-09-25 20:08:01 +0000
+++ bzrlib/tests/test_rio.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,18 +22,20 @@
 but this depends on the transport.
 """
 
-import cStringIO
-import os
 import re
-import sys
 from tempfile import TemporaryFile
 
 from bzrlib import (
     rio,
     )
-from bzrlib.tests import TestCaseInTempDir, TestCase
-from bzrlib.rio import (RioWriter, Stanza, read_stanza, read_stanzas, rio_file,
-                        RioReader)
+from bzrlib.tests import TestCase
+from bzrlib.rio import (
+    RioReader,
+    Stanza,
+    read_stanza,
+    read_stanzas,
+    rio_file,
+    )
 
 
 class TestRio(TestCase):

=== modified file 'bzrlib/tests/test_rules.py'
--- bzrlib/tests/test_rules.py	2010-01-14 15:24:04 +0000
+++ bzrlib/tests/test_rules.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,16 +16,13 @@
 
 """Tests for finding, parsing and searching rule-based preferences."""
 
-import os
 import sys
 
 from bzrlib import (
-    config,
     errors,
     rules,
     tests,
     )
-from bzrlib.util.configobj import configobj
 
 
 class TestIniBasedRulesSearcher(tests.TestCase):
@@ -118,10 +115,10 @@
 
     def setUp(self):
         super(TestRulesPath, self).setUp()
-        os.environ['HOME'] = '/home/bogus'
+        self.overrideEnv('HOME', '/home/bogus')
         if sys.platform == 'win32':
-            os.environ['BZR_HOME'] = \
-                r'C:\Documents and Settings\bogus\Application Data'
+            self.overrideEnv(
+                'BZR_HOME', r'C:\Documents and Settings\bogus\Application Data')
             self.bzr_home = \
                 'C:/Documents and Settings/bogus/Application Data/bazaar/2.0'
         else:

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2010-11-19 23:03:19 +0000
+++ bzrlib/tests/test_selftest.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 """Tests for the test framework."""
 
 from cStringIO import StringIO
-from doctest import ELLIPSIS
+import doctest
 import os
 import signal
 import sys
@@ -42,7 +42,6 @@
 from bzrlib import (
     branchbuilder,
     bzrdir,
-    debug,
     errors,
     lockdir,
     memorytree,
@@ -56,7 +55,6 @@
     )
 from bzrlib.repofmt import (
     groupcompress_repo,
-    pack_repo,
     weaverepo,
     )
 from bzrlib.symbol_versioning import (
@@ -68,12 +66,10 @@
     features,
     test_lsprof,
     test_server,
-    test_sftp_transport,
     TestUtil,
     )
 from bzrlib.trace import note, mutter
 from bzrlib.transport import memory
-from bzrlib.version import _get_bzr_source_tree
 
 
 def _test_ids(test_suite):
@@ -92,7 +88,7 @@
             "text", "plain", {"charset": "utf8"})))
         self.assertThat(u"".join(log.iter_text()), Equals(self.get_log()))
         self.assertThat(self.get_log(),
-            DocTestMatches(u"...a test message\n", ELLIPSIS))
+            DocTestMatches(u"...a test message\n", doctest.ELLIPSIS))
 
 
 class TestUnicodeFilename(tests.TestCase):
@@ -3210,7 +3206,8 @@
         tpr.register('bar', 'bBB.aAA.rRR')
         self.assertEquals('bbb.aaa.rrr', tpr.get('bar'))
         self.assertThat(self.get_log(),
-            DocTestMatches("...bar...bbb.aaa.rrr...BB.aAA.rRR", ELLIPSIS))
+            DocTestMatches("...bar...bbb.aaa.rrr...BB.aAA.rRR",
+                           doctest.ELLIPSIS))
 
     def test_get_unknown_prefix(self):
         tpr = self._get_registry()
@@ -3251,10 +3248,8 @@
         class Test(unittest.TestCase):
             def runTest(self):
                 pass
-            addCleanup = None # for when on Python 2.7 with native addCleanup
         result = self.LeakRecordingResult()
         test = Test()
-        self.assertIs(getattr(test, "addCleanup", None), None)
         result.startTestRun()
         test.run(result)
         result.stopTestRun()
@@ -3387,10 +3382,9 @@
         result = tests.ExtendedTestResult(StringIO(), 0, 1)
         post_mortem_calls = []
         self.overrideAttr(pdb, "post_mortem", post_mortem_calls.append)
-        self.addCleanup(osutils.set_or_unset_env, "BZR_TEST_PDB",
-            osutils.set_or_unset_env("BZR_TEST_PDB", None))
+        self.overrideEnv('BZR_TEST_PDB', None)
         result._post_mortem(1)
-        os.environ["BZR_TEST_PDB"] = "on"
+        self.overrideEnv('BZR_TEST_PDB', 'on')
         result._post_mortem(2)
         self.assertEqual([2], post_mortem_calls)
 
@@ -3411,3 +3405,144 @@
                                                 self.verbosity)
         tests.run_suite(suite, runner_class=MyRunner, stream=StringIO())
         self.assertLength(1, calls)
+
+
+class TestEnvironHandling(tests.TestCase):
+
+    def test_overrideEnv_None_called_twice_doesnt_leak(self):
+        self.failIf('MYVAR' in os.environ)
+        self.overrideEnv('MYVAR', '42')
+        # We use an embedded test to make sure we fix the _captureVar bug
+        class Test(tests.TestCase):
+            def test_me(self):
+                # The first call save the 42 value
+                self.overrideEnv('MYVAR', None)
+                self.assertEquals(None, os.environ.get('MYVAR'))
+                # Make sure we can call it twice
+                self.overrideEnv('MYVAR', None)
+                self.assertEquals(None, os.environ.get('MYVAR'))
+        output = StringIO()
+        result = tests.TextTestResult(output, 0, 1)
+        Test('test_me').run(result)
+        if not result.wasStrictlySuccessful():
+            self.fail(output.getvalue())
+        # We get our value back
+        self.assertEquals('42', os.environ.get('MYVAR'))
+
+
+class TestIsolatedEnv(tests.TestCase):
+    """Test isolating tests from os.environ.
+
+    Since we use tests that are already isolated from os.environ a bit of care
+    should be taken when designing the tests to avoid bootstrap side-effects.
+    The tests start an already clean os.environ which allow doing valid
+    assertions about which variables are present or not and design tests around
+    these assertions.
+    """
+
+    class ScratchMonkey(tests.TestCase):
+
+        def test_me(self):
+            pass
+
+    def test_basics(self):
+        # Make sure we know the definition of BZR_HOME: not part of os.environ
+        # for tests.TestCase.
+        self.assertTrue('BZR_HOME' in tests.isolated_environ)
+        self.assertEquals(None, tests.isolated_environ['BZR_HOME'])
+        # Being part of isolated_environ, BZR_HOME should not appear here
+        self.assertFalse('BZR_HOME' in os.environ)
+        # Make sure we know the definition of LINES: part of os.environ for
+        # tests.TestCase
+        self.assertTrue('LINES' in tests.isolated_environ)
+        self.assertEquals('25', tests.isolated_environ['LINES'])
+        self.assertEquals('25', os.environ['LINES'])
+
+    def test_injecting_unknown_variable(self):
+        # BZR_HOME is known to be absent from os.environ
+        test = self.ScratchMonkey('test_me')
+        tests.override_os_environ(test, {'BZR_HOME': 'foo'})
+        self.assertEquals('foo', os.environ['BZR_HOME'])
+        tests.restore_os_environ(test)
+        self.assertFalse('BZR_HOME' in os.environ)
+
+    def test_injecting_known_variable(self):
+        test = self.ScratchMonkey('test_me')
+        # LINES is known to be present in os.environ
+        tests.override_os_environ(test, {'LINES': '42'})
+        self.assertEquals('42', os.environ['LINES'])
+        tests.restore_os_environ(test)
+        self.assertEquals('25', os.environ['LINES'])
+
+    def test_deleting_variable(self):
+        test = self.ScratchMonkey('test_me')
+        # LINES is known to be present in os.environ
+        tests.override_os_environ(test, {'LINES': None})
+        self.assertTrue('LINES' not in os.environ)
+        tests.restore_os_environ(test)
+        self.assertEquals('25', os.environ['LINES'])
+
+
+class TestDocTestSuiteIsolation(tests.TestCase):
+    """Test that `tests.DocTestSuite` isolates doc tests from os.environ.
+
+    Since tests.TestCase alreay provides an isolation from os.environ, we use
+    the clean environment as a base for testing. To precisely capture the
+    isolation provided by tests.DocTestSuite, we use doctest.DocTestSuite to
+    compare against.
+
+    We want to make sure `tests.DocTestSuite` respect `tests.isolated_environ`,
+    not `os.environ` so each test overrides it to suit its needs.
+
+    """
+
+    def get_doctest_suite_for_string(self, klass, string):
+        class Finder(doctest.DocTestFinder):
+
+            def find(*args, **kwargs):
+                test = doctest.DocTestParser().get_doctest(
+                    string, {}, 'foo', 'foo.py', 0)
+                return [test]
+
+        suite = klass(test_finder=Finder())
+        return suite
+
+    def run_doctest_suite_for_string(self, klass, string):
+        suite = self.get_doctest_suite_for_string(klass, string)
+        output = StringIO()
+        result = tests.TextTestResult(output, 0, 1)
+        suite.run(result)
+        return result, output
+
+    def assertDocTestStringSucceds(self, klass, string):
+        result, output = self.run_doctest_suite_for_string(klass, string)
+        if not result.wasStrictlySuccessful():
+            self.fail(output.getvalue())
+
+    def assertDocTestStringFails(self, klass, string):
+        result, output = self.run_doctest_suite_for_string(klass, string)
+        if result.wasStrictlySuccessful():
+            self.fail(output.getvalue())
+
+    def test_injected_variable(self):
+        self.overrideAttr(tests, 'isolated_environ', {'LINES': '42'})
+        test = """
+            >>> import os
+            >>> os.environ['LINES']
+            '42'
+            """
+        # doctest.DocTestSuite fails as it sees '25'
+        self.assertDocTestStringFails(doctest.DocTestSuite, test)
+        # tests.DocTestSuite sees '42'
+        self.assertDocTestStringSucceds(tests.IsolatedDocTestSuite, test)
+
+    def test_deleted_variable(self):
+        self.overrideAttr(tests, 'isolated_environ', {'LINES': None})
+        test = """
+            >>> import os
+            >>> os.environ.get('LINES')
+            """
+        # doctest.DocTestSuite fails as it sees '25'
+        self.assertDocTestStringFails(doctest.DocTestSuite, test)
+        # tests.DocTestSuite sees None
+        self.assertDocTestStringSucceds(tests.IsolatedDocTestSuite, test)

=== modified file 'bzrlib/tests/test_server.py'
--- bzrlib/tests/test_server.py	2010-12-07 17:01:25 +0000
+++ bzrlib/tests/test_server.py	2011-01-14 22:36:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 import errno
 import socket
 import SocketServer
-import select
 import sys
 import threading
 
@@ -695,8 +694,6 @@
         server.SmartTCPServer.__init__(self, backing_transport,
                                        root_client_path)
     def serve(self):
-        # FIXME: No test are exercising the hooks for the test server
-        # -- vila 20100618
         self.run_server_started_hooks()
         try:
             TestingThreadingTCPServer.serve(self)
@@ -804,7 +801,3 @@
         """Get a backing transport from a server we are decorating."""
         url = 'readonly+' + backing_transport_server.get_url()
         return transport.get_transport(url)
-
-
-
-

=== modified file 'bzrlib/tests/test_setup.py'
--- bzrlib/tests/test_setup.py	2010-11-05 14:07:45 +0000
+++ bzrlib/tests/test_setup.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
 import os
 import sys
 import subprocess
-import shutil
-from tempfile import TemporaryFile
 
 import bzrlib
 from bzrlib.tests import TestCase, TestSkipped

=== modified file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	2010-06-23 08:17:20 +0000
+++ bzrlib/tests/test_sftp_transport.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Robey Pointer <robey@lag.net>
+# Copyright (C) 2005-2011 Robey Pointer <robey@lag.net>
 # Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,6 @@
 import os
 import socket
 import sys
-import threading
 import time
 
 from bzrlib import (
@@ -30,9 +29,7 @@
     ui,
     )
 from bzrlib.osutils import (
-    pathjoin,
     lexists,
-    set_or_unset_env,
     )
 from bzrlib.tests import (
     features,
@@ -47,8 +44,6 @@
     from bzrlib.transport import sftp as _mod_sftp
     from bzrlib.tests import stub_sftp
 
-from bzrlib.workingtree import WorkingTree
-
 
 def set_test_transport_to_sftp(testcase):
     """A helper to set transports on test case instances."""
@@ -174,27 +169,21 @@
         """Test that if no 'ssh' is available we get builtin paramiko"""
         from bzrlib.transport import ssh
         # set '.' as the only location in the path, forcing no 'ssh' to exist
-        orig_vendor = ssh._ssh_vendor_manager._cached_ssh_vendor
-        orig_path = set_or_unset_env('PATH', '.')
-        try:
-            # No vendor defined yet, query for one
-            ssh._ssh_vendor_manager.clear_cache()
-            vendor = ssh._get_ssh_vendor()
-            self.assertIsInstance(vendor, ssh.ParamikoVendor)
-        finally:
-            set_or_unset_env('PATH', orig_path)
-            ssh._ssh_vendor_manager._cached_ssh_vendor = orig_vendor
+        self.overrideAttr(ssh, '_ssh_vendor_manager')
+        self.overrideEnv('PATH', '.')
+        ssh._ssh_vendor_manager.clear_cache()
+        vendor = ssh._get_ssh_vendor()
+        self.assertIsInstance(vendor, ssh.ParamikoVendor)
 
     def test_abspath_root_sibling_server(self):
         server = stub_sftp.SFTPSiblingAbsoluteServer()
         server.start_server()
-        try:
-            transport = _mod_transport.get_transport(server.get_url())
-            self.assertFalse(transport.abspath('/').endswith('/~/'))
-            self.assertTrue(transport.abspath('/').endswith('/'))
-            del transport
-        finally:
-            server.stop_server()
+        self.addCleanup(server.stop_server)
+
+        transport = _mod_transport.get_transport(server.get_url())
+        self.assertFalse(transport.abspath('/').endswith('/~/'))
+        self.assertTrue(transport.abspath('/').endswith('/'))
+        del transport
 
 
 class SFTPBranchTest(TestCaseWithSFTPServer):

=== modified file 'bzrlib/tests/test_shelf.py'
--- bzrlib/tests/test_shelf.py	2010-09-17 15:15:34 +0000
+++ bzrlib/tests/test_shelf.py	2011-01-12 18:25:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -623,6 +623,29 @@
                               shelf_file)
         self.assertEqual('Shelf corrupt.', str(e))
 
+    def test_unshelve_subdir_in_now_removed_dir(self):
+        tree = self.make_branch_and_tree('.')
+        self.addCleanup(tree.lock_write().unlock)
+        self.build_tree(['dir/', 'dir/subdir/', 'dir/subdir/foo'])
+        tree.add(['dir'], ['dir-id'])
+        tree.commit('versioned dir')
+        tree.add(['dir/subdir', 'dir/subdir/foo'], ['subdir-id', 'foo-id'])
+        creator = shelf.ShelfCreator(tree, tree.basis_tree())
+        self.addCleanup(creator.finalize)
+        for change in creator.iter_shelvable():
+            creator.shelve_change(change)
+        shelf_manager = tree.get_shelf_manager()
+        shelf_id = shelf_manager.shelve_changes(creator)
+        self.failIfExists('dir/subdir')
+        tree.remove(['dir'])
+        unshelver = shelf_manager.get_unshelver(shelf_id)
+        self.addCleanup(unshelver.finalize)
+        unshelver.make_merger().do_merge()
+        self.failUnlessExists('dir/subdir/foo')
+        self.assertEqual('dir-id', tree.path2id('dir'))
+        self.assertEqual('subdir-id', tree.path2id('dir/subdir'))
+        self.assertEqual('foo-id', tree.path2id('dir/subdir/foo'))
+
 
 class TestShelfManager(tests.TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/test_smart.py'
--- bzrlib/tests/test_smart.py	2010-05-13 16:17:54 +0000
+++ bzrlib/tests/test_smart.py	2011-01-14 23:41:14 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,15 +25,11 @@
 """
 
 import bz2
-from cStringIO import StringIO
-import tarfile
 
 from bzrlib import (
-    bencode,
     branch as _mod_branch,
     bzrdir,
     errors,
-    pack,
     tests,
     transport,
     urlutils,
@@ -1474,7 +1470,7 @@
             request.execute('stacked', 1, (3, r3)))
 
 
-class TestSmartServerRepositoryGetStream(tests.TestCaseWithMemoryTransport):
+class GetStreamTestBase(tests.TestCaseWithMemoryTransport):
 
     def make_two_commit_repo(self):
         tree = self.make_branch_and_memory_tree('.')
@@ -1486,6 +1482,9 @@
         repo = tree.branch.repository
         return repo, r1, r2
 
+
+class TestSmartServerRepositoryGetStream(GetStreamTestBase):
+
     def test_ancestry_of(self):
         """The search argument may be a 'ancestry-of' some heads'."""
         backing = self.get_transport()
@@ -1512,6 +1511,18 @@
         stream_bytes = ''.join(response.body_stream)
         self.assertStartsWith(stream_bytes, 'Bazaar pack format 1')
 
+    def test_search_everything(self):
+        """A search of 'everything' returns a stream."""
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryGetStream_1_19(backing)
+        repo, r1, r2 = self.make_two_commit_repo()
+        serialised_fetch_spec = 'everything'
+        request.execute('', repo._format.network_name())
+        response = request.do_body(serialised_fetch_spec)
+        self.assertEqual(('ok',), response.args)
+        stream_bytes = ''.join(response.body_stream)
+        self.assertStartsWith(stream_bytes, 'Bazaar pack format 1')
+
 
 class TestSmartServerRequestHasRevision(tests.TestCaseWithMemoryTransport):
 
@@ -1911,6 +1922,8 @@
             smart_repo.SmartServerRepositoryGetRevisionGraph)
         self.assertHandlerEqual('Repository.get_stream',
             smart_repo.SmartServerRepositoryGetStream)
+        self.assertHandlerEqual('Repository.get_stream_1.19',
+            smart_repo.SmartServerRepositoryGetStream_1_19)
         self.assertHandlerEqual('Repository.has_revision',
             smart_repo.SmartServerRequestHasRevision)
         self.assertHandlerEqual('Repository.insert_stream',
@@ -1927,3 +1940,50 @@
             smart_repo.SmartServerRepositoryUnlock)
         self.assertHandlerEqual('Transport.is_readonly',
             smart_req.SmartServerIsReadonly)
+
+
+class SmartTCPServerHookTests(tests.TestCaseWithMemoryTransport):
+    """Tests for SmartTCPServer hooks."""
+
+    def setUp(self):
+        super(SmartTCPServerHookTests, self).setUp()
+        self.server = server.SmartTCPServer(self.get_transport())
+
+    def test_run_server_started_hooks(self):
+        """Test the server started hooks get fired properly."""
+        started_calls = []
+        server.SmartTCPServer.hooks.install_named_hook('server_started',
+            lambda backing_urls, url: started_calls.append((backing_urls, url)),
+            None)
+        started_ex_calls = []
+        server.SmartTCPServer.hooks.install_named_hook('server_started_ex',
+            lambda backing_urls, url: started_ex_calls.append((backing_urls, url)),
+            None)
+        self.server._sockname = ('example.com', 42)
+        self.server.run_server_started_hooks()
+        self.assertEquals(started_calls,
+            [([self.get_transport().base], 'bzr://example.com:42/')])
+        self.assertEquals(started_ex_calls,
+            [([self.get_transport().base], self.server)])
+
+    def test_run_server_started_hooks_ipv6(self):
+        """Test that socknames can contain 4-tuples."""
+        self.server._sockname = ('::', 42, 0, 0)
+        started_calls = []
+        server.SmartTCPServer.hooks.install_named_hook('server_started',
+            lambda backing_urls, url: started_calls.append((backing_urls, url)),
+            None)
+        self.server.run_server_started_hooks()
+        self.assertEquals(started_calls,
+                [([self.get_transport().base], 'bzr://:::42/')])
+
+    def test_run_server_stopped_hooks(self):
+        """Test the server stopped hooks."""
+        self.server._sockname = ('example.com', 42)
+        stopped_calls = []
+        server.SmartTCPServer.hooks.install_named_hook('server_stopped',
+            lambda backing_urls, url: stopped_calls.append((backing_urls, url)),
+            None)
+        self.server.run_server_stopped_hooks()
+        self.assertEquals(stopped_calls,
+            [([self.get_transport().base], 'bzr://example.com:42/')])

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2010-07-01 15:25:41 +0000
+++ bzrlib/tests/test_smart_transport.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -615,7 +615,7 @@
 
     def setUp(self):
         super(TestSmartServerStreamMedium, self).setUp()
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
 
     def portable_socket_pair(self):
         """Return a pair of TCP sockets connected to each other.
@@ -970,7 +970,7 @@
 
     def test_get_error_unexpected(self):
         """Error reported by server with no specific representation"""
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         class FlakyTransport(object):
             base = 'a_url'
             def external_url(self):
@@ -1077,14 +1077,14 @@
 
     def test_smart_transport_has(self):
         """Checking for file existence over smart."""
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         self.backing_transport.put_bytes("foo", "contents of foo\n")
         self.assertTrue(self.transport.has("foo"))
         self.assertFalse(self.transport.has("non-foo"))
 
     def test_smart_transport_get(self):
         """Read back a file over smart."""
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         self.backing_transport.put_bytes("foo", "contents\nof\nfoo\n")
         fp = self.transport.get("foo")
         self.assertEqual('contents\nof\nfoo\n', fp.read())
@@ -1094,7 +1094,7 @@
         # The path in a raised NoSuchFile exception should be the precise path
         # asked for by the client. This gives meaningful and unsurprising errors
         # for users.
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         err = self.assertRaises(
             errors.NoSuchFile, self.transport.get, 'not%20a%20file')
         self.assertSubset([err.path], ['not%20a%20file', './not%20a%20file'])
@@ -1119,7 +1119,7 @@
 
     def test_open_dir(self):
         """Test changing directory"""
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         transport = self.transport
         self.backing_transport.mkdir('toffee')
         self.backing_transport.mkdir('toffee/apple')
@@ -1147,7 +1147,7 @@
 
     def test_mkdir_error_readonly(self):
         """TransportNotPossible should be preserved from the backing transport."""
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         self.start_server(readonly=True)
         self.assertRaises(errors.TransportNotPossible, self.transport.mkdir,
             'foo')
@@ -1261,7 +1261,7 @@
 
     def setUp(self):
         super(SmartServerRequestHandlerTests, self).setUp()
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
 
     def build_handler(self, transport):
         """Returns a handler for the commands in protocol version one."""
@@ -1286,10 +1286,7 @@
         handler = vfs.HasRequest(None, '/')
         # set environment variable after construction to make sure it's
         # examined.
-        # Note that we can safely clobber BZR_NO_SMART_VFS here, because setUp
-        # has called _captureVar, so it will be restored to the right state
-        # afterwards.
-        os.environ['BZR_NO_SMART_VFS'] = ''
+        self.overrideEnv('BZR_NO_SMART_VFS', '')
         self.assertRaises(errors.DisabledMethod, handler.execute)
 
     def test_readonly_exception_becomes_transport_not_possible(self):
@@ -1602,7 +1599,7 @@
         self.assertTrue(self.end_received)
 
     def test_accept_request_and_body_all_at_once(self):
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         mem_transport = memory.MemoryTransport()
         mem_transport.put_bytes('foo', 'abcdefghij')
         out_stream = StringIO()
@@ -1868,7 +1865,7 @@
         self.assertTrue(self.end_received)
 
     def test_accept_request_and_body_all_at_once(self):
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         mem_transport = memory.MemoryTransport()
         mem_transport.put_bytes('foo', 'abcdefghij')
         out_stream = StringIO()
@@ -3517,7 +3514,7 @@
     def setUp(self):
         super(HTTPTunnellingSmokeTest, self).setUp()
         # We use the VFS layer as part of HTTP tunnelling tests.
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
 
     def test_smart_http_medium_request_accept_bytes(self):
         medium = FakeHTTPMedium()

=== modified file 'bzrlib/tests/test_smtp_connection.py'
--- bzrlib/tests/test_smtp_connection.py	2010-08-30 07:34:11 +0000
+++ bzrlib/tests/test_smtp_connection.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 from email.Message import Message
 import errno
 import smtplib

=== modified file 'bzrlib/tests/test_store.py'
--- bzrlib/tests/test_store.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_store.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007 Canonical Ltd
+# Copyright (C) 2005-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,8 +21,7 @@
 import gzip
 
 import bzrlib.errors as errors
-from bzrlib.errors import BzrError, UnlistableStore, NoSuchFile
-from bzrlib.transport.local import LocalTransport
+from bzrlib.errors import BzrError
 from bzrlib.store.text import TextStore
 from bzrlib.tests import TestCase, TestCaseInTempDir, TestCaseWithTransport
 import bzrlib.store as store

=== modified file 'bzrlib/tests/test_strace.py'
--- bzrlib/tests/test_strace.py	2010-09-25 20:08:01 +0000
+++ bzrlib/tests/test_strace.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
 
 """Tests for the strace-invoking support."""
 
-import errno
-import subprocess
 import threading
 
 from bzrlib import (

=== modified file 'bzrlib/tests/test_subsume.py'
--- bzrlib/tests/test_subsume.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_subsume.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from bzrlib import bzrdir, errors, repository, workingtree, tests
+from bzrlib import errors, workingtree, tests
 
 
 class TestWorkingTree(tests.TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_switch.py'
--- bzrlib/tests/test_switch.py	2010-05-03 04:08:50 +0000
+++ bzrlib/tests/test_switch.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
     merge as _mod_merge,
     switch,
     tests,
-)
+    )
 
 
 class TestSwitch(tests.TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_symbol_versioning.py'
--- bzrlib/tests/test_symbol_versioning.py	2010-06-25 06:11:21 +0000
+++ bzrlib/tests/test_symbol_versioning.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 from bzrlib.symbol_versioning import (
     deprecated_function,
     deprecated_in,
-    deprecated_list,
     deprecated_method,
     )
 from bzrlib.tests import TestCase

=== modified file 'bzrlib/tests/test_test_server.py'
--- bzrlib/tests/test_test_server.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test_test_server.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_timestamp.py'
--- bzrlib/tests/test_timestamp.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_timestamp.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,10 +14,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import random
+import time
+
+
 from bzrlib import (
     tests,
     timestamp,
     )
+from bzrlib.osutils import local_time_offset
+
 
 class TestPatchHeader(tests.TestCase):
 
@@ -52,3 +58,38 @@
         # offset of three minutes
         self.assertEqual((1173193459, +3 * 60),
             timestamp.parse_patch_date('2007-03-06 15:07:19 +0003'))
+
+
+class UnpackHighresDateTests(tests.TestCase):
+
+    def test_unpack_highres_date(self):
+        self.assertEquals(
+            (1120153132.3508501, -18000),
+            timestamp.unpack_highres_date('Thu 2005-06-30 12:38:52.350850105 -0500'))
+        self.assertEquals(
+            (1120153132.3508501, 0),
+            timestamp.unpack_highres_date('Thu 2005-06-30 17:38:52.350850105 +0000'))
+        self.assertEquals(
+            (1120153132.3508501, 7200),
+            timestamp.unpack_highres_date('Thu 2005-06-30 19:38:52.350850105 +0200'))
+        self.assertEquals(
+            (1152428738.867522, 19800),
+            timestamp.unpack_highres_date('Sun 2006-07-09 12:35:38.867522001 +0530'))
+
+    def test_random(self):
+        t = time.time()
+        o = local_time_offset()
+        t2, o2 = timestamp.unpack_highres_date(timestamp.format_highres_date(t, o))
+        self.assertEquals(t, t2)
+        self.assertEquals(o, o2)
+        t -= 24*3600*365*2 # Start 2 years ago
+        o = -12*3600
+        for count in xrange(500):
+            t += random.random()*24*3600*30
+            o = ((o/3600 + 13) % 25 - 12)*3600 # Add 1 wrap around from [-12, 12]
+            date = timestamp.format_highres_date(t, o)
+            t2, o2 = timestamp.unpack_highres_date(date)
+            self.assertEquals(t, t2,
+                'Failed on date %r, %s,%s diff:%s' % (date, t, o, t2-t))
+            self.assertEquals(o, o2,
+                'Failed on date %r, %s,%s diff:%s' % (date, t, o, t2-t))

=== modified file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	2010-10-11 09:53:16 +0000
+++ bzrlib/tests/test_trace.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -301,9 +301,7 @@
         # set up.
         self.overrideAttr(sys, 'stderr', StringIO())
         # Set the log file to something that cannot exist
-        # FIXME: A bit dangerous: we are not in an isolated dir here -- vilajam
-        # 20100125
-        os.environ['BZR_LOG'] = os.getcwd() + '/no-dir/bzr.log'
+        self.overrideEnv('BZR_LOG', os.getcwd() + '/no-dir/bzr.log')
         self.overrideAttr(trace, '_bzr_log_filename')
         logf = trace._open_bzr_log()
         self.assertIs(None, logf)

=== modified file 'bzrlib/tests/test_transactions.py'
--- bzrlib/tests/test_transactions.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_transactions.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2009, 2011 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,13 +17,9 @@
 
 """Tests for the behaviour of the Transaction concept in bzr."""
 
-# import system imports here
-import os
-import sys
-
 #import bzrlib specific imports here
 import bzrlib.errors as errors
-from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib.tests import TestCase
 import bzrlib.transactions as transactions
 
 

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2010-12-02 10:41:05 +0000
+++ bzrlib/tests/test_transform.py	2011-01-13 01:29:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import errno
 import os
 from StringIO import StringIO
 import sys
@@ -52,7 +53,6 @@
     ImmortalPendingDeletion,
     LockError,
     MalformedTransform,
-    NoSuchFile,
     ReusingTransform,
 )
 from bzrlib.osutils import (
@@ -64,7 +64,6 @@
     features,
     HardlinkFeature,
     SymlinkFeature,
-    TestCase,
     TestCaseInTempDir,
     TestSkipped,
 )
@@ -74,7 +73,6 @@
     cook_conflicts,
     _FileMover,
     FinalPaths,
-    get_backup_name,
     resolve_conflicts,
     resolve_checkout,
     ROOT_PARENT,
@@ -900,9 +898,14 @@
         # On windows looks like:
         # "Failed to rename .../work/myfile to 
         # .../work/.bzr/checkout/limbo/new-1: [Errno 13] Permission denied"
-        # The strerror will vary per OS and language so it's not checked here
-        self.assertContainsRe(str(e),
-            "Failed to rename .*(first-dir.newname:|myfile)")
+        # This test isn't concerned with exactly what the error looks like,
+        # and the strerror will vary across OS and locales, but the assert
+        # that the exeception attributes are what we expect
+        self.assertEqual(e.errno, errno.EACCES)
+        if os.name == "posix":
+            self.assertEndsWith(e.to_path, "/first-dir/newname")
+        else:
+            self.assertEqual(os.path.basename(e.from_path), "myfile")
 
     def test_set_executability_order(self):
         """Ensure that executability behaves the same, no matter what order.

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2010-10-08 07:17:16 +0000
+++ bzrlib/tests/test_transport.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 
 from cStringIO import StringIO
-import os
 import subprocess
 import sys
 import threading
@@ -967,7 +966,7 @@
             bzr_remote_path = sys.executable + ' ' + self.get_bzr_path()
         else:
             bzr_remote_path = self.get_bzr_path()
-        os.environ['BZR_REMOTE_PATH'] = bzr_remote_path
+        self.overrideEnv('BZR_REMOTE_PATH', bzr_remote_path)
 
         # Access the branch via a bzr+ssh URL.  The BZR_REMOTE_PATH environment
         # variable is used to tell bzr what command to run on the remote end.

=== modified file 'bzrlib/tests/test_transport_log.py'
--- bzrlib/tests/test_transport_log.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_transport_log.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
 """Tests for log+ transport decorator."""
 
 
-import types
-
 from bzrlib import transport
 from bzrlib.tests import TestCaseWithMemoryTransport
 from bzrlib.trace import mutter

=== modified file 'bzrlib/tests/test_tree.py'
--- bzrlib/tests/test_tree.py	2009-08-28 05:00:33 +0000
+++ bzrlib/tests/test_tree.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2008 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 from bzrlib import (
     errors,
     revision,
-    tests,
     tree as _mod_tree,
     )
 from bzrlib.tests import TestCaseWithTransport

=== modified file 'bzrlib/tests/test_tuned_gzip.py'
--- bzrlib/tests/test_tuned_gzip.py	2010-09-25 20:08:01 +0000
+++ bzrlib/tests/test_tuned_gzip.py	2011-01-11 18:10:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,11 +21,14 @@
 # do not use bzrlib test cases here - this should be suitable for sending
 # upstream.
 from cStringIO import StringIO
-from unittest import TestCase
 import zlib
 
 
-from bzrlib import tuned_gzip
+from bzrlib import (
+    symbol_versioning,
+    tuned_gzip,
+    tests,
+    )
 
 
 class FakeDecompress(object):
@@ -42,7 +45,7 @@
         return ''
 
 
-class TestFakeDecompress(TestCase):
+class TestFakeDecompress(tests.TestCase):
     """We use a fake decompressor to test GzipFile.
 
     This class tests the behaviours we want from it.
@@ -67,14 +70,16 @@
         self.assertEqual('1234567', decompress.unused_data)
 
 
-class TestGzip(TestCase):
+class TestGzip(tests.TestCase):
 
     def test__read_short_remainder(self):
         # a _read call at the end of a compressed hunk should
         # read more bytes if there is less than 8 bytes (the
         # gzip trailer) unread.
         stream = StringIO('\0\0\0\0\0\0\0\0')
-        myfile = tuned_gzip.GzipFile(fileobj=stream)
+        myfile = self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            tuned_gzip.GzipFile, fileobj=stream)
         # disable the _new_member check, we are microtesting.
         myfile._new_member = False
         myfile.crc = zlib.crc32('')
@@ -89,21 +94,25 @@
     def test_negative_crc(self):
         """Content with a negative crc should not break when written"""
         sio = StringIO()
-        gfile = tuned_gzip.GzipFile(mode="w", fileobj=sio)
+        gfile = self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            tuned_gzip.GzipFile, mode="w", fileobj=sio)
         gfile.write("\xFF")
         gfile.close()
         self.assertEqual(gfile.crc & 0xFFFFFFFFL, 0xFF000000L)
         self.assertEqual(sio.getvalue()[-8:-4], "\x00\x00\x00\xFF")
 
 
-class TestToGzip(TestCase):
+class TestToGzip(tests.TestCase):
 
     def assertToGzip(self, chunks):
         bytes = ''.join(chunks)
         gzfromchunks = tuned_gzip.chunks_to_gzip(chunks)
         gzfrombytes = tuned_gzip.bytes_to_gzip(bytes)
         self.assertEqual(gzfrombytes, gzfromchunks)
-        decoded = tuned_gzip.GzipFile(fileobj=StringIO(gzfromchunks)).read()
+        decoded = self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            tuned_gzip.GzipFile, fileobj=StringIO(gzfromchunks)).read()
         self.assertEqual(bytes, decoded)
 
     def test_single_chunk(self):

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2010-12-02 10:41:05 +0000
+++ bzrlib/tests/test_ui.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 """Tests for the bzrlib ui
 """
 
-import os
 import time
 
 from StringIO import StringIO
@@ -26,15 +25,10 @@
 
 from bzrlib import (
     config,
-    errors,
     remote,
-    repository,
     tests,
     ui as _mod_ui,
     )
-from bzrlib.symbol_versioning import (
-    deprecated_in,
-    )
 from bzrlib.tests import (
     fixtures,
     test_progress,
@@ -158,7 +152,7 @@
     def test_text_factory_prompts_and_clears(self):
         # a get_boolean call should clear the pb before prompting
         out = test_progress._TTYStringIO()
-        os.environ['TERM'] = 'xterm'
+        self.overrideEnv('TERM', 'xterm')
         factory = _mod_ui_text.TextUIFactory(
             stdin=tests.StringIOWrapper("yada\ny\n"),
             stdout=out, stderr=out)
@@ -226,7 +220,7 @@
             pb.finished()
 
     def test_quietness(self):
-        os.environ['BZR_PROGRESS_BAR'] = 'text'
+        self.overrideEnv('BZR_PROGRESS_BAR', 'text')
         ui_factory = _mod_ui_text.TextUIFactory(None,
             test_progress._TTYStringIO(),
             test_progress._TTYStringIO())
@@ -313,12 +307,8 @@
             # however, it can still be forced on
             (FileStringIO, 'dumb', 'text', _mod_ui_text.TextProgressView),
             ):
-            os.environ['TERM'] = term
-            if pb is None:
-                if 'BZR_PROGRESS_BAR' in os.environ:
-                    del os.environ['BZR_PROGRESS_BAR']
-            else:
-                os.environ['BZR_PROGRESS_BAR'] = pb
+            self.overrideEnv('TERM', term)
+            self.overrideEnv('BZR_PROGRESS_BAR', pb)
             stdin = file_class('')
             stderr = file_class()
             stdout = file_class()
@@ -335,10 +325,7 @@
         stderr = test_progress._NonTTYStringIO()
         stdout = test_progress._NonTTYStringIO()
         for term_type in ['dumb', None, 'xterm']:
-            if term_type is None:
-                del os.environ['TERM']
-            else:
-                os.environ['TERM'] = term_type
+            self.overrideEnv('TERM', term_type)
             uif = _mod_ui.make_ui_for_terminal(stdin, stdout, stderr)
             self.assertIsInstance(uif, _mod_ui_text.TextUIFactory,
                 'TERM=%r' % (term_type,))

=== modified file 'bzrlib/tests/test_upgrade.py'
--- bzrlib/tests/test_upgrade.py	2010-12-07 09:06:39 +0000
+++ bzrlib/tests/test_upgrade.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,32 +22,24 @@
 # TODO queue for upgrade:
 # test the error message when upgrading an unknown BzrDir format.
 
-import base64
-import os
-import sys
-
 from bzrlib import (
-    branch as _mod_branch,
+    branch,
     bzrdir,
-    progress,
     repository,
-    transport,
+    tests,
+    upgrade,
     workingtree,
     workingtree_4,
     )
-import bzrlib.branch
-from bzrlib.branch import Branch
-from bzrlib.tests import TestCaseWithTransport
-from bzrlib.upgrade import upgrade
-
-
-class TestUpgrade(TestCaseWithTransport):
+
+
+class TestUpgrade(tests.TestCaseWithTransport):
 
     def test_upgrade_simple(self):
         """Upgrade simple v0.0.4 format to latest format"""
         eq = self.assertEquals
         self.build_tree_contents(_upgrade1_template)
-        upgrade(u'.')
+        upgrade.upgrade(u'.')
         control = bzrdir.BzrDir.open('.')
         b = control.open_branch()
         # tsk, peeking under the covers.
@@ -74,7 +66,7 @@
             rt.unlock()
         # check a backup was made:
         backup_dir = 'backup.bzr.~1~'
-        t = transport.get_transport(b.base)
+        t = self.get_transport('.')
         t.stat(backup_dir)
         t.stat(backup_dir + '/README')
         t.stat(backup_dir + '/branch-format')
@@ -108,8 +100,8 @@
         its contents."""
         eq = self.assertEquals
         self.build_tree_contents(_ghost_template)
-        upgrade(u'.')
-        b = Branch.open(u'.')
+        upgrade.upgrade(u'.')
+        b = branch.Branch.open(u'.')
         revision_id = b.revision_history()[1]
         rev = b.repository.get_revision(revision_id)
         eq(len(rev.parent_ids), 2)
@@ -117,13 +109,13 @@
 
     def test_upgrade_makes_dir_weaves(self):
         self.build_tree_contents(_upgrade_dir_template)
-        old_repodir = bzrlib.bzrdir.BzrDir.open_unsupported('.')
+        old_repodir = bzrdir.BzrDir.open_unsupported('.')
         old_repo_format = old_repodir.open_repository()._format
-        upgrade('.')
+        upgrade.upgrade('.')
         # this is the path to the literal file. As format changes
         # occur it needs to be updated. FIXME: ask the store for the
         # path.
-        repo = bzrlib.repository.Repository.open('.')
+        repo = repository.Repository.open('.')
         # it should have changed the format
         self.assertNotEqual(old_repo_format.__class__, repo._format.__class__)
         # and we should be able to read the names for the file id
@@ -137,7 +129,7 @@
 
     def test_upgrade_to_meta_sets_workingtree_last_revision(self):
         self.build_tree_contents(_upgrade_dir_template)
-        upgrade('.', bzrdir.BzrDirMetaFormat1())
+        upgrade.upgrade('.', bzrdir.BzrDirMetaFormat1())
         tree = workingtree.WorkingTree.open('.')
         self.assertEqual([tree.branch.revision_history()[-1]],
             tree.get_parent_ids())
@@ -146,15 +138,15 @@
         # Some format6 branches do not have checkout files. Upgrading
         # such a branch to metadir must not setup a working tree.
         self.build_tree_contents(_upgrade1_template)
-        upgrade('.', bzrdir.BzrDirFormat6())
-        t = transport.get_transport('.')
+        upgrade.upgrade('.', bzrdir.BzrDirFormat6())
+        t = self.get_transport('.')
         t.delete_multi(['.bzr/pending-merges', '.bzr/inventory'])
         self.assertFalse(t.has('.bzr/stat-cache'))
         # XXX: upgrade fails if a backup.bzr is already present
         # -- David Allouche 2006-08-11
         t.delete_tree('backup.bzr.~1~')
         # At this point, we have a format6 branch without checkout files.
-        upgrade('.', bzrdir.BzrDirMetaFormat1())
+        upgrade.upgrade('.', bzrdir.BzrDirMetaFormat1())
         # The upgrade should not have set up a working tree.
         control = bzrdir.BzrDir.open('.')
         self.assertFalse(control.has_workingtree())
@@ -162,27 +154,27 @@
         # upgrade has not eaten our data, even if it's a bit redundant with
         # other tests.
         self.failUnless(isinstance(control._format, bzrdir.BzrDirMetaFormat1))
-        branch = control.open_branch()
-        self.assertEquals(branch.revision_history(),
+        b = control.open_branch()
+        self.assertEquals(b.revision_history(),
            ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
             'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
 
     def test_upgrade_rich_root(self):
         tree = self.make_branch_and_tree('tree', format='rich-root')
         rev_id = tree.commit('first post')
-        upgrade('tree')
+        upgrade.upgrade('tree')
 
     def test_convert_branch5_branch6(self):
-        branch = self.make_branch('branch', format='knit')
-        branch.set_revision_history(['AB', 'CD'])
-        branch.set_parent('file:///EF')
-        branch.set_bound_location('file:///GH')
-        branch.set_push_location('file:///IJ')
+        b = self.make_branch('branch', format='knit')
+        b.set_revision_history(['AB', 'CD'])
+        b.set_parent('file:///EF')
+        b.set_bound_location('file:///GH')
+        b.set_push_location('file:///IJ')
         target = bzrdir.format_registry.make_bzrdir('dirstate-with-subtree')
-        converter = branch.bzrdir._format.get_converter(target)
-        converter.convert(branch.bzrdir, None)
-        new_branch = _mod_branch.Branch.open(self.get_url('branch'))
-        self.assertIs(new_branch.__class__, _mod_branch.BzrBranch6)
+        converter = b.bzrdir._format.get_converter(target)
+        converter.convert(b.bzrdir, None)
+        new_branch = branch.Branch.open(self.get_url('branch'))
+        self.assertIs(new_branch.__class__, branch.BzrBranch6)
         self.assertEqual('CD', new_branch.last_revision())
         self.assertEqual('file:///EF', new_branch.get_parent())
         self.assertEqual('file:///GH', new_branch.get_bound_location())
@@ -190,21 +182,21 @@
         self.assertEqual('file:///IJ',
             branch_config.get_user_option('push_location'))
 
-        branch2 = self.make_branch('branch2', format='knit')
-        converter = branch2.bzrdir._format.get_converter(target)
-        converter.convert(branch2.bzrdir, None)
-        branch2 = _mod_branch.Branch.open(self.get_url('branch'))
-        self.assertIs(branch2.__class__, _mod_branch.BzrBranch6)
+        b2 = self.make_branch('branch2', format='knit')
+        converter = b2.bzrdir._format.get_converter(target)
+        converter.convert(b2.bzrdir, None)
+        b2 = branch.Branch.open(self.get_url('branch'))
+        self.assertIs(b2.__class__, branch.BzrBranch6)
 
     def test_convert_branch7_branch8(self):
-        branch = self.make_branch('branch', format='1.9')
+        b = self.make_branch('branch', format='1.9')
         target = bzrdir.format_registry.make_bzrdir('1.9')
-        target.set_branch_format(_mod_branch.BzrBranchFormat8())
-        converter = branch.bzrdir._format.get_converter(target)
-        converter.convert(branch.bzrdir, None)
-        branch = _mod_branch.Branch.open(self.get_url('branch'))
-        self.assertIs(branch.__class__, _mod_branch.BzrBranch8)
-        self.assertEqual({}, branch._get_all_reference_info())
+        target.set_branch_format(branch.BzrBranchFormat8())
+        converter = b.bzrdir._format.get_converter(target)
+        converter.convert(b.bzrdir, None)
+        b = branch.Branch.open(self.get_url('branch'))
+        self.assertIs(b.__class__, branch.BzrBranch8)
+        self.assertEqual({}, b._get_all_reference_info())
 
     def test_convert_knit_dirstate_empty(self):
         # test that asking for an upgrade from knit to dirstate works.
@@ -423,3 +415,88 @@
     ),
     ( './dir/', ),
 ]
+
+
+class TestSmartUpgrade(tests.TestCaseWithTransport):
+
+    from_format = bzrdir.format_registry.make_bzrdir("pack-0.92")
+    to_format = bzrdir.format_registry.make_bzrdir("2a")
+
+    def make_standalone_branch(self):
+        wt = self.make_branch_and_tree("branch1", format=self.from_format)
+        return wt.bzrdir
+
+    def test_upgrade_standalone_branch(self):
+        control = self.make_standalone_branch()
+        tried, worked, issues = upgrade.smart_upgrade(
+            [control], format=self.to_format)
+        self.assertLength(1, tried)
+        self.assertEqual(tried[0], control)
+        self.assertLength(1, worked)
+        self.assertEqual(worked[0], control)
+        self.assertLength(0, issues)
+        self.failUnlessExists('branch1/backup.bzr.~1~')
+        self.assertEqual(control.open_repository()._format,
+                         self.to_format._repository_format)
+
+    def test_upgrade_standalone_branch_cleanup(self):
+        control = self.make_standalone_branch()
+        tried, worked, issues = upgrade.smart_upgrade(
+            [control], format=self.to_format, clean_up=True)
+        self.assertLength(1, tried)
+        self.assertEqual(tried[0], control)
+        self.assertLength(1, worked)
+        self.assertEqual(worked[0], control)
+        self.assertLength(0, issues)
+        self.failUnlessExists('branch1')
+        self.failUnlessExists('branch1/.bzr')
+        self.failIfExists('branch1/backup.bzr.~1~')
+        self.assertEqual(control.open_repository()._format,
+                         self.to_format._repository_format)
+
+    def make_repo_with_branches(self):
+        repo = self.make_repository('repo', shared=True,
+            format=self.from_format)
+        # Note: self.make_branch() always creates a new repo at the location
+        # so we need to avoid using that here ...
+        b1 = bzrdir.BzrDir.create_branch_convenience("repo/branch1",
+            format=self.from_format)
+        b2 = bzrdir.BzrDir.create_branch_convenience("repo/branch2",
+            format=self.from_format)
+        return repo.bzrdir
+
+    def test_upgrade_repo_with_branches(self):
+        control = self.make_repo_with_branches()
+        tried, worked, issues = upgrade.smart_upgrade(
+            [control], format=self.to_format)
+        self.assertLength(3, tried)
+        self.assertEqual(tried[0], control)
+        self.assertLength(3, worked)
+        self.assertEqual(worked[0], control)
+        self.assertLength(0, issues)
+        self.failUnlessExists('repo/backup.bzr.~1~')
+        self.failUnlessExists('repo/branch1/backup.bzr.~1~')
+        self.failUnlessExists('repo/branch2/backup.bzr.~1~')
+        self.assertEqual(control.open_repository()._format,
+                         self.to_format._repository_format)
+        b1 = branch.Branch.open('repo/branch1')
+        self.assertEqual(b1._format, self.to_format._branch_format)
+
+    def test_upgrade_repo_with_branches_cleanup(self):
+        control = self.make_repo_with_branches()
+        tried, worked, issues = upgrade.smart_upgrade(
+            [control], format=self.to_format, clean_up=True)
+        self.assertLength(3, tried)
+        self.assertEqual(tried[0], control)
+        self.assertLength(3, worked)
+        self.assertEqual(worked[0], control)
+        self.assertLength(0, issues)
+        self.failUnlessExists('repo')
+        self.failUnlessExists('repo/.bzr')
+        self.failIfExists('repo/backup.bzr.~1~')
+        self.failIfExists('repo/branch1/backup.bzr.~1~')
+        self.failIfExists('repo/branch2/backup.bzr.~1~')
+        self.assertEqual(control.open_repository()._format,
+                         self.to_format._repository_format)
+        b1 = branch.Branch.open('repo/branch1')
+        self.assertEqual(b1._format, self.to_format._branch_format)

=== modified file 'bzrlib/tests/test_upgrade_stacked.py'
--- bzrlib/tests/test_upgrade_stacked.py	2010-12-09 04:26:53 +0000
+++ bzrlib/tests/test_upgrade_stacked.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_version.py'
--- bzrlib/tests/test_version.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_version.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 """Tests for versioning of bzrlib."""
 
 from cStringIO import StringIO
-import os
 import re
 
 from bzrlib import version, workingtree

=== modified file 'bzrlib/tests/test_version_info.py'
--- bzrlib/tests/test_version_info.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/test_version_info.py	2011-01-12 18:39:25 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 from bzrlib import (
     errors,
     registry,
-    symbol_versioning,
     tests,
     version_info_formats,
     )
@@ -193,6 +192,7 @@
         val = sio.getvalue()
         self.assertContainsRe(val, "'revision_id': None")
         self.assertContainsRe(val, "'revno': 0")
+        self.assertNotContainsString(val, '\n\n\n\n')
 
     def test_python_version(self):
         wt = self.create_branch()

=== modified file 'bzrlib/tests/test_weave.py'
--- bzrlib/tests/test_weave.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_weave.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
     )
 from bzrlib.osutils import sha_string
 from bzrlib.tests import TestCase, TestCaseInTempDir
-from bzrlib.weave import Weave, WeaveFormatError, WeaveError
+from bzrlib.weave import Weave, WeaveFormatError
 from bzrlib.weavefile import write_weave, read_weave
 
 

=== modified file 'bzrlib/tests/test_whitebox.py'
--- bzrlib/tests/test_whitebox.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_whitebox.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,13 +15,11 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import os
-import unittest
 
 from bzrlib import (
     osutils,
     )
-from bzrlib.tests import TestCaseWithTransport, TestCase
-from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.errors import PathNotChild
 from bzrlib.osutils import relpath, pathjoin, abspath, realpath
 

=== modified file 'bzrlib/tests/test_win32utils.py'
--- bzrlib/tests/test_win32utils.py	2010-12-07 17:28:31 +0000
+++ bzrlib/tests/test_win32utils.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 """Tests for win32utils."""
 
 import os
-import sys
 
 from bzrlib import (
     osutils,
@@ -217,7 +216,7 @@
     def test_appdata_not_using_environment(self):
         # Test that we aren't falling back to the environment
         first = win32utils.get_appdata_location()
-        self._captureVar("APPDATA", None)
+        self.overrideEnv("APPDATA", None)
         self.assertPathsEqual(first, win32utils.get_appdata_location())
 
     def test_appdata_matches_environment(self):
@@ -234,7 +233,7 @@
     def test_local_appdata_not_using_environment(self):
         # Test that we aren't falling back to the environment
         first = win32utils.get_local_appdata_location()
-        self._captureVar("LOCALAPPDATA", None)
+        self.overrideEnv("LOCALAPPDATA", None)
         self.assertPathsEqual(first, win32utils.get_local_appdata_location())
 
     def test_local_appdata_matches_environment(self):

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2010-11-07 10:20:30 +0000
+++ bzrlib/tests/test_workingtree.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 # Authors:  Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 import os
 
 from bzrlib import (
@@ -25,8 +24,6 @@
     transport,
     workingtree,
     )
-from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
 from bzrlib.lockdir import LockDir
 from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped

=== modified file 'bzrlib/tests/test_wsgi.py'
--- bzrlib/tests/test_wsgi.py	2009-10-21 11:13:40 +0000
+++ bzrlib/tests/test_wsgi.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 from cStringIO import StringIO
 
 from bzrlib import tests
-from bzrlib.smart import medium, message, protocol
+from bzrlib.smart import medium, protocol
 from bzrlib.transport.http import wsgi
 from bzrlib.transport import chroot, memory
 

=== modified file 'bzrlib/tests/test_xml.py'
--- bzrlib/tests/test_xml.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_xml.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,10 +23,9 @@
     xml6,
     xml7,
     xml8,
-    serializer,
     )
 from bzrlib.tests import TestCase
-from bzrlib.inventory import Inventory, InventoryEntry
+from bzrlib.inventory import Inventory
 from bzrlib.xml4 import serializer_v4
 import bzrlib.xml5
 

=== modified file 'bzrlib/timestamp.py'
--- bzrlib/timestamp.py	2009-03-23 14:59:43 +0000
+++ bzrlib/timestamp.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2007, 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -72,34 +72,6 @@
     :param date: A date formated by format_highres_date
     :type date: string
 
-    >>> import time, random
-    >>> unpack_highres_date('Thu 2005-06-30 12:38:52.350850105 -0500')
-    (1120153132.3508501, -18000)
-    >>> unpack_highres_date('Thu 2005-06-30 17:38:52.350850105 +0000')
-    (1120153132.3508501, 0)
-    >>> unpack_highres_date('Thu 2005-06-30 19:38:52.350850105 +0200')
-    (1120153132.3508501, 7200)
-    >>> unpack_highres_date('Sun 2006-07-09 12:35:38.867522001 +0530')
-    (1152428738.867522, 19800)
-    >>> from bzrlib.osutils import local_time_offset
-    >>> t = time.time()
-    >>> o = local_time_offset()
-    >>> t2, o2 = unpack_highres_date(format_highres_date(t, o))
-    >>> t == t2
-    True
-    >>> o == o2
-    True
-    >>> t -= 24*3600*365*2 # Start 2 years ago
-    >>> o = -12*3600
-    >>> for count in xrange(500):
-    ...   t += random.random()*24*3600*30
-    ...   o = ((o/3600 + 13) % 25 - 12)*3600 # Add 1 wrap around from [-12, 12]
-    ...   date = format_highres_date(t, o)
-    ...   t2, o2 = unpack_highres_date(date)
-    ...   if t != t2 or o != o2:
-    ...      print 'Failed on date %r, %s,%s diff:%s' % (date, t, o, t2-t)
-    ...      break
-
     """
     # Weekday parsing is locale sensitive, so drop the weekday
     space_loc = date.find(' ')

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2010-10-05 23:30:18 +0000
+++ bzrlib/trace.py	2011-01-13 00:14:31 +0000
@@ -123,14 +123,12 @@
     # FIXME: clearing the ui and then going through the abstract logging
     # framework is whack; we should probably have a logging Handler that
     # deals with terminal output if needed.
-    import bzrlib.ui
-    bzrlib.ui.ui_factory.clear_term()
+    ui.ui_factory.clear_term()
     _bzr_logger.info(*args, **kwargs)
 
 
 def warning(*args, **kwargs):
-    import bzrlib.ui
-    bzrlib.ui.ui_factory.clear_term()
+    ui.ui_factory.clear_term()
     _bzr_logger.warning(*args, **kwargs)
 
 

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2010-11-24 16:01:57 +0000
+++ bzrlib/transform.py	2011-01-13 02:43:21 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,10 +19,12 @@
 from stat import S_ISREG, S_IEXEC
 import time
 
+import bzrlib
 from bzrlib import (
     errors,
     lazy_import,
     registry,
+    tree,
     )
 lazy_import.lazy_import(globals(), """
 from bzrlib import (
@@ -38,6 +40,7 @@
     revision as _mod_revision,
     trace,
     ui,
+    urlutils,
     )
 """)
 from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
@@ -50,22 +53,18 @@
     delete_any,
     file_kind,
     has_symlinks,
-    lexists,
     pathjoin,
     sha_file,
     splitpath,
     supports_executable,
-)
+    )
 from bzrlib.progress import ProgressPhase
 from bzrlib.symbol_versioning import (
-        deprecated_function,
-        deprecated_in,
-        deprecated_method,
-        )
-from bzrlib.trace import mutter, warning
-from bzrlib import tree
-import bzrlib.ui
-import bzrlib.urlutils as urlutils
+    deprecated_function,
+    deprecated_in,
+    deprecated_method,
+    )
+from bzrlib.trace import warning
 
 
 ROOT_PARENT = "root-parent"
@@ -528,6 +527,8 @@
         for trans_id in self._removed_id:
             file_id = self.tree_file_id(trans_id)
             if file_id is not None:
+                # XXX: This seems like something that should go via a different
+                #      indirection.
                 if self._tree.inventory[file_id].kind == 'directory':
                     parents.append(trans_id)
             elif self.tree_kind(trans_id) == 'directory':
@@ -1675,7 +1676,7 @@
         """
         if not no_conflicts:
             self._check_malformed()
-        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        child_pb = ui.ui_factory.nested_progress_bar()
         try:
             if precomputed_delta is None:
                 child_pb.update('Apply phase', 0, 2)
@@ -1708,7 +1709,7 @@
     def _generate_inventory_delta(self):
         """Generate an inventory delta for the current transform."""
         inventory_delta = []
-        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        child_pb = ui.ui_factory.nested_progress_bar()
         new_paths = self._inventory_altered()
         total_entries = len(new_paths) + len(self._removed_id)
         try:
@@ -1776,7 +1777,7 @@
         """
         tree_paths = list(self._tree_path_ids.iteritems())
         tree_paths.sort(reverse=True)
-        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        child_pb = ui.ui_factory.nested_progress_bar()
         try:
             for num, data in enumerate(tree_paths):
                 path, trans_id = data
@@ -1811,7 +1812,7 @@
         modified_paths = []
         new_path_file_ids = dict((t, self.final_file_id(t)) for p, t in
                                  new_paths)
-        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        child_pb = ui.ui_factory.nested_progress_bar()
         try:
             for num, (path, trans_id) in enumerate(new_paths):
                 if (num % 10) == 0:
@@ -2429,7 +2430,7 @@
         if num > 0:  # more than just a root
             raise errors.WorkingTreeAlreadyPopulated(base=wt.basedir)
     file_trans_id = {}
-    top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+    top_pb = ui.ui_factory.nested_progress_bar()
     pp = ProgressPhase("Build phase", 2, top_pb)
     if tree.inventory.root is not None:
         # This is kind of a hack: we should be altering the root
@@ -2448,7 +2449,7 @@
         pp.next_phase()
         file_trans_id[wt.get_root_id()] = \
             tt.trans_id_tree_file_id(wt.get_root_id())
-        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        pb = ui.ui_factory.nested_progress_bar()
         try:
             deferred_contents = []
             num = 0
@@ -2778,7 +2779,7 @@
 def _prepare_revert_transform(working_tree, target_tree, tt, filenames,
                               backups, pp, basis_tree=None,
                               merge_modified=None):
-    child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+    child_pb = ui.ui_factory.nested_progress_bar()
     try:
         if merge_modified is None:
             merge_modified = working_tree.merge_modified()
@@ -2787,7 +2788,7 @@
                                       merge_modified, basis_tree)
     finally:
         child_pb.finished()
-    child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+    child_pb = ui.ui_factory.nested_progress_bar()
     try:
         raw_conflicts = resolve_conflicts(tt, child_pb,
             lambda t, c: conflict_pass(t, c, target_tree))
@@ -3020,7 +3021,8 @@
             file_id = tt.inactive_file_id(conflict[1])
             # special-case the other tree root (move its children instead)
             if path_tree and file_id in path_tree:
-                if path_tree.inventory[file_id].parent_id is None:
+                if path_tree.path2id('') == file_id:
+                    # This is the root entry, skip it
                     continue
             tt.version_file(file_id, conflict[1])
             new_conflicts.add((c_type, 'Versioned directory', conflict[1]))

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2010-10-18 10:01:47 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2011-01-12 21:22:20 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -110,15 +110,19 @@
         self.report_activity(len(s), 'read')
         return s
 
-    def readline(self):
-        # This should be readline(self, size=-1), but httplib in python 2.4 and
-        #  2.5 defines a SSLFile wrapper whose readline method lacks the size
-        #  parameter.  So until we drop support for 2.4 and 2.5 and since we
-        #  don't *need* the size parameter we'll stay with readline(self)
-        #  --  vila 20090209
-        s = self.filesock.readline()
-        self.report_activity(len(s), 'read')
-        return s
+    # httplib in python 2.4 and 2.5 defines a SSLFile wrapper whose readline
+    # method lacks the size parameter. python2.6 provides a proper ssl socket
+    # and added it. python2.7 uses it, forcing us to provide it.
+    if sys.version_info < (2, 6):
+        def readline(self):
+            s = self.filesock.readline()
+            self.report_activity(len(s), 'read')
+            return s
+    else:
+        def readline(self, size=-1):
+            s = self.filesock.readline(size)
+            self.report_activity(len(s), 'read')
+            return s
 
     def __getattr__(self, name):
         return getattr(self.filesock, name)
@@ -594,7 +598,9 @@
                         'Bad status line received',
                         orig_error=exc_val)
                 elif (isinstance(exc_val, socket.error) and len(exc_val.args)
-                      and exc_val.args[0] in (errno.ECONNRESET, 10054)):
+                      and exc_val.args[0] in (errno.ECONNRESET, 10053, 10054)):
+                      # 10053 == WSAECONNABORTED
+                      # 10054 == WSAECONNRESET
                     raise errors.ConnectionReset(
                         "Connection lost while sending request.")
                 else:

=== modified file 'bzrlib/transport/ssh.py'
--- bzrlib/transport/ssh.py	2010-11-18 00:22:24 +0000
+++ bzrlib/transport/ssh.py	2011-01-11 20:20:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Robey Pointer <robey@lag.net>
+# Copyright (C) 2006-2011 Robey Pointer <robey@lag.net>
 # Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
@@ -359,6 +359,7 @@
         # whatever) chunks.
         try:
             my_sock, subproc_sock = socket.socketpair()
+            osutils.set_fd_cloexec(my_sock)
         except (AttributeError, socket.error):
             # This platform doesn't support socketpair(), so just use ordinary
             # pipes instead.

=== modified file 'bzrlib/tuned_gzip.py'
--- bzrlib/tuned_gzip.py	2010-09-25 20:08:01 +0000
+++ bzrlib/tuned_gzip.py	2011-01-11 18:10:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 # Written by Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
 import zlib
 
 # we want a \n preserved, break on \n only splitlines.
-import bzrlib
+from bzrlib import symbol_versioning
 
 __all__ = ["GzipFile", "bytes_to_gzip"]
 
@@ -118,6 +118,13 @@
     Yes, its only 1.6 seconds, but they add up.
     """
 
+    def __init__(self, *args, **kwargs):
+        symbol_versioning.warn(
+            symbol_versioning.deprecated_in((2, 3, 0))
+            % 'bzrlib.tuned_gzip.GzipFile',
+            DeprecationWarning, stacklevel=2)
+        gzip.GzipFile.__init__(self, *args, **kwargs)
+
     def _add_read_data(self, data):
         # 4169 calls in 183
         # temp var for len(data) and switch to +='s.

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2010-04-21 11:27:04 +0000
+++ bzrlib/upgrade.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,20 +17,42 @@
 """bzr upgrade logic."""
 
 
-from bzrlib.bzrdir import BzrDir, format_registry
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    trace,
+    ui,
+    )
+from bzrlib.bzrdir import (
+    BzrDir,
+    format_registry,
+    )
 from bzrlib.remote import RemoteBzrDir
-import bzrlib.ui as ui
 
 
 class Convert(object):
 
-    def __init__(self, url, format=None):
+    def __init__(self, url=None, format=None, control_dir=None):
+        """Convert a Bazaar control directory to a given format.
+
+        Either the url or control_dir parameter must be given.
+
+        :param url: the URL of the control directory or None if the
+          control_dir is explicitly given instead
+        :param format: the format to convert to or None for the default
+        :param control_dir: the control directory or None if it is
+          specified via the URL parameter instead
+        """
         self.format = format
-        self.bzrdir = BzrDir.open_unsupported(url)
         # XXX: Change to cleanup
         warning_id = 'cross_format_fetch'
         saved_warning = warning_id in ui.ui_factory.suppressed_warnings
+        if url is None and control_dir is None:
+            raise AssertionError(
+                "either the url or control_dir parameter must be set.")
+        if control_dir is not None:
+            self.bzrdir = control_dir
+        else:
+            self.bzrdir = BzrDir.open_unsupported(url)
         if isinstance(self.bzrdir, RemoteBzrDir):
             self.bzrdir._ensure_real()
             self.bzrdir = self.bzrdir._real_bzrdir
@@ -48,9 +70,9 @@
         try:
             branch = self.bzrdir.open_branch()
             if branch.user_url != self.bzrdir.user_url:
-                ui.ui_factory.note("This is a checkout. The branch (%s) needs to be "
-                             "upgraded separately." %
-                             branch.user_url)
+                ui.ui_factory.note(
+                    'This is a checkout. The branch (%s) needs to be upgraded'
+                    ' separately.' % (branch.user_url,))
             del branch
         except (errors.NotBranchError, errors.IncompatibleRepositories):
             # might not be a format we can open without upgrading; see e.g.
@@ -76,13 +98,203 @@
         self.bzrdir.check_conversion_target(format)
         ui.ui_factory.note('starting upgrade of %s' % self.transport.base)
 
-        self.bzrdir.backup_bzrdir()
+        self.backup_oldpath, self.backup_newpath = self.bzrdir.backup_bzrdir()
         while self.bzrdir.needs_format_conversion(format):
             converter = self.bzrdir._format.get_converter(format)
             self.bzrdir = converter.convert(self.bzrdir, None)
-        ui.ui_factory.note("finished")
-
-
-def upgrade(url, format=None):
-    """Upgrade to format, or the default bzrdir format if not supplied."""
-    Convert(url, format)
+        ui.ui_factory.note('finished')
+
+    def clean_up(self):
+        """Clean-up after a conversion.
+
+        This removes the backup.bzr directory.
+        """
+        transport = self.transport
+        backup_relpath = transport.relpath(self.backup_newpath)
+        child_pb = ui.ui_factory.nested_progress_bar()
+        child_pb.update('Deleting backup.bzr')
+        try:
+            transport.delete_tree(backup_relpath)
+        finally:
+            child_pb.finished()
+
+
+def upgrade(url, format=None, clean_up=False, dry_run=False):
+    """Upgrade locations to format.
+ 
+    This routine wraps the smart_upgrade() routine with a nicer UI.
+    In particular, it ensures all URLs can be opened before starting
+    and reports a summary at the end if more than one upgrade was attempted.
+    This routine is useful for command line tools. Other bzrlib clients
+    probably ought to use smart_upgrade() instead.
+
+    :param url: a URL of the locations to upgrade.
+    :param format: the format to convert to or None for the best default
+    :param clean-up: if True, the backup.bzr directory is removed if the
+      upgrade succeeded for a given repo/branch/tree
+    :param dry_run: show what would happen but don't actually do any upgrades
+    :return: the list of exceptions encountered
+    """
+    control_dirs = [BzrDir.open_unsupported(url)]
+    attempted, succeeded, exceptions = smart_upgrade(control_dirs,
+        format, clean_up=clean_up, dry_run=dry_run)
+    if len(attempted) > 1:
+        attempted_count = len(attempted)
+        succeeded_count = len(succeeded)
+        failed_count = attempted_count - succeeded_count
+        ui.ui_factory.note(
+            '\nSUMMARY: %d upgrades attempted, %d succeeded, %d failed'
+            % (attempted_count, succeeded_count, failed_count))
+    return exceptions
+
+
+def smart_upgrade(control_dirs, format, clean_up=False,
+    dry_run=False):
+    """Convert control directories to a new format intelligently.
+
+    If the control directory is a shared repository, dependent branches
+    are also converted provided the repository converted successfully.
+    If the conversion of a branch fails, remaining branches are still tried.
+
+    :param control_dirs: the BzrDirs to upgrade
+    :param format: the format to convert to or None for the best default
+    :param clean_up: if True, the backup.bzr directory is removed if the
+      upgrade succeeded for a given repo/branch/tree
+    :param dry_run: show what would happen but don't actually do any upgrades
+    :return: attempted-control-dirs, succeeded-control-dirs, exceptions
+    """
+    all_attempted = []
+    all_succeeded = []
+    all_exceptions = []
+    for control_dir in control_dirs:
+        attempted, succeeded, exceptions = _smart_upgrade_one(control_dir,
+            format, clean_up=clean_up, dry_run=dry_run)
+        all_attempted.extend(attempted)
+        all_succeeded.extend(succeeded)
+        all_exceptions.extend(exceptions)
+    return all_attempted, all_succeeded, all_exceptions
+
+
+def _smart_upgrade_one(control_dir, format, clean_up=False,
+    dry_run=False):
+    """Convert a control directory to a new format intelligently.
+
+    See smart_upgrade for parameter details.
+    """
+    # If the URL is a shared repository, find the dependent branches
+    dependents = None
+    try:
+        repo = control_dir.open_repository()
+    except errors.NoRepositoryPresent:
+        # A branch or checkout using a shared repository higher up
+        pass
+    else:
+        # The URL is a repository. If it successfully upgrades,
+        # then upgrade the dependent branches as well.
+        if repo.is_shared():
+            dependents = repo.find_branches(using=True)
+
+    # Do the conversions
+    attempted = [control_dir]
+    succeeded, exceptions = _convert_items([control_dir], format, clean_up,
+                                           dry_run)
+    if succeeded and dependents:
+        ui.ui_factory.note('Found %d dependent branches - upgrading ...'
+                           % (len(dependents),))
+        # Convert dependent branches
+        branch_cdirs = [b.bzrdir for b in dependents]
+        successes, problems = _convert_items(branch_cdirs, format, clean_up,
+            dry_run, label="branch")
+        attempted.extend(branch_cdirs)
+        succeeded.extend(successes)
+        exceptions.extend(problems)
+
+    # Return the result
+    return attempted, succeeded, exceptions
+
+# FIXME: There are several problems below:
+# - RemoteRepository doesn't support _unsupported (really ?)
+# - raising AssertionError is rude and may not be necessary
+# - no tests
+# - the only caller uses only the label
+def _get_object_and_label(control_dir):
+    """Return the primary object and type label for a control directory.
+
+    :return: object, label where
+      object is a Branch, Repository or WorkingTree and
+      label is one of:
+        branch            - a branch
+        repository        - a repository
+        tree              - a lightweight checkout
+    """
+    try:
+        try:
+            br = control_dir.open_branch(unsupported=True,
+                                         ignore_fallbacks=True)
+        except NotImplementedError:
+            # RemoteRepository doesn't support the unsupported parameter
+            br = control_dir.open_branch(ignore_fallbacks=True)
+    except errors.NotBranchError:
+        pass
+    else:
+        return br, "branch"
+    try:
+        repo = control_dir.open_repository()
+    except errors.NoRepositoryPresent:
+        pass
+    else:
+        return repo, "repository"
+    try:
+        wt = control_dir.open_workingtree()
+    except (errors.NoWorkingTree, errors.NotLocalUrl):
+        pass
+    else:
+        return wt, "tree"
+    raise AssertionError("unknown type of control directory %s", control_dir)
+
+
+def _convert_items(items, format, clean_up, dry_run, label=None):
+    """Convert a sequence of control directories to the given format.
+ 
+    :param items: the control directories to upgrade
+    :param format: the format to convert to or None for the best default
+    :param clean-up: if True, the backup.bzr directory is removed if the
+      upgrade succeeded for a given repo/branch/tree
+    :param dry_run: show what would happen but don't actually do any upgrades
+    :param label: the label for these items or None to calculate one
+    :return: items successfully upgraded, exceptions
+    """
+    succeeded = []
+    exceptions = []
+    child_pb = ui.ui_factory.nested_progress_bar()
+    child_pb.update('Upgrading bzrdirs', 0, len(items))
+    for i, control_dir in enumerate(items):
+        # Do the conversion
+        location = control_dir.root_transport.base
+        bzr_object, bzr_label = _get_object_and_label(control_dir)
+        type_label = label or bzr_label
+        child_pb.update("Upgrading %s" % (type_label), i+1, len(items))
+        ui.ui_factory.note('Upgrading %s %s ...' % (type_label, location,))
+        try:
+            if not dry_run:
+                cv = Convert(control_dir=control_dir, format=format)
+        except Exception, ex:
+            trace.warning('conversion error: %s' % ex)
+            exceptions.append(ex)
+            continue
+
+        # Do any required post processing
+        succeeded.append(control_dir)
+        if clean_up:
+            try:
+                ui.ui_factory.note('Removing backup ...')
+                if not dry_run:
+                    cv.clean_up()
+            except Exception, ex:
+                trace.warning('failed to clean-up %s: %s' % (location, ex))
+                exceptions.append(ex)
+
+    child_pb.finished()
+
+    # Return the result
+    return succeeded, exceptions

=== modified file 'bzrlib/version_info_formats/format_python.py'
--- bzrlib/version_info_formats/format_python.py	2009-04-03 21:23:55 +0000
+++ bzrlib/version_info_formats/format_python.py	2011-01-12 21:28:39 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,7 +38,6 @@
 
 
 _py_version_footer = '''
-
 if __name__ == '__main__':
     print 'revision: %(revno)d' % version_info
     print 'nick: %(branch_nick)s' % version_info

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2010-11-04 17:48:47 +0000
+++ bzrlib/workingtree.py	2011-01-13 02:05:17 +0000
@@ -209,13 +209,7 @@
         else:
             self._branch = self.bzrdir.open_branch()
         self.basedir = realpath(basedir)
-        # if branch is at our basedir and is a format 6 or less
-        if isinstance(self._format, WorkingTreeFormat2):
-            # share control object
-            self._control_files = self.branch.control_files
-        else:
-            # assume all other formats have their own control files.
-            self._control_files = _control_files
+        self._control_files = _control_files
         self._transport = self._control_files._transport
         # update the whole cache up front and write to disk if anything changed;
         # in the future we might want to do this more selectively
@@ -2974,7 +2968,8 @@
                          inv,
                          _internal=True,
                          _format=self,
-                         _bzrdir=a_bzrdir)
+                         _bzrdir=a_bzrdir,
+                         _control_files=branch.control_files)
         basis_tree = branch.repository.revision_tree(revision_id)
         if basis_tree.inventory.root is not None:
             wt.set_root_id(basis_tree.get_root_id())
@@ -3005,7 +3000,8 @@
         wt = WorkingTree2(a_bzrdir.root_transport.local_abspath('.'),
                            _internal=True,
                            _format=self,
-                           _bzrdir=a_bzrdir)
+                           _bzrdir=a_bzrdir,
+                           _control_files=a_bzrdir.open_branch().control_files)
         return wt
 
 class WorkingTreeFormat3(WorkingTreeFormat):

=== modified file 'doc/developers/cycle.txt'
--- doc/developers/cycle.txt	2010-10-12 15:50:39 +0000
+++ doc/developers/cycle.txt	2011-01-03 23:48:17 +0000
@@ -274,10 +274,9 @@
 ---------
 
 At present we have three upstream-maintained PPAs containing Ubuntu packages
-of Bazaar: ``~bzr-nightly-ppa``, ``~bzr-beta-ppa`` (beta releases) and
-``~bzr`` (ie stable).  We will keep these PPAs, and reorient beta to contain
-the monthly beta releases, and the stable PPA to contain stable releases and
-bugfixes to those releases.
+of Bazaar: ``bzr/daily`` (snapshots), ``bzr-beta-ppa/ppa`` (beta releases) and
+``bzr/ppa`` (ie stable).  Beta contains the monthly beta releases, and the
+stable PPA contains stable releases and bugfixes to those releases.
 
 Some platforms with relatively less active packagers may choose to ship
 only the stable releases.  This is probably better than having them only

=== added file 'doc/developers/fetch.txt'
--- doc/developers/fetch.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/fetch.txt	2011-01-12 22:10:10 +0000
@@ -0,0 +1,86 @@
+=============
+Fetching data
+=============
+
+Overview of a fetch
+===================
+
+Inside bzr, a typical fetch happens like this:
+
+* a user runs a command like ``bzr branch`` or ``bzr pull``
+
+* ``Repository.fetch`` is called (by a higher-level method such as
+  ``ControlDir.sprout``, ``Branch.fetch``, etc).
+
+* An ``InterRepository`` object is created.  The exact implementation of
+  ``InterRepository`` chosen depends on the format/capabilities of the
+  source and target repos.
+
+* The source and target repositories are compared to determine which data
+  needs to be transferred.
+
+* The repository data is copied.  Often this is done by creating a
+  ``StreamSource`` and ``StreamSink`` from the source and target
+  repositories and feeding the stream from the source into the sink, but
+  some ``InterRepository`` implementations do differently.
+
+
+How objects to be transferred are determined
+============================================
+
+See ``InterRepository._walk_to_common_revisions``.  The basic idea is to
+do a breadth-first search in the source repository's revision graph
+(starting from the head or heads the caller asked for), and look in the
+target repository to see if those revisions are already present.
+Eventually this will find the common ancestors in both graphs, and thus
+the set of revisions to be copied has been identified.
+
+All inventories for the copied revisions need to be present (and all
+parent inventories at the stacking boundary too, to support stacking).
+
+All texts versions introduced by those inventories need to be transferred
+(but see also stacking constraints).
+
+Fetch specs
+===========
+
+The most ``fetch`` methods accept a ``fetch_spec`` parameter.  This is how
+the caller controls what is fetched: e.g. all revisions for a given head
+(that aren't already present in the target), the full ancestry for one or
+more heads, or even the full contents of the source repository.
+
+The ``fetch_spec`` parameter is an object that implements the interface
+defined by ``AbstractSearchResult`` in ``bzrlib.graph``.  It describes
+which keys should be fetched.  Current implementations are
+``SearchResult``, ``PendingAncestryResult``, ``EmptySearchResult``, and
+``EverythingResult``.  Some have options controlling if missing revisions
+cause errors or not, etc.
+
+There are also some “search” objects, which can be used to conveniently
+construct a search result for common cases: ``EverythingNotInOther`` and
+``NotInOtherForRevs``.  They provide an ``execute`` method that performs
+the search and returns a search result.
+
+Also, ``Graph._make_breadth_first_searcher`` returns an object with a
+``get_result`` method that returns a search result.
+
+
+Streams
+=======
+
+A **stream** is an iterable of (substream type, substream) pairs.
+The **substream type** is a ``str`` that will be one of ``texts``,
+``inventories``, ``inventory-deltas``, ``chk_bytes``, ``revisions`` or
+``signatures``.  A **substream** is a record stream.  The format of those
+records depends on the repository format being streamed, except for
+``inventory-deltas`` records which are format-independent.
+
+A stream source can be constructed with ``repo._get_source(to_format)``,
+and it provides a ``get_stream(search)`` method (among others).  A stream
+sink can be constructed with ``repo._get_sink()``, and provides an
+``insert_stream(stream, src_format, resume_tokens)`` method (among
+others).
+
+
+..
+   vim: ft=rst tw=74 ai

=== modified file 'doc/developers/index.txt'
--- doc/developers/index.txt	2010-10-13 04:13:48 +0000
+++ doc/developers/index.txt	2011-01-06 06:26:23 +0000
@@ -38,6 +38,7 @@
 
    transports
    ui
+   fetch
 
 Releasing and Packaging
 =======================

=== modified file 'doc/developers/ppa.txt'
--- doc/developers/ppa.txt	2010-10-12 15:50:39 +0000
+++ doc/developers/ppa.txt	2011-01-11 11:25:27 +0000
@@ -13,24 +13,32 @@
 
   __ https://help.launchpad.net/PPAQuickStart
 
-As of September 2010, there are four PPAs:
+As of January 2011, there are the following PPAs:
 
 <https://launchpad.net/~bzr/+archive/ppa>
     Final released versions and updates.
     Most users who want updates to bzr should add this.
 
 <https://launchpad.net/~bzr/+archive/proposed>
-    Proposed uploads to move into ~bzr, awaiting testing.
-
-<https://launchpad.net/~bzr-beta-ppa/+archive>
+    Proposed uploads to move into ~bzr/ppa, awaiting testing.
+
+<https://launchpad.net/~bzr/+archive/obsolete>
+    A preserved copy of the final version of packages from ~bzr/ppa for
+    obsolete Ubuntu series.
+
+<https://launchpad.net/~bzr-beta-ppa/+archive/ppa>
     Beta releases.
 
-<https://launchpad.net/~bzr-nightly-ppa/+archive>
+<https://launchpad.net/~bzr-beta-ppa/+archive/obsolete>
+    A preserved copy of the final version of packages from
+    ~bzr-beta-ppa/ppa for obsolete Ubuntu series.
+
+<https://launchpad.net/~bzr/+archive/daily>
     Automatic nightly builds from trunk.
 
-We build a distinct package for each distrorelease.  
+We build a distinct package for each distrorelease.
 If you upload a release-specific version, you should add a suffix to the
-package version, e.g. ``bzr.1.3-1~bazaar1~dapper1``.
+package version, e.g. ``1.3-1~bazaar1~dapper1``.
 
 Dapper uses the ``python-support`` framework and later distributions use
 ``python-central``.  This has little effect on everyday packaging but does
@@ -46,8 +54,8 @@
 <lp:~bzr/ubuntu/hardy/bzr/bzr-ppa>.  These branches are intended to be used
 with the ``bzr-builddeb`` plugin.
 
-**You should almost always upload to the beta ppa first** and then either 
-upload again or copy the packages into the release ppa.  That reduces the 
+**You should almost always upload to the proposed or beta ppa first** and
+then copy the packages into the release ppa.  That reduces the 
 risk of breaking the main archive from which people get bzr updates.
 
 
@@ -82,7 +90,7 @@
 On reasonably recent versions of Ubuntu you no longer need special dput
 configuration, because you can just say ::
 
-  dput ppa:bzr/proposed <source.changes
+  dput ppa:bzr/proposed source.changes
   
 
 However, you may still want to add these lines to ``~/.dput.cf`` prevent 
@@ -140,8 +148,8 @@
 use the script from <http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=255165> 
 to wrap it, and to give it sensible defaults for your local machine.
 
-Update all packages in proposed before copping the main ppa
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+Update all packages in proposed before copying to the main ppa
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 If one updates bzr, and there are plugins that are not compatible with the
 new version of bzr, this can cause pain for users using the ppa. In order to

=== modified file 'doc/developers/releasing.txt'
--- doc/developers/releasing.txt	2010-12-06 07:48:48 +0000
+++ doc/developers/releasing.txt	2011-01-13 21:51:32 +0000
@@ -163,7 +163,7 @@
    Double check that ./bzr ``_script_version`` matches ``version_info``. Check
    the output of ``bzr --version``.
 
-   For betahttps://code.edge.launchpad.net/~knittl/bzr/fix-inventory-r0/+merge/354985 releases use::
+   For beta releases use::
 
        version_info = (2, 1, 0, 'beta', SERIAL)
 
@@ -251,7 +251,7 @@
    appropriate release branch::
 
      bzr push
-     bzr pqm-submit -m "(mbp) prepare 1.14"
+     bzr pqm-submit -m "(mbp) Release 1.14 (Martin Pool)"
 
    Or with hydrazine::
 
@@ -374,7 +374,7 @@
       feedback.
 
       Bazaar is now available for download from
-      https://launchpad.net/bzr/2.x/2.x/ as a source tarball; packages
+      https://launchpad.net/bzr/x.y/x.y.z/ as a source tarball; packages
       for various systems will be available soon.
 
       <<release notes from this release back to the last major release>>
@@ -392,6 +392,25 @@
    releases. If you do not have a Freshmeat account yet, ask one of the
    existing admins.
 
+   The purpose here is to point users to the latest stable release while still
+   publishing announcements for development releases.
+
+   There are several kinds of modifications that could be done there via the
+   ``Administration`` box in the lower right area of the page:
+
+   * Edit the project: This is where most of the URLs proposed in the
+     ``Links`` box are edited. This should rarely change except for the URLs
+     related to the latest stable release.
+
+   * New announcement: When doing a release (beta, candidates, final), put the
+     summary of the release (you can't embed URLs there, the moderation staff
+     remove them). Users can still access the releases notes via the ``Release
+     Notes`` URL in the ``Links`` box in the upper right area of the page.
+
+   * Set direct download: When releasing a new stable release, this should
+     point to the corresponding launchpad page:
+     <https://launchpad.net/bzr/x.y/x.y.z/>
+
 #. Update `<http://en.wikipedia.org/wiki/Bazaar_(software)>`_ -- this should
    be done for final releases but not for beta releases or Release Candidates.
 

=== modified file 'doc/developers/testing.txt'
--- doc/developers/testing.txt	2010-12-09 00:18:26 +0000
+++ doc/developers/testing.txt	2011-01-03 13:13:50 +0000
@@ -329,8 +329,11 @@
 We make selective use of doctests__.  In general they should provide
 *examples* within the API documentation which can incidentally be tested.  We
 don't try to test every important case using doctests |--| regular Python
-tests are generally a better solution.  That is, we just use doctests to
-make our documentation testable, rather than as a way to make tests.
+tests are generally a better solution.  That is, we just use doctests to make
+our documentation testable, rather than as a way to make tests. Be aware that
+doctests are not as well isolated as the unit tests, if you need more
+isolation, you're likely want to write unit tests anyway if only to get a
+better control of the test environment.
 
 Most of these are in ``bzrlib/doc/api``.  More additions are welcome.
 
@@ -977,6 +980,25 @@
 
     self.overrideAttr(osutils, '_cached_user_encoding', 'latin-1')
 
+Temporarily changing environment variables
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+If yout test needs to temporarily change some environment variable value
+(which generally means you want it restored at the end), you can use::
+
+    self.overrideEnv('BZR_ENV_VAR', 'new_value')
+
+If you want to remove a variable from the environment, you should use the
+special ``None`` value::
+
+    self.overrideEnv('PATH', None)
+
+If you add a new feature which depends on a new environment variable, make
+sure it behaves properly when this variable is not defined (if applicable) and
+if you need to enforce a specific default value, check the
+``TestCase._cleanEnvironment`` in ``bzrlib.tests.__init__.py`` which defines a
+proper set of values for all tests.
+
 Cleaning up
 ~~~~~~~~~~~
 

=== modified file 'doc/en/release-notes/bzr-2.2.txt'
--- doc/en/release-notes/bzr-2.2.txt	2010-12-16 20:41:47 +0000
+++ doc/en/release-notes/bzr-2.2.txt	2010-12-24 17:47:53 +0000
@@ -32,6 +32,13 @@
   because this can cause knock-on errors at awkward times.
   (Andrew Bennetts, #687653)
 
+* ``https`` access works again with recent versions of python2.7.
+  (Vincent Ladeuil, #693880)
+
+* RevisionTree.is_executable no longer returns None for directories and
+  symlinks.  Instead, it returns False, like other Trees and methods.
+  (Aaron Bentley, #681885)
+
 Improvements
 ************
 

=== modified file 'doc/en/release-notes/bzr-2.3.txt'
--- doc/en/release-notes/bzr-2.3.txt	2010-12-16 20:41:47 +0000
+++ doc/en/release-notes/bzr-2.3.txt	2011-01-15 19:43:23 +0000
@@ -5,10 +5,10 @@
 .. toctree::
    :maxdepth: 1
 
-bzr 2.3b5
+bzr 2.3.0
 #########
 
-:2.3.b5: NOT RELEASED YET
+:2.3.0: NOT RELEASED YET
 
 External Compatibility Breaks
 *****************************
@@ -26,6 +26,61 @@
 .. Improvements to existing commands, especially improved performance 
    or memory usage, or better results.
 
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+
+bzr 2.3b5
+#########
+
+:2.3.b5: 2011-01-13
+
+This is the fifth and **last** beta of the 2.3 series, leading up to a 2.3.0
+release in February. Beta releases are suitable for everyday use but may cause
+some incompatibilities with plugins.
+
+2.3b5 includes bug fixes for committing to stacked branches, smoother upgrades
+of multiple branches, compatibility with python-2.7, full test suite passing
+on Ubuntu Natty and windows, less round-trips for several smart server
+operations, better support text conflicts resolve actions and some more.
+
+All known fixed bugs in other series (2.0, 2.1, 2.2) are also included here.
+
+External Compatibility Breaks
+*****************************
+
+(none)
+
+Improvements
+************
+
 * A redundant parent inventories calculation was removed from
   ``fetch.py``, as ``Repository.insert_stream`` already reports any
   missing inventories.  This removes at least one network roundtrip when
@@ -35,32 +90,81 @@
   once.  This avoids some unnecessary IO, and removes a network roundtrip
   when doing ``bzr branch`` to a smart server URL.  (Andrew Bennetts)
 
+* ``bzr modified`` now read-locks the working tree (and branch and
+  repository) just once.  (Andrew Bennetts)
+  
 * ``bzr resolve`` now accepts ``--take-this`` and ``--take-other`` actions
   for text conflicts. This *replace* the whole file with the content
   designated by the action. This will *ignore* all differences that would
   have been merge cleanly otherwise. (Vincent Ladeuil, #638451)
 
+* ``bzr tags``'s "sort" argument now allows registering custom sort
+  methods using the ``bzrlib.tag.tag_sort_methods`` registry. 
+  (Jelmer Vernooij, #701244)
+
 * ``bt.test_http`` was breaking ``os.environ`` by erasing the values saved by
   ``TestCase`` leading to ``bt.test_import_tariff`` failures.
   (Vincent Ladeuil, #690563)
 
+* ``upgrade`` now upgrades dependent branches when a shared repository is
+  specified. It also supports new options: ``--dry-run`` for showing what
+  will happen and ``--clean`` to remove the backup directory on successful
+  completion. (Ian Clatworthy, Matthew Fuller, #89830, #374734, #422450)
+
 Bug Fixes
 *********
 
 .. Fixes for situations where bzr would previously crash or give incorrect
    or undesirable results.
 
+* Avoid leaking SSH subprocess communication socket into unrelated child
+  processes, which could cause bzr to hang on exit. (Max Bowsher, #696285)
+
+* ``bzr break-lock`` on a corrupted lock file works correctly, rather than
+  raising a PermissionDenied error. We were accidentally holding open the
+  file we were trying to delete. (John Arbash Meinel, #659978)
+
+* ``bzr update`` in a checkout of a readonly branch works again, without
+  trying to set the tags in the master branch. This had been broken by the
+  bug fix for bug #603395.  (John Arbash Meinel, #701212)
+
+* Per-transport tests now prefer to use ``Transport.get_bytes()`` rather
+  than ``Transport.get().read()``. The SFTP code uses an async message to
+  close the file handle if you let the handle die from refcounting, while
+  it uses a synchronous message if you close it directly. This should help
+  prevent random test suite failures from race conditions.
+  (John Arbash Meinel, #681047)
+
+* Stop using ``bzrlib.tuned_gzip.GzipFile``. It is incompatible with
+  python-2.7 and was only used for Knit format repositories, which haven't
+  been recommended since 2007. The file itself will be removed in the next
+  release. (John Arbash Meinel)
+
+* The BZR_COLUMNS environment variable can be set to 0 to indicate no
+  limitation on the width of the terminal.  (Neil Martinsen-Burrell, #675652)
+
+* Treat WSAECONNABORTED the same as WSAECONNRESET for the purposes of
+  considering a smart data stream as being interrupted. This fixes a
+  failure in the windows test suite, that was trying to ensure we cleanly
+  handled a server disconnect.  (John Arbash Meinel, #581311, #686587)
+
+* Unshelving changes that occur in a now-unversioned directory now restore
+  the directory properly rather than crashing.
+  (John Arbash Meinel, #389674)
+
+* You are now able to commit directly to a stacked branch. Any needed
+  parent inventories will be filled in as part of the commit process.
+  (John Arbash Meinel, #375013)
+
 Documentation
 *************
 
-.. Improved or updated documentation.
+* Better document the rules to update the bzr freshmeat page when
+  doing a release. (Vincent Ladeuil, #690515)
 
 API Changes
 ***********
 
-.. Changes that may require updates in plugins or other code that uses
-   bzrlib.
-
 * ``Branch.sprout``, ``BranchFormat.initalize`` and
   ``ControlDir.create_branch`` now take an optional ``repository`` keyword
   argument, and ``BranchFormat.open`` now takes an optional
@@ -69,18 +173,20 @@
   already opened that repository).  Implementations of these APIs will
   need to be updated to accept these arguments.  (Andrew Bennetts)
 
-Internals
-*********
+* ``bzrlib.tuned_gzip.GzipFile`` is now deprecated and will be removed in
+  the bzr-2.4 series. Code that was using it can just use the python
+  stdlib ``gzip.GzipFile``. (John Arbash Meinel)
 
-.. Major internal changes, unlikely to be visible to users or plugin 
-   developers, but interesting for bzr developers.
 
 Testing
 *******
 
-.. Fixes and changes that are only relevant to bzr's test framework and 
-   suite.  This can include new facilities for writing tests, fixes to 
-   spurious test failures and changes to the way things should be tested.
+* ``bzrlib.tests`` defines ``isolated_environ`` with the definitions of all
+  the environment variables the tests should care about. It also defines
+  ``override_os_environ`` and ``restore_os_environ`` to properly implement
+  isolation from ``os.environ`` for tests. ``bzrlib.tests`` now defines a
+  ``DocTestSuite`` class using this facility for all ``bzrlib``
+  doctests. (Vincent Ladeuil, #321320)
 
 * Catch exceptions related to bug #637821 during test cleanup to avoid
   spurious failures. (Vincent Ladeuil, #686008).
@@ -88,8 +194,15 @@
 * Check sphinx compatibility for tests requiring older sphinx versions.
   (Vincent Ladeuil, #688072)
 
+* ``test_onto_transport`` in the Launchpad plugin can now run with Python
+  2.7. (Vincent Ladeuil, #654733)
+
+* ``TestCase._captureVar`` and ``TestCase._old_env`` have been deleted due to
+  bug #690563. Test writers are encouraged to use ``TestCase.overrideEnv``
+  instead. (Vincent Ladeuil)
+
 * ``TestDebuntuExpansions`` was escaping the test isolation by calling the
-  wrong base class ``setUp``. (Vincent Ladeuil, #684622)
+  wrong base class ``setUp``. (Vincent Ladeuil, #684662)
 
 bzr 2.3b4
 #########
@@ -162,6 +275,11 @@
   crashes when encountering private bugs (they are just displayed as such).
   (Vincent Ladeuil, #354985)
 
+* The ``revision_id`` parameter of
+  ``Repository.search_missing_revision_ids`` and
+  ``InterRepository.search_missing_revision_ids`` is deprecated.  It is
+  replaced by the ``revision_ids`` parameter.  (Andrew Bennetts)
+
 Internals
 *********
 
@@ -189,6 +307,11 @@
   to 0.9.5 which will allow tests that need the fixed unicode handling to be
   written. (Martin [gz])
 
+* Introduce an ``overrideEnv()`` helper for tests that needs to change the
+  environment variables while respecting the isolation rules. Get rid of
+  TestCase._restoreEnvironment which is now useless.
+  (Vincent Ladeuil, #690563)
+
 * Printing selftest results to a non-UTF-8 console will now escape characters
   that can't be encoded rather than aborting the test run with an exception.
   (Martin [gz], #633216)

=== added file 'doc/en/release-notes/bzr-2.4.txt'
--- doc/en/release-notes/bzr-2.4.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-2.4.txt	2011-01-19 18:18:48 +0000
@@ -0,0 +1,80 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 2.4b1
+#########
+
+:2.4b1: NOT RELEASED YET
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+* The ``lp:`` directory service now supports Launchpad's QA staging.
+  (Jelmer Vernooij, #667483)
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+* ``bzr cat-revision`` no longer requires a working tree. (Jelmer Vernooij, #704405)
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* ``bzr push --overwrite`` with an older revision specified will now correctly
+  roll back the target branch. (Jelmer Vernooij, #386576)
+
+* ``bzr serve`` no longer crashes when a server_started hook is installed and IPv6
+  support is available on the system. (Jelmer Vernooij, #293697)
+
+* ``bzr lp-propose`` can now propose merges against packaging branches on
+  Launhcpad without requiring the target branch to be specified. (Jelmer Vernooij,
+  #704647)
+
+* ``bzr lp-propose`` no longer requires a reviewer to be specified. It will instead
+  leave setting the reviewer up to Launchpad if it was not specified.
+  (Jelmer Vernooij, #583772)
+
+* ``bzr whoami`` will now display an error if both a new identity and ``--email``
+  were specified. (Jelmer Vernooij, #680449)
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+

=== modified file 'doc/en/upgrade-guide/data_migration.txt'
--- doc/en/upgrade-guide/data_migration.txt	2010-11-12 22:43:38 +0000
+++ doc/en/upgrade-guide/data_migration.txt	2010-12-20 12:01:56 +0000
@@ -202,11 +202,15 @@
    name then **merge** your changes from the matching branch in the
    old repository.
 
-The first method will give you a branch which is identical (in terms
-of revision history) to the old branch, but it's parent branch will
-be set to the old branch, not your new trunk. If you use this method,
-you'll probably want to edit your branch.conf file to update the
-parent branch setting.
+The first method will give you a branch which is identical (in terms of
+revision history) to the old branch, but it's parent branch will be set to the
+old branch, not your new trunk. If you use this method, you'll probably update
+the ``parent_location`` configuration variable in the ``branch.conf`` file
+with::
+
+    bzr config parent_location=XXX
+
+``XXX`` being the URL to your new trunk.
 
 In contrast, the second approach sets up the parent branch correctly.
 However, it isn't ideal if you're not ready to include all the latest

=== modified file 'doc/en/upgrade-guide/index.txt'
--- doc/en/upgrade-guide/index.txt	2009-09-02 16:03:51 +0000
+++ doc/en/upgrade-guide/index.txt	2010-12-17 10:16:48 +0000
@@ -1,6 +1,6 @@
-########################
-Bazaar 2.0 Upgrade Guide
-########################
+####################
+Bazaar Upgrade Guide
+####################
 
 .. Please mark sections in included files as following:
 ..   level 1 ========

=== modified file 'doc/en/upgrade-guide/overview.txt'
--- doc/en/upgrade-guide/overview.txt	2010-08-13 19:08:57 +0000
+++ doc/en/upgrade-guide/overview.txt	2010-12-17 10:16:48 +0000
@@ -4,7 +4,7 @@
 High level upgrade process
 --------------------------
 
-In broad terms, there are 3 steps involved in upgrading to Bazaar 2.x:
+In broad terms, there are 3 steps involved in upgrading a new format:
 
 1. Upgrade the core software
 
@@ -12,34 +12,29 @@
 
 3. Migrate data to the new default format.
 
-Bazaar 2.x supports branches in earlier formats so the third step is
-strictly not required. However, the new default format in Bazaar 2.x
-is more space efficient, faster on large projects and provides a range
-of new features, so it is recommended that most projects migrate to it
-at a convenient time.
+Bazaar supports branches in earlier formats so the third step is strictly not
+required. However, when new default formats are introduced, they are more
+space efficient, faster on large projects and/or provide new features. So it
+is recommended that most projects migrate to it at a convenient time.
 
-For most users, upgrading to 2.x and migrating to the new format is
-straight forward. For projects with a large community of developers
-though, things become more complex. In these cases, careful planning
-and good communications become essential. This document provides
-general advice which aims to assist in this regard. If in doubt,
-please contact us on our mailing list or IRC channel with any
-questions or concerns you have.
+For most users, upgrading and migrating to the new format is straight
+forward. For projects with a large community of developers though, things
+become more complex. In these cases, careful planning and good communications
+become essential. This document provides general advice which aims to assist
+in this regard. If in doubt, please contact us on our mailing list or IRC
+channel with any questions or concerns you have.
 
 
 Upgrading the core software
 ---------------------------
 
-The steps required to upgrade the core software vary from operating
-system to operating system. There is nothing special about upgrading
-from Bazaar 1.x to Bazaar 2.0 compared to upgrading from Bazaar 1.x
-to Bazaar 1.y. In either case, a brief outline of the steps is given
-below.
+The steps required to upgrade the core software vary from operating system to
+operating system.  A brief outline of the steps is given below.
 
 To upgrade Bazaar on Ubuntu:
 
-1. Ensure your package manager is configured with the required
-   software sources, e.g. the official release PPA for Ubuntu:
+1. Ensure your package manager is configured with the required software
+   sources, e.g. the official stable release PPA for Ubuntu:
    https://launchpad.net/~bzr/+archive
 
 2. Use your package manager to upgrade to the latest version.

=== modified file 'doc/en/whats-new/whats-new-in-2.3.txt'
--- doc/en/whats-new/whats-new-in-2.3.txt	2010-12-14 23:32:28 +0000
+++ doc/en/whats-new/whats-new-in-2.3.txt	2011-01-12 16:45:49 +0000
@@ -20,6 +20,11 @@
 Changed Behaviour
 *****************
 
+* Committing a new revision in a stacked branch is now supported, as long as
+  you are using the current repository format (2a). It will preserve the
+  stacking invariants, etc, so that fetching after commit is guaranteed to
+  work. (John Arbash Meinel, #375013)
+
 * Support for some old development formats have been removed:
   ``development-rich-root``, ``development6-rich-root``, and
   ``development7-rich-root``.  These formats were always labelled experimental
@@ -45,6 +50,11 @@
   ~/.bazaar/plugins.  To use a different directory for plugins, use the
   environment variable BZR_PLUGIN_PATH.  (Neil Martinsen-Burrell, #195397)
 
+* ``bzr upgrade`` now operates recursively when run on a shared
+  repository, automatically upgrading the branches within it, and has
+  grown additional options for showing what it will do and cleaning up
+  after itself.  (Ian Clatworthy, Matthew Fuller, #89830, #374734, #422450)
+
 Launchpad integration
 *********************
 

=== added file 'doc/en/whats-new/whats-new-in-2.4.txt'
--- doc/en/whats-new/whats-new-in-2.4.txt	1970-01-01 00:00:00 +0000
+++ doc/en/whats-new/whats-new-in-2.4.txt	2011-01-14 05:34:20 +0000
@@ -0,0 +1,33 @@
+*************************
+What's New in Bazaar 2.4?
+*************************
+
+Bazaar 2.4 is still under development, and will be released in August 2011.
+This document accumulates a high level summary of what's changed.  See the
+:doc:`../release-notes/index` for a full list.
+
+Users are encouraged to upgrade from the other stable series.  This
+document outlines the improvements in Bazaar 2.4 vs Bazaar 2.3. As well as
+summarizing improvements made to the core product, it highlights
+enhancements within the broader Bazaar world of potential interest to
+those upgrading.
+
+Bazaar 2.4.0 is fully compatible both locally and on the network with 2.0
+2.1, 2.2 and 2.3, and can read and write repositories generated by all
+previous versions.
+
+
+Further information
+*******************
+
+For more detailed information on the changes made, see the the
+:doc:`../release-notes/index` for:
+
+* the interim bzr `milestones <https://launchpad.net/bzr/2.4>`_
+* the plugins you use.
+
+For a summary of changes made in earlier releases, see:
+
+* :doc:`whats-new-in-2.1`
+* :doc:`whats-new-in-2.2`
+* :doc:`whats-new-in-2.3`

=== modified file 'setup.py'
--- setup.py	2010-12-02 10:41:05 +0000
+++ setup.py	2010-12-26 13:19:11 +0000
@@ -395,7 +395,8 @@
     # ditto for the tbzrcommand tool
     tbzrcommand = dict(
         script = os.path.join(tbzr_root, "scripts", "tbzrcommand.py"),
-        icon_resources = [(0,'bzr.ico')],
+        icon_resources = icon_resources,
+        other_resources = other_resources,
     )
     console_targets.append(tbzrcommand)
     tbzrcommandw = tbzrcommand.copy()

=== modified file 'tools/packaging/build-packages.sh'
--- tools/packaging/build-packages.sh	2009-05-25 10:59:42 +0000
+++ tools/packaging/build-packages.sh	2011-01-11 11:26:48 +0000
@@ -1,5 +1,5 @@
 #!/bin/bash
-# A helper script to build packages for the various distributions
+# Helper to build packages for the various distributions
 
 if [ -z "$UBUNTU_RELEASES" ]; then
     echo "Configure the distro platforms that you want to"

=== modified file 'tools/packaging/update-changelogs.sh'
--- tools/packaging/update-changelogs.sh	2010-08-19 08:07:42 +0000
+++ tools/packaging/update-changelogs.sh	2011-01-11 11:26:48 +0000
@@ -1,4 +1,5 @@
 #!/bin/bash
+# Helper to insert a new entry in debian/changelog
 
 if [ -z "$UBUNTU_RELEASES" ]; then
     echo "Configure the distro platforms that you want to"

=== modified file 'tools/packaging/update-control.sh'
--- tools/packaging/update-control.sh	2009-09-02 04:08:08 +0000
+++ tools/packaging/update-control.sh	2011-01-11 11:26:48 +0000
@@ -1,4 +1,5 @@
 #!/bin/bash
+# Helper to run sed on versions in dependencies in debian/control
 
 if [ -z "$UBUNTU_RELEASES" ]; then
     echo "Configure the distro platforms that you want to"

